
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

// --- definitions ---

/*
 * definice konstant oznacujicich specialni znaky v regularnich vyrazech
 */

/*
 * pocet zakladnich znaku pri rozpoznavani lexemu je 257
 * 256ty znak popisuje jakoukoli escape sekvenci, vyuziva se jen pri rozpoznavani (final_automata_s::recognize)
 * znaky c_base_char_cnt az c_base_char_cnt + c_def_special_char_cnt, popisuji "zadratovane" mnoziny znaku
 * znaky c_no_char_base az c_no_char_base + 255 - popisuji mnoziny znaku s absenci jednoho definovaneho znaku
 */
const unsigned c_base_char_cnt = 257;
const unsigned c_def_special_char_cnt = 3;
const unsigned c_no_char_base = c_base_char_cnt + c_def_special_char_cnt;

const unsigned short c_escape_char = 256;

const unsigned short reg_char_left_bracket =	   c_no_char_base + 256;      // '['
const unsigned short reg_char_right_bracket =	   c_no_char_base + 256 + 1;  // ']'
const unsigned short reg_char_star_iteration =	   c_no_char_base + 256 + 2;  // '*'

const unsigned short reg_char_white =		   c_base_char_cnt;	   // 'w' {' ','\t','\n'}
const unsigned short reg_char_digit =		   c_base_char_cnt + 1;    // 'd' {'0' - '9'}
const unsigned short reg_char_letter =		   c_base_char_cnt + 2;    // 'l' {'a' - 'z','A' - 'Z'}

//****************************** REGULAR ********************************************

/*
 * identifikace operaci prirazenych k redukcim
 */

enum {
   c_reduce_null_0 = 0,
   c_reduce_alter,
   c_reduce_null_1,
   c_reduce_conc,
   c_reduce_null_2,
   c_reduce_iter,
   c_reduce_char_eps,
   c_reduce_char,
   c_reduce_char_white,
   c_reduce_char_digit,
   c_reduce_char_letter,
   c_reduce_char_no_char,
   c_reduce_char_str,
   c_reduce_char_range,
   c_reduce_char_alter,
   c_reduce_char_except,
   c_reduce_null_3
};

// rozkladove konstanty
const unsigned c_rule_cnt = 17;
const unsigned rule_head_idxs[c_rule_cnt] = {17, 18, 18, 19, 19, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, };
const unsigned rule_body_lengths[c_rule_cnt] = {2, 3, 1, 3, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, };

//rozkladova tabulka
#define blank c_idx_not_exist
#define SHIFT(VALUE) VALUE
#define REDUCE(VALUE) c_reg_lalr_table_reduce_base + VALUE
#define GOTO(VALUE) VALUE

const unsigned c_reg_lalr_table_reduce_base = 0x80000000;
const unsigned c_terminal_plus_nonterminal_cnt = 21;
const unsigned lalr_state_cnt = 23;

const unsigned lalr_table[lalr_state_cnt*c_terminal_plus_nonterminal_cnt] = {
    SHIFT(4),    SHIFT(5),    SHIFT(6),    SHIFT(7),    SHIFT(8),    SHIFT(9),   SHIFT(10),   SHIFT(11),   SHIFT(12),   SHIFT(13),   SHIFT(14),       blank,       blank,       blank,       blank,       blank,       blank,       blank,     GOTO(1),     GOTO(2),     GOTO(3),
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   SHIFT(15),       blank,   SHIFT(16),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   REDUCE(2),       blank,   SHIFT(17),   REDUCE(2),       blank,   REDUCE(2),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   REDUCE(4),   SHIFT(18),   REDUCE(4),   REDUCE(4),       blank,   REDUCE(4),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   REDUCE(6),   REDUCE(6),   REDUCE(6),   REDUCE(6),       blank,   REDUCE(6),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   REDUCE(7),   REDUCE(7),   REDUCE(7),   REDUCE(7),       blank,   REDUCE(7),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   REDUCE(8),   REDUCE(8),   REDUCE(8),   REDUCE(8),       blank,   REDUCE(8),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   REDUCE(9),   REDUCE(9),   REDUCE(9),   REDUCE(9),       blank,   REDUCE(9),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,  REDUCE(10),  REDUCE(10),  REDUCE(10),  REDUCE(10),       blank,  REDUCE(10),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,  REDUCE(11),  REDUCE(11),  REDUCE(11),  REDUCE(11),       blank,  REDUCE(11),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,  REDUCE(12),  REDUCE(12),  REDUCE(12),  REDUCE(12),       blank,  REDUCE(12),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,  REDUCE(13),  REDUCE(13),  REDUCE(13),  REDUCE(13),       blank,  REDUCE(13),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,  REDUCE(14),  REDUCE(14),  REDUCE(14),  REDUCE(14),       blank,  REDUCE(14),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,  REDUCE(15),  REDUCE(15),  REDUCE(15),  REDUCE(15),       blank,  REDUCE(15),       blank,       blank,       blank,       blank,
    SHIFT(4),    SHIFT(5),    SHIFT(6),    SHIFT(7),    SHIFT(8),    SHIFT(9),   SHIFT(10),   SHIFT(11),   SHIFT(12),   SHIFT(13),   SHIFT(14),       blank,       blank,       blank,       blank,       blank,       blank,       blank,    GOTO(19),     GOTO(2),     GOTO(3),
    SHIFT(4),    SHIFT(5),    SHIFT(6),    SHIFT(7),    SHIFT(8),    SHIFT(9),   SHIFT(10),   SHIFT(11),   SHIFT(12),   SHIFT(13),   SHIFT(14),       blank,       blank,       blank,       blank,       blank,       blank,       blank,    GOTO(20),     GOTO(2),     GOTO(3),
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,
    SHIFT(4),    SHIFT(5),    SHIFT(6),    SHIFT(7),    SHIFT(8),    SHIFT(9),   SHIFT(10),   SHIFT(11),   SHIFT(12),   SHIFT(13),   SHIFT(14),       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,    GOTO(21),     GOTO(3),
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   REDUCE(5),   REDUCE(5),   REDUCE(5),   REDUCE(5),       blank,   REDUCE(5),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   SHIFT(22),       blank,       blank,   SHIFT(15),       blank,       blank,       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   REDUCE(1),       blank,       blank,   SHIFT(15),       blank,   REDUCE(1),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,   REDUCE(3),       blank,   SHIFT(17),   REDUCE(3),       blank,   REDUCE(3),       blank,       blank,       blank,       blank,
       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,       blank,  REDUCE(16),  REDUCE(16),  REDUCE(16),  REDUCE(16),       blank,  REDUCE(16),       blank,       blank,       blank,       blank,
};

//***********************************************************************************

/*
 * globalni funkce
 */

int compare_unsigned_shorts(const void *first,const void *second)
{
   if (*(unsigned short *)first < *(unsigned short *)second) {
      return -1;
   }

   if (*(unsigned short *)first > *(unsigned short *)second) {
      return 1;
   }

   return 0;
}

// -- funkce struktury ui_array_s --

bool ui_array_s::set_compare(ui_array_s &second)
{
   if (used != second.used) return false;
   if (used == 0) return true;

   unsigned *ptr = data;
   unsigned *ptr_end = ptr + used;

   do {
      unsigned *s_ptr = second.data;
      unsigned *s_ptr_end = s_ptr + second.used;

      do {
         if (*s_ptr == *ptr) {
            break;
         }
      } while(++s_ptr < s_ptr_end);

      if (s_ptr >= s_ptr_end) {
         return false;
      }

   } while(++ptr < ptr_end);

   return true;
}

/* 
 * prida prvky mnoziny do dalsi mnoziny
 */
void p_follow_set_s::add_to_set(p_follow_set_s &add_set)
{
   if (add_set.used == 0) return;

   unsigned *a_ptr = add_set.data;
   unsigned *a_ptr_end = a_ptr + add_set.used;

   do {
      bool exist = false;

      if (used != 0) {
	 unsigned *ptr = data;
	 unsigned *ptr_end = ptr + used;

	 do {
	    if (*ptr == *a_ptr) {
	       exist = true;
	       break;
	    }
	 } while(++ptr < ptr_end);
      }

      if (!exist) {
	 push(*a_ptr);
      }
   } while(++a_ptr < a_ptr_end);
}

// -- funkce struktury reg_mul_state_move_array_s --

unsigned reg_mul_state_move_array_s::get_idx_by_move_char(unsigned short a_move_char)
{
   if (used == 0) return c_idx_not_exist;

   reg_mul_state_move_s *ptr = data;
   reg_mul_state_move_s *ptr_end = ptr + used;

   do {
      if (ptr->move_char == a_move_char) {
         return ptr - data;
      }
   } while(++ptr < ptr_end);

   return c_idx_not_exist;
}

// -- funkce struktury reg_mul_state_map_array_s --

unsigned reg_mul_state_map_array_s::get_idx_by_mul_state(ui_array_s &a_mul_state)
{
   if (used == 0) return c_idx_not_exist;

   reg_mul_state_map_s *ptr = data;
   reg_mul_state_map_s *ptr_end = ptr + used;

   do {
      if (ptr->mul_state.set_compare(a_mul_state)) {
         return ptr - data;
      }
   } while(++ptr < ptr_end);

   return c_idx_not_exist;
}

// -- funkce struktury ushort_reg_exp_s --

/*
 * zpracuje znak v regularnim vyrazu, tj. zpracuje escape sekvence a posloupnosti oznacujici specialni znaky
 */
inline void ushort_reg_exp_s::process_reg_exp_char(char **c_ptr)
{
   if (**c_ptr != '\\') {
      push((unsigned short)**c_ptr);
   }
   else {
      switch(*(++(*c_ptr))) {

      // - formatovani vstupnich znaku -
      case 't':
	 push((unsigned short)'\t');
	 break;
      case 'n':
	 push((unsigned short)'\n');
	 break;
      case '0':
	 push((unsigned short)'\0');
	 break;
      case '\\':
	 push((unsigned short)'\\');
	 break;
      
      // - vytvoreni specialnich regularnich znaku -
      case '[':
	 push(reg_char_left_bracket);
	 break;
      case ']':
	 push(reg_char_right_bracket);
	 break;
      case '*':
	 push(reg_char_star_iteration);
	 break;
      case 'w':
	 push(reg_char_white);
	 break;
      case 'd':
	 push(reg_char_digit);
	 break;
      case 'l':
	 push(reg_char_letter);
	 break;

      // - spracovani tzv. no_char znaku (tj. mnozina znaku s absenci jednoho definovaneho) -
      case '!':
	 if (*(++(*c_ptr)) != '\\') {
	    push(c_no_char_base + **c_ptr);
	 }
	 else {
	    switch(*(++(*c_ptr))) {
	    case 't':
	       push(c_no_char_base + '\t');
	       break;
	    case 'n':
	       push(c_no_char_base + '\n');
	       break;
	    case '0':
	       push(c_no_char_base + '\0');
	       break;
	    case '\\':
	       push(c_no_char_base + '\\');
	       break;
	    default:
	       assert(0);
	    }
	 }
	 break;
      default:
	 //FIXME je to v poradku?
	 push((unsigned short)**c_ptr);
      }
   }
}

/*
 * prevede retezec popisujici regularni vyraz na posloupnost short integeru a zaroven zpracuje
 * escape sekvence a specialni znaky
 */
void ushort_reg_exp_s::load_ascii_reg_exp(ascii_reg_exp_s &a_reg_exp)
{
   assert(a_reg_exp.used != 0);

   clear();
   
   char *ptr = a_reg_exp.data;
   char *ptr_end = ptr + a_reg_exp.used;

   do {
      process_reg_exp_char(&ptr);
   } while(++ptr < ptr_end);

   flush();
}

/*
 * prevede retezec popisujici regularni vyraz na posloupnost short integeru a zaroven zpracuje
 * escape sekvence a specialni znaky
 */
void ushort_reg_exp_s::load_char_ptr_reg_exp(char *reg_exp)
{
   assert(reg_exp != NULL);

   clear();

   char *ptr = reg_exp;

   do {
      if (*ptr == '\0') {
	 break;
      }
      else {
	 process_reg_exp_char(&ptr);
      }
   } while(++ptr,1);

   flush();
}

/*
 * nalezne znaky ktere prijima regularni vyraz v aktualni pozici, a umisteni v regularnim vyrazu
 * po prechodu s timto znakem
 */
bool ushort_reg_exp_s::next_chars(unsigned reg_pos,fa_state_moves_s &ret_moves,ui_array_s &stack)
{
   ret_moves.used = 0;
   stack.used = 0;

   bool in_brackets = false;
   bool confirm = false;

   unsigned pos = reg_pos;
   do {

      // - hledani nasledujiciho znaku -
      if (!confirm) {

	 unsigned short next_char;
	 if (pos < used) {
	    next_char = data[pos];
	 }
	 else {
	    return true;
	 }

	 //pocita s tim ze next_char nikdy nebude mit hodnotu escape_char
	 if (next_char < c_no_char_base + 256) {

	    ret_moves.push_blank();
	    ret_moves.last().idx = next_char;
	    stack.push(ret_moves.used - 1);

	    if (!in_brackets) {
	       confirm = true;
	    }
	 }
	 else if (next_char == reg_char_left_bracket) {
	    in_brackets = true;
	 }
	 else if (next_char == reg_char_right_bracket) {
	    in_brackets = false;
	    confirm = true;
	 }
	 else {
	    assert(0);
	 }
      }

      // - potvrzeni prijmuti znaku -
      else {
	 unsigned short next_char = (pos < used)?data[pos]:(unsigned short)'\0';

	 if (next_char < c_no_char_base + 256 ||
	    next_char == reg_char_left_bracket) {

	    while (stack.used > 0) {
	       ret_moves[stack.pop()].value = pos;
	    }

	    return false;
	 }
	 else if (next_char == reg_char_star_iteration) {
	    unsigned move_pos = pos - 1;

	    if (data[move_pos] == reg_char_right_bracket) {
	       while(data[--move_pos] != reg_char_left_bracket) {};
	    }

	    while (stack.used > 0) {
	       ret_moves[stack.pop()].value = move_pos;
	    }

	    confirm = false;
	 }
	 else {
	    assert(0);
	 }
      }
   } while(++pos,1);

   return false;
}

// -- funkce struktury ushort_reg_exps_s --

/*
 * nacte mnozinu regularnich vyrazu v znakove reprezentaci do mnoziny posloupnosti short integeru
 * a zaroven zpracuje jejich format a escape znaky
 */
void ushort_reg_exps_s::load_ascii_reg_exps(ascii_reg_exps_s &a_reg_exps)
{
   assert(a_reg_exps.used != 0);

   clear();
   copy_resize(a_reg_exps.used);

   ushort_reg_exp_s *ptr = data;
   ushort_reg_exp_s *ptr_end = ptr + used;
   ascii_reg_exp_s *a_ptr = a_reg_exps.data;

   do {
      ptr->load_ascii_reg_exp(*a_ptr);
   } while(++a_ptr,++ptr < ptr_end);

   used = a_reg_exps.used;
}

/*
 * nacte mnozinu regularnich vyrazu v znakove reprezentaci do mnoziny posloupnosti short integeru
 * a zaroven zpracuje jejich format a escape znaky
 */
void ushort_reg_exps_s::load_char_ptrs_reg_exps(unsigned reg_exp_cnt,char **reg_exps)
{
   assert(reg_exp_cnt != 0);

   clear();
   copy_resize(reg_exp_cnt);

   ushort_reg_exp_s *ptr = data;
   ushort_reg_exp_s *ptr_end = ptr + reg_exp_cnt;
   char **r_ptr = reg_exps;

   do {
      ptr->load_char_ptr_reg_exp(*r_ptr);
   } while(++r_ptr,++ptr < ptr_end);

   used = reg_exp_cnt;
}

// -- funkce struktury reg_lalr_stack_s --

inline void reg_lalr_stack_s::push(unsigned a_ui_first)
{
   if (used >= size) {
      copy_resize(size + c_array_add);
   }

   reg_lalr_stack_element_s &target = data[used++];

   target.ui_first = a_ui_first;
}

inline void reg_lalr_stack_s::push(unsigned a_ui_first,unsigned a_ui_second,unsigned a_ui_third)
{
   if (used >= size) {
      copy_resize(size + c_array_add);
   }

   reg_lalr_stack_element_s &target = data[used++];

   target.ui_first = a_ui_first;
   target.ui_second = a_ui_second;
   target.ui_third = a_ui_third;
}

//**************************** REGULAR **************************************

// -- funkce struktury reg_parser_s --

inline unsigned short reg_parser_s::process_char(char **c_ptr)
{
   if (**c_ptr == '\\') {
      switch (*(++(*c_ptr))) {
      case '\'':
	 return '\'';
      case '\"':
	 return'\"';
      case 't':
	 return '\t';
      case 'n':
	 return '\n';
      case '0':
	 return '\0';
      case '\\':
	 return '\\';
      case '}':
	 return '}';
      default:
	 assert(0);
	 return 0xffff;
      }
   }
   else {
      return **c_ptr;
   }
}

unsigned reg_parser_s::recognize_terminal(string_s &source_string,unsigned &input_idx,unsigned &line_cnt)
{

   unsigned source_string_length = source_string.size - 1;

#define GET_NEXT_CHAR() \
{\
   if (input_idx < source_string_length) {\
      in_char = (unsigned char)source_string[input_idx];\
\
      if (in_char == '\\') {\
         assert(input_idx + 1 < source_string_length);\
         in_char = c_escape_char;\
      }\
   }\
   else {\
      in_char = '\0';\
   }\
}

#define CLOSE_CHAR(RET_TERM_IDX) \
{\
   if (in_char == '\0') {\
      return RET_TERM_IDX;\
   }\
\
   if (in_char == '\n') {\
      line_cnt++;\
   }\
\
   if (in_char == c_escape_char) {\
      input_idx += 2;\
   }\
   else {\
      input_idx++;\
   }\
}

   unsigned short in_char;

// - STAV 0 - 
   GET_NEXT_CHAR();

   if (in_char == 0)
      goto state_1_label;

   if (in_char == '!')
      goto state_2_label;

   if (in_char == '"')
      goto state_3_label;

   if (in_char == '\'')
      goto state_4_label;

   if (in_char == '(')
      goto state_5_label;

   if (in_char == ')')
      goto state_6_label;

   if (in_char == '*')
      goto state_7_label;

   if (in_char == '+')
      goto state_8_label;

   if (in_char == '.')
      goto state_9_label;

   if (in_char == '<')
      goto state_10_label;

   if (in_char == '[')
      goto state_11_label;

   if (in_char == 'd')
      goto state_12_label;

   if (in_char == 'e')
      goto state_13_label;

   if (in_char == 'l')
      goto state_14_label;

   if (in_char == 'w')
      goto state_15_label;

   if (in_char == '|')
      goto state_16_label;

   if (in_char == ' ' || in_char == '\t' || in_char == '\n')
      goto state_17_label;

   return c_idx_not_exist;

// - STAV 1 - 
state_1_label:
   CLOSE_CHAR(16)
   return 16;

// - STAV 2 - 
state_2_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char == '\'')
      goto state_18_label;

   return c_idx_not_exist;

// - STAV 3 - 
state_3_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char == '"')
      goto state_19_label;

   if (in_char != 34)
      goto state_3_label;

   return c_idx_not_exist;

// - STAV 4 - 
state_4_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char != 39)
      goto state_20_label;

   return c_idx_not_exist;

// - STAV 5 - 
state_5_label:
   CLOSE_CHAR(10)
   return 10;

// - STAV 6 - 
state_6_label:
   CLOSE_CHAR(11)
   return 11;

// - STAV 7 - 
state_7_label:
   CLOSE_CHAR(12)
   return 12;

// - STAV 8 - 
state_8_label:
   CLOSE_CHAR(14)
   return 14;

// - STAV 9 - 
state_9_label:
   CLOSE_CHAR(13)
   return 13;

// - STAV 10 - 
state_10_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char != 62)
      goto state_21_label;

   return c_idx_not_exist;

// - STAV 11 - 
state_11_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char != 93)
      goto state_22_label;

   return c_idx_not_exist;

// - STAV 12 - 
state_12_label:
   CLOSE_CHAR(3)
   return 3;

// - STAV 13 - 
state_13_label:
   CLOSE_CHAR(0)
   return 0;

// - STAV 14 - 
state_14_label:
   CLOSE_CHAR(4)
   return 4;

// - STAV 15 - 
state_15_label:
   CLOSE_CHAR(2)
   return 2;

// - STAV 16 - 
state_16_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char != 124)
      goto state_23_label;

   return c_idx_not_exist;

// - STAV 17 - 
state_17_label:
   CLOSE_CHAR(15)
   GET_NEXT_CHAR();

   if (in_char == ' ' || in_char == '\t' || in_char == '\n')
      goto state_17_label;

   return 15;

// - STAV 18 - 
state_18_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char != 39)
      goto state_24_label;

   return c_idx_not_exist;

// - STAV 19 - 
state_19_label:
   CLOSE_CHAR(6)
   return 6;

// - STAV 20 - 
state_20_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char == '\'')
      goto state_25_label;

   return c_idx_not_exist;

// - STAV 21 - 
state_21_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char != 62)
      goto state_26_label;

   return c_idx_not_exist;

// - STAV 22 - 
state_22_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char == ']')
      goto state_27_label;

   if (in_char != 93)
      goto state_22_label;

   return c_idx_not_exist;

// - STAV 23 - 
state_23_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char == '|')
      goto state_28_label;

   if (in_char != 124)
      goto state_23_label;

   return c_idx_not_exist;

// - STAV 24 - 
state_24_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char == '\'')
      goto state_29_label;

   return c_idx_not_exist;

// - STAV 25 - 
state_25_label:
   CLOSE_CHAR(1)
   return 1;

// - STAV 26 - 
state_26_label:
   CLOSE_CHAR(c_idx_not_exist)
   GET_NEXT_CHAR();

   if (in_char == '>')
      goto state_30_label;

   return c_idx_not_exist;

// - STAV 27 - 
state_27_label:
   CLOSE_CHAR(8)
   return 8;

// - STAV 28 - 
state_28_label:
   CLOSE_CHAR(9)
   return 9;

// - STAV 29 - 
state_29_label:
   CLOSE_CHAR(5)
   return 5;

// - STAV 30 - 
state_30_label:
   CLOSE_CHAR(7)
   return 7;

}

void reg_parser_s::parse_reg_exp(string_s &source_string)
{
   lalr_stack.used = 0;
   lalr_stack.push(0);

   fas_idx = 0;
   states.clear();

   // - pole obsahujici zacatky a konce jednotlivych casti konecneho automatu -
   ui_array_s begin_idxs;
   begin_idxs.init();

   ui_array_s end_idxs;
   end_idxs.init();

   // - promenne pouzivane pri rozkladu regularniho vyrazu -
   unsigned old_input_idx = 0;
   unsigned input_idx = 0;
   unsigned line_cnt = 1;
   unsigned ret_term = c_idx_not_exist;

   do {

      while (ret_term == c_idx_not_exist) {
         old_input_idx = input_idx;

         ret_term = recognize_terminal(source_string,input_idx,line_cnt);
         assert(ret_term != c_idx_not_exist);

         if (ret_term == 15) {
            ret_term = c_idx_not_exist;
         }
      }

      // - nalezeni akce v tabulce akci -
      unsigned parse_action = lalr_table[lalr_stack.last().ui_first*c_terminal_plus_nonterminal_cnt + ret_term];
      assert(parse_action != c_idx_not_exist);

      // - akce SHIFT -
      if (parse_action < c_reg_lalr_table_reduce_base) {

         if (ret_term == 16) {
            break;
         }

         // - vlozi na zasobnik novy stav a pozici terminalu ve zdrojovem retezci -
         lalr_stack.push(parse_action,old_input_idx,input_idx);
         ret_term = c_idx_not_exist;
      }

      // -akce REDUCE-
      else {
         parse_action -= c_reg_lalr_table_reduce_base;

         // - vypis indexu pravidla redukce na vystup -
	 switch (parse_action) {
	 case c_reduce_alter:
	    {
	       unsigned second_begin_idx = begin_idxs.pop();
	       unsigned first_begin_idx = begin_idxs.pop();

	       unsigned second_end_idx = end_idxs.pop();
	       unsigned first_end_idx = end_idxs.pop();

	       // - vytvoreni noveho stavu -
	       states.push_blank();
	       unsigned begin_state_idx = states.used - 1;
	       fa_state_s &begin_state = states[begin_state_idx];
	       begin_state.final = 0;

	       begin_state.moves.push_blank();
	       begin_state.moves.last().set(0xffff,first_begin_idx);

	       begin_state.moves.push_blank();
	       begin_state.moves.last().set(0xffff,second_begin_idx);

	       // - zpetne eps. prechod -
	       // FIXME magic test
	       /*{
		  fa_state_s &f_begin_state = states[first_begin_idx];
		  f_begin_state.moves.push_blank();
		  f_begin_state.moves.last().set(0xffff,begin_state_idx);

		  fa_state_s &s_begin_state = states[second_begin_idx];
		  s_begin_state.moves.push_blank();
		  s_begin_state.moves.last().set(0xffff,begin_state_idx);
	       }*/

	       // - vytvoreni koncoveho stavu -
	       states.push_blank();
	       unsigned end_state_idx = states.used - 1;
	       states[end_state_idx].final = 0;
	       
	       // - spojeni koncu obou casti do koncoveho stavu -
	       {
		  fa_state_s &s_state = states[second_end_idx];
		  s_state.moves.push_blank();
		  s_state.moves.last().set(0xffff,end_state_idx);

		  fa_state_s &f_state = states[first_end_idx];
		  f_state.moves.push_blank();
		  f_state.moves.last().set(0xffff,end_state_idx);
	       }

	       // - zpetne eps. prechody -
	       {
		  /*fa_state_s &end_state = states[end_state_idx];
		  end_state.moves.push_blank();
		  end_state.moves.last().set(0xffff,first_end_idx);

		  end_state.moves.push_blank();
		  end_state.moves.last().set(0xffff,second_end_idx);*/
	       }

	       begin_idxs.push(begin_state_idx);
	       end_idxs.push(end_state_idx);
	    }
	    break;
	 case c_reduce_conc:
	    {
	       unsigned second_begin_idx = begin_idxs.pop();
	       
	       // - poznamenani konce druhe casti -
	       unsigned tmp_end = end_idxs.pop();

	       unsigned first_end_idx = end_idxs.pop();

	       // - pripojeni konce prvni casti k zacatku druhe casti -
	       fa_state_s &state = states[first_end_idx];
	       state.moves.push_blank();
	       state.moves.last().set(0xffff,second_begin_idx);

	       // - zpetny eps. prechod -
	       /*fa_state_s &second_begin_state = states[second_begin_idx];
	       second_begin_state.moves.push_blank();
	       second_begin_state.moves.last().set(0xffff,first_end_idx);*/

	       // - zapis konce druhe casti -
	       end_idxs.push(tmp_end);
	    }
	    break;
	 case c_reduce_iter:
	    {
	       unsigned begin_idx = begin_idxs.last();
	       unsigned end_idx = end_idxs.pop();

	       fa_state_s &state = states[end_idx];
	       state.moves.push_blank();
	       state.moves.last().set(0xffff,begin_idx);

	       // - zpetny eps. prechod -
	       /*fa_state_s &begin_state = states[begin_idx];
	       begin_state.moves.push_blank();
	       begin_state.moves.last().set(0xffff,end_idx);*/

	       end_idxs.push(begin_idx);
	    }
	    break;
	 case c_reduce_char_eps:
	    {
	       states.push_blank();
	       unsigned state_idx = states.used - 1;
	       
	       states[state_idx].final = 0;

	       begin_idxs.push(state_idx);
	       end_idxs.push(state_idx);
	    }
	    break;
	 {
	    unsigned short in_char;

	 case c_reduce_char:
	    {
	       reg_lalr_stack_element_s &lse = lalr_stack[lalr_stack.used - 1];

	       register char *t_ptr = source_string.data + lse.ui_second + 1;
	       in_char = process_char(&t_ptr);

	       goto c_reduce_char_create_states_label;
	    }
	 case c_reduce_char_white:
	    in_char = reg_char_white;
	    goto c_reduce_char_create_states_label;
	 case c_reduce_char_digit:
	    in_char = reg_char_digit;
	    goto c_reduce_char_create_states_label;
	 case c_reduce_char_letter:
	    in_char = reg_char_letter;
	    goto c_reduce_char_create_states_label;
	 case c_reduce_char_no_char:
	    {
	       reg_lalr_stack_element_s &lse = lalr_stack[lalr_stack.used - 1];

	       register char *t_ptr = source_string.data + lse.ui_second + 2;
	       in_char = c_no_char_base + process_char(&t_ptr);
	    }
c_reduce_char_create_states_label:

	    states.push_blank();
	    unsigned begin_state_idx = states.used - 1;
	    fa_state_s &begin_state = states[begin_state_idx];
	    begin_state.final = 0;
	    
	    states.push_blank();
	    unsigned end_state_idx = states.used - 1;
	    fa_state_s &end_state = states[end_state_idx];
	    end_state.final = 0;

	    begin_state.moves.push_blank();
	    begin_state.moves.last().set(in_char,end_state_idx);

	    begin_idxs.push(begin_state_idx);
	    end_idxs.push(end_state_idx);

	    break;
	 }
	 case c_reduce_char_str:
	    {
	       reg_lalr_stack_element_s &lse = lalr_stack[lalr_stack.used - 1];
	      
	       unsigned first_state_idx;
	       unsigned state_idx;

	       first_state_idx = states.used;

	       char *c_ptr = source_string.data + lse.ui_second + 1;
	       char *c_ptr_end = source_string.data + lse.ui_third - 1;
	       unsigned last_state_idx = c_idx_not_exist;

	       do {
		  unsigned short in_char = process_char(&c_ptr);

		  states.push_blank();
		  state_idx = states.used - 1;
		  fa_state_s &state = states[state_idx];

		  state.final = 0;
		  state.moves.push_blank();
		  state.moves.last().set(in_char,c_idx_not_exist);

		  if (last_state_idx != c_idx_not_exist) {
		     states[last_state_idx].moves[0].value = state_idx;
		  }
		  
		  last_state_idx = state_idx;

	       } while(++c_ptr < c_ptr_end);

	       states.push_blank();
	       state_idx = states.used - 1;
	       fa_state_s &state = states[state_idx];

	       state.final = 0;

	       if (last_state_idx != c_idx_not_exist) {
		  states[last_state_idx].moves[0].value = state_idx;
	       }

	       begin_idxs.push(first_state_idx);
	       end_idxs.push(state_idx);
	    }
	    break;
	 case c_reduce_char_range:
	    {
	       reg_lalr_stack_element_s &lse = lalr_stack[lalr_stack.used - 1];
	       unsigned short begin_char;
	       unsigned short end_char;

	       {
		  char *ptr = source_string.data + lse.ui_second + 1;

		  begin_char = process_char(&ptr);
		  ptr++;
		  end_char = process_char(&ptr);
	       }

	       assert(begin_char <= end_char);

	       states.push_blank();
	       unsigned begin_state_idx = states.used - 1;
	       fa_state_s &begin_state = states[begin_state_idx];
	       begin_state.final = 0;
	       
	       states.push_blank();
	       unsigned end_state_idx = states.used - 1;
	       fa_state_s &end_state = states[end_state_idx];
	       end_state.final = 0;

	       do {
		  begin_state.moves.push_blank();
		  begin_state.moves.last().set(begin_char,end_state_idx);
	       } while(++begin_char <= end_char);

	       begin_idxs.push(begin_state_idx);
	       end_idxs.push(end_state_idx);
	    }
	    break;
	 case c_reduce_char_alter:
	    {
	       reg_lalr_stack_element_s &lse = lalr_stack[lalr_stack.used - 1];

	       char *ptr = source_string.data + lse.ui_second + 1;
	       char *ptr_end = source_string.data + lse.ui_third - 1;
	       
	       states.push_blank();
	       unsigned begin_state_idx = states.used - 1;
	       fa_state_s &begin_state = states[begin_state_idx];
	       begin_state.final = 0;
	       
	       states.push_blank();
	       unsigned end_state_idx = states.used - 1;
	       fa_state_s &end_state = states[end_state_idx];
	       end_state.final = 0;

	       do {
		  unsigned short in_char = process_char(&ptr);

		  begin_state.moves.push_blank();
		  begin_state.moves.last().set(in_char,end_state_idx);

	       } while(++ptr < ptr_end);

	       begin_idxs.push(begin_state_idx);
	       end_idxs.push(end_state_idx);
	    }
	    break;
	 case c_reduce_char_except:
	    {
	       reg_lalr_stack_element_s &lse = lalr_stack[lalr_stack.used - 1];

	       char *ptr = source_string.data + lse.ui_second + 1;
	       char *ptr_end = source_string.data + lse.ui_third - 1;

	       us_array_s char_array;
	       char_array.init();
	       
	       do {
		  char_array.push(process_char(&ptr));
	       } while(++ptr < ptr_end);

	       qsort(char_array.data,char_array.used,sizeof(unsigned short),compare_unsigned_shorts);

	       states.push_blank();
	       unsigned begin_state_idx = states.used - 1;
	       fa_state_s &begin_state = states[begin_state_idx];
	       begin_state.final = 0;
	       
	       states.push_blank();
	       unsigned end_state_idx = states.used - 1;
	       fa_state_s &end_state = states[end_state_idx];
	       end_state.final = 0;

	       // - vygenerovani prijimanych znaku -
	       {
		  unsigned b_char = 0;
		  unsigned e_char_idx = 0;

		  do {
		     unsigned short e_char = char_array[e_char_idx];

		     if (b_char < e_char) {
			do {
			   begin_state.moves.push_blank();
			   begin_state.moves.last().set(b_char,end_state_idx);
			} while(++b_char < e_char);
		     }

		     b_char = e_char + 1;

		  } while(++e_char_idx < char_array.used);

		  // - doplneni znaku -
		  if (b_char < 256) {
		     do {
			begin_state.moves.push_blank();
			begin_state.moves.last().set(b_char,end_state_idx);
		     } while(++b_char < 256);
		  }
	       }

	       char_array.clear();

	       begin_idxs.push(begin_state_idx);
	       end_idxs.push(end_state_idx);
	    }
	    break;
	 }

         // - odstraneni tela pravidla z vrcholu zasobniku -
         lalr_stack.used -= rule_body_lengths[parse_action];

         // - ulozeni noveho stavu automatu na zasobnik -
         unsigned goto_val = lalr_table[lalr_stack.last().ui_first*c_terminal_plus_nonterminal_cnt + rule_head_idxs[parse_action]];
         lalr_stack.push(goto_val);
      }

   } while(1);

   // - zapsani pocatecniho stavu automatu -
   fas_idx = begin_idxs.pop();

   // - oznaceni konecnych stavu -
   states[end_idxs.pop()].final = 1;
   
   end_idxs.clear();
   begin_idxs.clear();

   //FIXME debug vypis states
   /*printf("\nNEDETERMINISTICKY KONECNY AUTOMAT S EPS. PRECHODY:\n");
   printf("BEGIN_IDX: %u\n",fas_idx);
   unsigned s_idx = 0;
   do {
      fa_state_s &state = states[s_idx];

      printf("STATE %d:",s_idx);

      if (state.final) {
	 printf(" FINAL\n");
      }
      else {
	 putchar('\n');
      }

      if (state.moves.used != 0) {
	 unsigned m_idx = 0;
	 do {
	    reg_state_move_s &move = state.moves[m_idx];
	    
	    if (move.idx == 0xffff) {
	       printf("\teps -> %u\n",move.value);
	    }
	    else {
	       if (move.idx >= 32 && move.idx <= 126) {
		  printf("\t\'%c\' -> %u\n",move.idx,move.value);
	       }
	       else {
		  printf("\t%hu -> %u\n",move.idx,move.value);
	       }
	    }
	 } while(++m_idx < state.moves.used);
      }
      else {
	 printf("\tno moves\n");
      }
   } while(++s_idx < states.used);*/
}

void reg_parser_s::NKA_to_DKA()
{
   // - presunuti nedeterministickeho automatu -
   unsigned nka_fas_idx = fas_idx;
   fas_idx = 0;

   fa_states_s nka_states;
   nka_states.init();
   nka_states.swap(states);

   // - mnoziny eps-uzaveru jednotlivych stavu nk-automatu -
   ui_arrays_s eps_closures;
   eps_closures.init();

   // - seznam jiz zpracovanych multi stavu s jejich indexy ve vytvarenem automatu -
   reg_mul_state_map_array_s ms_map_array;
   ms_map_array.init();

   // - rada potencialne nezpracovanych stavu - 
   reg_mul_state_move_queue_s msm_queue;
   msm_queue.init();

   // - spocitani eps-uzaveru jednotlivych stavu, a konecnosti stavu pri uvazovani eps. prechodu -
   {
      eps_closures.copy_resize(nka_states.used);
      eps_closures.used = nka_states.used;

      ui_array_s state_idx_stack;
      state_idx_stack.init();

      bb_array_s processed_states;
      processed_states.init();
      processed_states.copy_resize(nka_states.used);

      // - cyklus pres vsechny stavy nedeterministickeho automatu -
      unsigned s_idx = 0;
      do {
	 ui_array_s &eps_closure = eps_closures[s_idx];

	 state_idx_stack.used = 0;
	 state_idx_stack.push(s_idx);

	 processed_states.fill(false);

	 // - cyklus do vyprazdneni zasobniku -
	 do {
	    unsigned ss_idx = state_idx_stack.pop();

	    // - pokud je jiz uzaver stavu na pozici ss_idx spocitan, zkopiruj jeho obsah do pocitaneho uzaveru -
	    if (ss_idx < s_idx) {
	       ui_array_s &ss_eps_closure = eps_closures[ss_idx];
	       eps_closure.add_to_set(ss_eps_closure);

	       unsigned ss_ec_idx = 0;
	       do {
		  processed_states[ss_eps_closure[ss_ec_idx]] = true;
	       } while(++ss_ec_idx < ss_eps_closure.used);
	    }
	    else {

	       // - pokud se stav na pozici ss_idx v uzaveru nenachazi tak jej do nej vloz -
	       if (!processed_states[ss_idx]) {

		  // - stavy do kterych se z vlozeneho stavu prechazi s eps. prechodem pridej do
		  // zasobniku -
		  unsigned have_no_eps_move = false;
		  
		  fa_state_s &state = nka_states[ss_idx];
		  if (state.moves.used != 0) {
		     unsigned m_idx = 0;

		     do {
			reg_state_move_s &move = state.moves[m_idx];
			
			if (move.idx == 0xffff) {
			   state_idx_stack.push(move.value);
			}
			else {
			   have_no_eps_move = true;
			}
		     } while(++m_idx < state.moves.used);
		  }

		  // - pokud stav obsahuje i jine nez eps. prechody pridej jej do eps uzaveru -
		  if (have_no_eps_move || state.final) {
		     eps_closure.push(ss_idx);
		  }

		  processed_states[ss_idx] = true;
	       }
	    }
	 } while(state_idx_stack.used > 0);
      } while(++s_idx < nka_states.used);

      processed_states.clear();
      state_idx_stack.clear();
   }

   // - vlozeni pocatecniho stavu {0} do vstupni rady -
   msm_queue.insert_blank();
   {
      reg_mul_state_move_s &mul_state_move = msm_queue.last();

      mul_state_move.move_char = 0xffff;
      mul_state_move.from_state = c_idx_not_exist;
      mul_state_move.mul_state = eps_closures[nka_fas_idx];
   }

   reg_mul_state_move_s mul_state_move;
   mul_state_move.init();

   reg_mul_state_move_array_s new_mul_state_moves;
   new_mul_state_moves.init();

   do {
      mul_state_move.swap(msm_queue.next());
      
      unsigned mul_state_idx = ms_map_array.get_idx_by_mul_state(mul_state_move.mul_state);
      if (mul_state_idx == c_idx_not_exist) {

	 // - vytvoreni noveho stavu -
	 states.push_blank();
	 fa_state_s &state = states.last();
	 state.final = 0;

	 ms_map_array.push_blank();
	 reg_mul_state_map_s &ms_map = ms_map_array.last();
	 ms_map.mul_state.swap(mul_state_move.mul_state);
	 ms_map.dka_state_idx = states.used - 1;

	 mul_state_idx = ms_map_array.used - 1;

	 // - nalezeni dalsich prechodu z noveho stavu -
	 new_mul_state_moves.used = 0;
	 ui_array_s &mul_state = ms_map.mul_state;

	 // - cyklus pres vsechny nka stavy v eps. uzaveru -
	 if (mul_state.used != 0) {
	    unsigned s_idx = 0;
	    do {
	       fa_state_s &nka_state = nka_states[mul_state[s_idx]];
	       
	       // - nastaveni konecnosti noveho stavu -
	       if (nka_state.final) {
		  state.final = 1;
	       }

	       reg_state_moves_s &state_moves = nka_state.moves;

	       // - cyklus pres vsechny prechody v aktualnim stavu (z eps. uzaveru) -
	       if (state_moves.used != 0) {
		  unsigned m_idx = 0;
		  do {
		     reg_state_move_s &move = state_moves[m_idx];
		     
		     // - kdyz se nejedna o eps. prechod -
		     if (move.idx != 0xffff) {

			// - generuj nove kandidaty na stav deterministickeho automatu -
			unsigned mc_idx = new_mul_state_moves.get_idx_by_move_char(move.idx);
			if (mc_idx == c_idx_not_exist) {
			   new_mul_state_moves.push_clear();
			   reg_mul_state_move_s &mul_state_move = new_mul_state_moves.last();
			   mul_state_move.move_char = move.idx;
			   mul_state_move.from_state = ms_map.dka_state_idx;

			   mc_idx = new_mul_state_moves.used - 1;
			}
			
			reg_mul_state_move_s &mul_state_move = new_mul_state_moves[mc_idx];
			mul_state_move.mul_state.add_to_set(eps_closures[move.value]);
		     }

		  } while(++m_idx < state_moves.used);
	       }
	    } while(++s_idx < mul_state.used);
	 }

	 if (new_mul_state_moves.used != 0) {
	    unsigned idx = 0;
	    do {
	       msm_queue.insert_blank();
	       msm_queue.last().swap(new_mul_state_moves[idx]);
	    } while(++idx < new_mul_state_moves.used);
	 }
      }

      unsigned this_state_idx = ms_map_array[mul_state_idx].dka_state_idx;

      // - zapis prechodu do tohoto stavu -
      if (mul_state_move.from_state != c_idx_not_exist) {
	 fa_state_s &state = states[mul_state_move.from_state];
	 state.moves.push_blank();

	 reg_state_move_s &move = state.moves.last();
	 move.idx = mul_state_move.move_char;
	 move.value = this_state_idx;
      }

      mul_state_move.clear();
   } while(msm_queue.used > 0);
   
   new_mul_state_moves.clear();
   mul_state_move.clear();

   eps_closures.clear();
   ms_map_array.clear();
   msm_queue.clear();
   nka_states.clear();

   //FIXME debug vypis states
   /*printf("\nDETERMINISTICKY KONECNY AUTOMAT:\n");
   unsigned s_idx = 0;
   do {
      fa_state_s &state = states[s_idx];

      printf("STATE %d:",s_idx);

      if (state.final) {
	 printf(" FINAL\n");
      }
      else {
	 putchar('\n');
      }

      if (state.moves.used != 0) {
	 unsigned m_idx = 0;
	 do {
	    reg_state_move_s &move = state.moves[m_idx];

	    if (move.idx >= 32 && move.idx <= 126) {
	       printf("\t\'%c\' -> %u\n",move.idx,move.value);
	    }
	    else {
	       printf("\t%hu -> %u\n",move.idx,move.value);
	    }
	 } while(++m_idx < state.moves.used);
      }
      else {
	 printf("\tno moves\n");
      }
   } while(++s_idx < states.used);*/
}

inline void reg_parser_s::process_reg_exp(string_s &source_string)
{
   parse_reg_exp(source_string);
   NKA_to_DKA();
}

void reg_parser_s::process_reg_exps(string_array_s &source_strings,fa_states_array_s &states_array)
{
   states_array.clear();

   if (source_strings.used == 0) {
      return;
   }

   states_array.copy_resize(source_strings.used);

   unsigned idx = 0;
   do {
      process_reg_exp(source_strings[idx]);

      states_array.push_blank();
      states_array.last().swap(states);

   } while(++idx < source_strings.used);
}

//******************************************************************************

// -- funkce struktury final_automata_s --

/*
 * vytvori konecny automat, ktery od sebe rozpozna jednotlive regularni vyrazy predane parametrem
 */
void final_automata_s::create(ushort_reg_exps_s &reg_exps)
{
   assert(reg_exps.used != 0);

   clear();

   // - rada popisu novych stavu -
   fa_state_descr_queue_s queue;
   queue.init();

   fa_state_reg_states_s state_descrs;
   state_descrs.init();

   // - pole prechodu vyuzivane v ramci jednoho stavu -
   fa_state_moves_array_s moves_array;
   moves_array.init();
  
   // - pracovni zasobnik pro nektere volane funkce, aby se zbytecne nemusel alokovat pro kazdou novy -
   ui_array_s work_stack;
   work_stack.init();

   // - promenna popisujici nove vytvareny stav automatu -
   fa_state_descr_s new_state_descr;
   new_state_descr.init();

   // - vlozeni prvniho zaznamu do rady popisu stavu -
   {
      /*
       * stav ze ktereho se vstupuje do prvniho stavu neexistuje
       * mnozina reg. vyrazu obsahuje vsechny s indexem na pozici 0
       */
      queue.insert_blank();
      queue.last().from_state_id = c_idx_not_exist;
      reg_states_s &init_reg_states = queue.last().reg_states;
      init_reg_states.copy_resize(reg_exps.used);

      unsigned idx = 0;
      do {
	 init_reg_states.push_blank();
	 init_reg_states[idx].set(idx,0);
      } while(++idx < reg_exps.used);
   }

   // - cyklus dokud neni prazdna rada popisu stavu -
   do {
      
      // - ziska nasledujici popis stavu z rady -
      fa_state_descr_s q_state_descr;
      q_state_descr.init();
      q_state_descr.swap(queue.next());

      // - kontrola jestli jiz neni stejny stav definovany -
      unsigned state_descr_idx = state_descrs.get_idx(q_state_descr.reg_states);
      if (state_descr_idx != c_idx_not_exist) {

	 /*
	  * pokud ano, tak ziskej stav ze ktereho se prechazi do tohoto stavu
	  * ziskej index prechodu v tomto stavu, ktery souhlasi se znakem
	  * se kterym se prechazi, a ve stavu ze ktereho se prechazi indentifikuj jako
	  * cilovy tento stav
	  * pokracuj od zacatku
	  */
	 fa_state_s &state = states[q_state_descr.from_state_id];

	 fa_state_move_s s_move;
	 s_move.idx = q_state_descr.with_char;

	 unsigned move_idx = state.moves.get_idx(s_move);
	 assert(move_idx != c_idx_not_exist);
	 state.moves[move_idx].value = state_descr_idx;

	 q_state_descr.clear();
	 continue;
      }

      // - vytvoreni noveho stavu automatu, vytvor ref. na jeho koncovy terminal a mnozinu prechodu-
      states.push_blank();
      unsigned &state_final = states.last().final;
      state_final = c_idx_not_exist;
      fa_state_moves_s &state_moves = states.last().moves;

      // - nastaveni prechodu do tohoto stavu -
      reg_states_s &p_reg_states = q_state_descr.reg_states;
      assert(p_reg_states.used != 0);

      // - pokud existuje stav ze ktereho se vstoupilo do tohoto stavu -
      if (q_state_descr.from_state_id != c_idx_not_exist) {

	 // - tak v tomto stavu nastav prechod do noveho stavu -
	 fa_state_s &from_state = states[q_state_descr.from_state_id];

	 fa_state_move_s s_move;
	 s_move.idx = q_state_descr.with_char;

	 unsigned move_idx = from_state.moves.get_idx(s_move);
	 assert(move_idx != c_idx_not_exist);
	 from_state.moves[move_idx].value = states.used - 1;
      }

      // - vygenerovani pole prechodu z aktualnich pozic ve vyrazech -
      moves_array.used = 0;
      reg_state_s *p_reg_state_ptr = p_reg_states.data;
      reg_state_s *p_reg_state_ptr_end = p_reg_state_ptr + p_reg_states.used;

      // - cyklus pres seznam regularnich vyrazu stavu a generovani mnoziny moznych prechodu -
      do {
	 moves_array.push_blank();
	 bool ret_val = reg_exps[p_reg_state_ptr->ui_first].next_chars(p_reg_state_ptr->ui_second,moves_array.last(),work_stack);

	 // - prirazeni regularniho vyrazu ke koncovemu stavu -
	 if (ret_val) {
	    if (state_final == c_idx_not_exist || moves_array.last().used == 0) {
	       state_final = p_reg_state_ptr->ui_first;
	    }
	 }

      } while(++p_reg_state_ptr < p_reg_state_ptr_end);

      do {
	 unsigned short move_char = 0xffff;

	 // - vybere dalsi prechodovy znak z mnoziny -
	 fa_state_moves_s *moves_ptr = moves_array.data;
	 fa_state_moves_s *moves_ptr_end = moves_ptr + moves_array.used;

	 do {
	    if (moves_ptr->used > 0) {
	       fa_state_move_s *move_ptr = moves_ptr->data;
	       fa_state_move_s *move_ptr_end = move_ptr + moves_ptr->used;

	       do {
		  if (move_ptr->idx < move_char) {
		     move_char = move_ptr->idx;
		  }
	       } while(++move_ptr < move_ptr_end);
	    }
	 } while(++moves_ptr < moves_ptr_end);

	 if (move_char == 0xffff) {
	    break;
	 }

	 // - pridani noveho prechodu ze stavu bez urceni ciloveho stavu -
	 state_moves.push_blank();
	 state_moves.last().idx = move_char;

	 // - vytvoreni popisu noveho stavu (cile prechodu s move_char) -
	 new_state_descr.from_state_id = states.used - 1;
	 new_state_descr.with_char = move_char;
	 reg_states_s &new_reg_states = new_state_descr.reg_states;

	 new_reg_states.used = 0;

	 // - nalezeni a zpracovani vsech prechodu s timto znakem -
	 p_reg_state_ptr = p_reg_states.data;
	 moves_ptr = moves_array.data;
	 moves_ptr_end = moves_ptr + moves_array.used;

	 // - cyklus pres vsechny prechody -
	 do {
	    if (moves_ptr->used > 0) {
	       fa_state_move_s *move_ptr = moves_ptr->data;
	       fa_state_move_s *move_ptr_end = move_ptr + moves_ptr->used;

	       do {
		  if (move_ptr->idx != 0xffff) {

		     unsigned short ptr_char = move_ptr->idx;
		     bool match = false;
		     bool eq_match = false;

		     //TODO zefektivnit porovnani
		     if (ptr_char >= c_no_char_base) {
			if (move_char == ptr_char) {
			   eq_match = true;
			}
			else if (move_char != (ptr_char - c_no_char_base)) {
			   match = true;
			}
		     }
		     else if (ptr_char == reg_char_digit) {
			if (move_char == reg_char_digit) {
			   eq_match = true;
			}
			else if (move_char >= '0' && move_char <= '9') {
			   match = true;
			}
		     }
		     else if (ptr_char == reg_char_letter) {
			if (move_char == reg_char_letter) {
			   eq_match = true;
			}
			else if ((move_char >= 'a' && move_char <= 'z') || (move_char >= 'A' && move_char <= 'Z')) {
			   match = true;
			}
		     }
		     else if (ptr_char == reg_char_white) {
			if (move_char == reg_char_white) {
			   eq_match = true;
			}
			else if (move_char == ' ' || move_char == '\t' || move_char == '\n') {
			   match = true;
			}
		     }
		     else if (ptr_char == move_char) {
			eq_match = true;
		     }

		     if (eq_match || match) {
			new_reg_states.push_blank();
			new_reg_states.last().set(p_reg_state_ptr->ui_first,move_ptr->value);
			
			if (eq_match) {
			   move_ptr->idx = 0xffff;
			}
		     }
		  }
	       } while(++move_ptr < move_ptr_end);
	    }
	 } while(++p_reg_state_ptr,++moves_ptr < moves_ptr_end);
	
	 // - vlozi popis noveho stavu do rady popisu stavu -
	 queue.insert_blank();
	 queue.last().swap(new_state_descr);
      } while(1);

      // - ulozi popis jiz akceptovaneho stavu, pro kontrolu podobnosti stavu -
      state_descrs.push_blank();
      state_descrs.last().swap(q_state_descr.reg_states);
      q_state_descr.clear();

   } while(queue.used > 0);

   // - orezani zbytecne pridelene pameti -
   states.flush();

   /*
    * FIXME debug vypis stavu konecneho automatu
    * je vhodne vypis presmerovat do souboru, kvuli
    * nekterym znakum ktere nejdou dost dobre zobrazit v terminalu
    */
   /*{
      if (states.used > 0) {
	 unsigned state_id = 0;

	 fa_state_s *state_ptr = states.data;
	 fa_state_s *state_ptr_end = state_ptr + states.used;

	 do {
	    printf("%d:",state_id);
	    
	    if (state_ptr->final != c_idx_not_exist) {
	       printf(" end: %d\n",state_ptr->final);
	    }
	    else {
	       putchar('\n');
	    }

	    if (state_ptr->moves.used > 0) {
	       fa_state_move_s *move_ptr = state_ptr->moves.data;
	       fa_state_move_s *move_ptr_end = move_ptr + state_ptr->moves.used;
	       
	       do {
		  printf("\t%c -> %d\n",move_ptr->idx,move_ptr->value);
	       } while(++move_ptr < move_ptr_end);
	    }
	    else {
	       putchar('\n');
	    }
	 } while(++state_id,++state_ptr < state_ptr_end);
      }
   }*/

   // - odstrani docasne pouzivane dynamicke data -
   new_state_descr.clear();
   work_stack.clear();
   moves_array.clear();
   state_descrs.clear();
   queue.clear();

   /*
    * vytvoreni efektivni ale rozsahle mnoziny prechodu jednotlivych stavu
    */

   state_moves.copy_resize(states.used);

   // - cyklus pres vsechny stavy automatu -
   if (states.used != 0) {
      fa_state_s *state_ptr = states.data;
      fa_state_s *state_ptr_end = state_ptr + states.used;

      do {
	 state_moves.push_blank();
	 ui_array_s &moves = state_moves.last();

	 moves.init();
	 moves.copy_resize(c_base_char_cnt);
	 moves.fill(c_idx_not_exist);

	 // - cyklus pres prechody stavu a jejich vyplneni do tabulky prechodu -
	 if (state_ptr->moves.used != 0) {
	    fa_state_move_s *move_ptr = state_ptr->moves.data;
	    fa_state_move_s *move_ptr_end = move_ptr + state_ptr->moves.used;

	    do {
	       if (move_ptr->idx < c_base_char_cnt) {
		  moves[move_ptr->idx] = move_ptr->value;
	       }
	       else {
		  
		  // - zpracovani specialnich reg. znaku -
		  if (move_ptr->idx >= c_no_char_base) {
		     unsigned *mt_ptr = moves.data;
		     unsigned *mt_ptr_end = moves.data + (move_ptr->idx - c_no_char_base);

		     do {
			if (*mt_ptr == c_idx_not_exist) {
			   *mt_ptr = move_ptr->value;
			}
		     } while(++mt_ptr < mt_ptr_end);

		     mt_ptr = mt_ptr_end + 1;
		     mt_ptr_end = moves.data + moves.used;

		     do {
			if (*mt_ptr == c_idx_not_exist) {
			   *mt_ptr = move_ptr->value;
			}
		     } while(++mt_ptr < mt_ptr_end);
		  }
		  else {
		     switch (move_ptr->idx) {
		     case reg_char_white:
			{
			   unsigned *mt_ptr = moves.data + ' ';
			   if (*mt_ptr == c_idx_not_exist) {
			      *mt_ptr = move_ptr->value;
			   }

			   mt_ptr = moves.data + '\t';
			   if (*mt_ptr == c_idx_not_exist) {
			      *mt_ptr = move_ptr->value;
			   }

			   mt_ptr = moves.data + '\n';
			   if (*mt_ptr == c_idx_not_exist) {
			      *mt_ptr = move_ptr->value;
			   }
			}
			break;
		     case reg_char_digit:
			{
			   unsigned *mt_ptr = moves.data + '0';
			   unsigned *mt_ptr_end = moves.data + '9' + 1;

			   do {
			      if (*mt_ptr == c_idx_not_exist) {
				 *mt_ptr = move_ptr->value;
			      }
			   } while(++mt_ptr < mt_ptr_end);
			}
			break;
		     case reg_char_letter:
			{
			   unsigned *mt_ptr = moves.data + 'a';
			   unsigned *mt_ptr_end = moves.data + 'z' + 1;

			   do {
			      if (*mt_ptr == c_idx_not_exist) {
				 *mt_ptr = move_ptr->value;
			      }
			   } while(++mt_ptr < mt_ptr_end);

			   mt_ptr = moves.data + 'A';
			   mt_ptr_end = moves.data + 'Z' + 1;

			   do {
			      if (*mt_ptr == c_idx_not_exist) {
				 *mt_ptr = move_ptr->value;
			      }
			   } while(++mt_ptr < mt_ptr_end);
			}
			break;
		     default:
			assert(0);
		     }
		  }
	       }
	    } while(++move_ptr < move_ptr_end);
	 }

      } while(++state_ptr < state_ptr_end);
   }
}

void final_automata_s::create_new(fa_states_array_s &states_array)
{
   assert(states_array.used != 0);

   clear();

   // - rada popisu novych stavu -
   fa_state_descr_queue_s queue;
   queue.init();

   fa_state_reg_states_s state_descrs;
   state_descrs.init();

   // - pole prechodu vyuzivane v ramci jednoho stavu -
   fa_state_moves_array_s moves_array;
   moves_array.init();
  
   // - pracovni zasobnik pro nektere volane funkce, aby se zbytecne nemusel alokovat pro kazdou novy -
   ui_array_s work_stack;
   work_stack.init();

   // - promenna popisujici nove vytvareny stav automatu -
   fa_state_descr_s new_state_descr;
   new_state_descr.init();

   // - vlozeni prvniho zaznamu do rady popisu stavu -
   {
      /*
       * stav ze ktereho se vstupuje do prvniho stavu neexistuje
       * mnozina reg. vyrazu obsahuje vsechny s indexem na pozici 0
       */
      queue.insert_blank();
      queue.last().from_state_id = c_idx_not_exist;
      reg_states_s &init_reg_states = queue.last().reg_states;
      init_reg_states.copy_resize(states_array.used);

      unsigned idx = 0;
      do {
	 init_reg_states.push_blank();
	 init_reg_states[idx].set(idx,0);
      } while(++idx < states_array.used);
   }

   // - cyklus dokud neni prazdna rada popisu stavu -
   do {
      
      // - ziska nasledujici popis stavu z rady -
      fa_state_descr_s q_state_descr;
      q_state_descr.init();
      q_state_descr.swap(queue.next());

      // - kontrola jestli jiz neni stejny stav definovany -
      unsigned state_descr_idx = state_descrs.get_idx(q_state_descr.reg_states);
      if (state_descr_idx != c_idx_not_exist) {

	 /*
	  * pokud ano, tak ziskej stav ze ktereho se prechazi do tohoto stavu
	  * ziskej index prechodu v tomto stavu, ktery souhlasi se znakem
	  * se kterym se prechazi, a ve stavu ze ktereho se prechazi indentifikuj jako
	  * cilovy tento stav
	  * pokracuj od zacatku
	  */
	 fa_state_s &state = states[q_state_descr.from_state_id];

	 fa_state_move_s s_move;
	 s_move.idx = q_state_descr.with_char;

	 unsigned move_idx = state.moves.get_idx(s_move);
	 assert(move_idx != c_idx_not_exist);
	 state.moves[move_idx].value = state_descr_idx;

	 q_state_descr.clear();
	 continue;
      }

      // - vytvoreni noveho stavu automatu, vytvor ref. na jeho koncovy terminal a mnozinu prechodu-
      states.push_blank();
      unsigned &state_final = states.last().final;
      state_final = c_idx_not_exist;
      fa_state_moves_s &state_moves = states.last().moves;

      // - nastaveni prechodu do tohoto stavu -
      reg_states_s &p_reg_states = q_state_descr.reg_states;
      assert(p_reg_states.used != 0);

      // - pokud existuje stav ze ktereho se vstoupilo do tohoto stavu -
      if (q_state_descr.from_state_id != c_idx_not_exist) {

	 // - tak v tomto stavu nastav prechod do noveho stavu -
	 fa_state_s &from_state = states[q_state_descr.from_state_id];

	 fa_state_move_s s_move;
	 s_move.idx = q_state_descr.with_char;

	 unsigned move_idx = from_state.moves.get_idx(s_move);
	 assert(move_idx != c_idx_not_exist);
	 from_state.moves[move_idx].value = states.used - 1;
      }

      // - vygenerovani pole prechodu z aktualnich pozic ve vyrazech -
      moves_array.used = 0;
      reg_state_s *p_reg_state_ptr = p_reg_states.data;
      reg_state_s *p_reg_state_ptr_end = p_reg_state_ptr + p_reg_states.used;

      // - cyklus pres seznam regularnich vyrazu stavu a generovani mnoziny moznych prechodu -
      do {
	 moves_array.push_blank();

	 fa_state_s &state = states_array[p_reg_state_ptr->ui_first][p_reg_state_ptr->ui_second];
	 moves_array.last() = state.moves;

	 // - prirazeni regularniho vyrazu ke koncovemu stavu -
	 if (state.final) {
	    if (state_final == c_idx_not_exist || moves_array.last().used == 0) {
	       state_final = p_reg_state_ptr->ui_first;
	    }
	 }

      } while(++p_reg_state_ptr < p_reg_state_ptr_end);

      do {
	 unsigned short move_char = 0xffff;

	 // - vybere dalsi prechodovy znak z mnoziny -
	 fa_state_moves_s *moves_ptr = moves_array.data;
	 fa_state_moves_s *moves_ptr_end = moves_ptr + moves_array.used;

	 do {
	    if (moves_ptr->used > 0) {
	       fa_state_move_s *move_ptr = moves_ptr->data;
	       fa_state_move_s *move_ptr_end = move_ptr + moves_ptr->used;

	       do {
		  if (move_ptr->idx < move_char) {
		     move_char = move_ptr->idx;
		  }
	       } while(++move_ptr < move_ptr_end);
	    }
	 } while(++moves_ptr < moves_ptr_end);

	 if (move_char == 0xffff) {
	    break;
	 }

	 // - pridani noveho prechodu ze stavu bez urceni ciloveho stavu -
	 state_moves.push_blank();
	 state_moves.last().idx = move_char;

	 // - vytvoreni popisu noveho stavu (cile prechodu s move_char) -
	 new_state_descr.from_state_id = states.used - 1;
	 new_state_descr.with_char = move_char;
	 reg_states_s &new_reg_states = new_state_descr.reg_states;

	 new_reg_states.used = 0;

	 // - nalezeni a zpracovani vsech prechodu s timto znakem -
	 p_reg_state_ptr = p_reg_states.data;
	 moves_ptr = moves_array.data;
	 moves_ptr_end = moves_ptr + moves_array.used;

	 // - cyklus pres vsechny prechody -
	 do {
	    if (moves_ptr->used > 0) {
	       fa_state_move_s *move_ptr = moves_ptr->data;
	       fa_state_move_s *move_ptr_end = move_ptr + moves_ptr->used;

	       do {
		  if (move_ptr->idx != 0xffff) {

		     unsigned short ptr_char = move_ptr->idx;
		     bool match = false;
		     bool eq_match = false;

		     //TODO zefektivnit porovnani
		     if (ptr_char >= c_no_char_base) {
			if (move_char == ptr_char) {
			   eq_match = true;
			}
			else if (move_char != (ptr_char - c_no_char_base)) {
			   match = true;
			}
		     }
		     else if (ptr_char == reg_char_digit) {
			if (move_char == reg_char_digit) {
			   eq_match = true;
			}
			else if (move_char >= '0' && move_char <= '9') {
			   match = true;
			}
		     }
		     else if (ptr_char == reg_char_letter) {
			if (move_char == reg_char_letter) {
			   eq_match = true;
			}
			else if ((move_char >= 'a' && move_char <= 'z') || (move_char >= 'A' && move_char <= 'Z')) {
			   match = true;
			}
		     }
		     else if (ptr_char == reg_char_white) {
			if (move_char == reg_char_white) {
			   eq_match = true;
			}
			else if (move_char == ' ' || move_char == '\t' || move_char == '\n') {
			   match = true;
			}
		     }
		     else if (ptr_char == move_char) {
			eq_match = true;
		     }

		     if (eq_match || match) {
			new_reg_states.push_blank();
			new_reg_states.last().set(p_reg_state_ptr->ui_first,move_ptr->value);
			
			if (eq_match) {
			   move_ptr->idx = 0xffff;
			}
		     }
		  }
	       } while(++move_ptr < move_ptr_end);
	    }
	 } while(++p_reg_state_ptr,++moves_ptr < moves_ptr_end);
	
	 // - vlozi popis noveho stavu do rady popisu stavu -
	 queue.insert_blank();
	 queue.last().swap(new_state_descr);
      } while(1);

      // - ulozi popis jiz akceptovaneho stavu, pro kontrolu podobnosti stavu -
      state_descrs.push_blank();
      state_descrs.last().swap(q_state_descr.reg_states);
      q_state_descr.clear();

   } while(queue.used > 0);

   // - orezani zbytecne pridelene pameti -
   states.flush();

   /*
    * FIXME debug vypis stavu konecneho automatu
    * je chodne vypis presmerovat do souboru, kvuli
    * nekterym znakum ktere nejdou dost dobre zobrazit v terminalu
    */
   /*{
      if (states.used > 0) {
	 unsigned state_id = 0;

	 fa_state_s *state_ptr = states.data;
	 fa_state_s *state_ptr_end = state_ptr + states.used;

	 do {
	    printf("%d:",state_id);
	    
	    if (state_ptr->final != c_idx_not_exist) {
	       printf(" end: %d\n",state_ptr->final);
	    }
	    else {
	       putchar('\n');
	    }

	    if (state_ptr->moves.used > 0) {
	       fa_state_move_s *move_ptr = state_ptr->moves.data;
	       fa_state_move_s *move_ptr_end = move_ptr + state_ptr->moves.used;
	       
	       do {
		  printf("\t%c -> %d\n",move_ptr->idx,move_ptr->value);
	       } while(++move_ptr < move_ptr_end);
	    }
	    else {
	       putchar('\n');
	    }
	 } while(++state_id,++state_ptr < state_ptr_end);
      }
   }*/

   // - odstrani docasne pouzivane dynamicke data -
   new_state_descr.clear();
   work_stack.clear();
   moves_array.clear();
   state_descrs.clear();
   queue.clear();

   /*
    * vytvoreni efektivni ale rozsahle mnoziny prechodu jednotlivych stavu
    */

   state_moves.copy_resize(states.used);

   // - cyklus pres vsechny stavy automatu -
   if (states.used != 0) {
      fa_state_s *state_ptr = states.data;
      fa_state_s *state_ptr_end = state_ptr + states.used;

      do {
	 state_moves.push_blank();
	 ui_array_s &moves = state_moves.last();

	 moves.init();
	 moves.copy_resize(c_base_char_cnt);
	 moves.fill(c_idx_not_exist);

	 // - cyklus pres prechody stavu a jejich vyplneni do tabulky prechodu -
	 if (state_ptr->moves.used != 0) {
	    fa_state_move_s *move_ptr = state_ptr->moves.data;
	    fa_state_move_s *move_ptr_end = move_ptr + state_ptr->moves.used;

	    do {
	       if (move_ptr->idx < c_base_char_cnt) {
		  moves[move_ptr->idx] = move_ptr->value;
	       }
	       else {
		  
		  // - zpracovani specialnich reg. znaku -
		  if (move_ptr->idx >= c_no_char_base) {
		     unsigned *mt_ptr = moves.data;
		     unsigned *mt_ptr_end = moves.data + (move_ptr->idx - c_no_char_base);

		     do {
			if (*mt_ptr == c_idx_not_exist) {
			   *mt_ptr = move_ptr->value;
			}
		     } while(++mt_ptr < mt_ptr_end);

		     mt_ptr = mt_ptr_end + 1;
		     mt_ptr_end = moves.data + moves.used;

		     do {
			if (*mt_ptr == c_idx_not_exist) {
			   *mt_ptr = move_ptr->value;
			}
		     } while(++mt_ptr < mt_ptr_end);
		  }
		  else {
		     switch (move_ptr->idx) {
		     case reg_char_white:
			{
			   unsigned *mt_ptr = moves.data + ' ';
			   if (*mt_ptr == c_idx_not_exist) {
			      *mt_ptr = move_ptr->value;
			   }

			   mt_ptr = moves.data + '\t';
			   if (*mt_ptr == c_idx_not_exist) {
			      *mt_ptr = move_ptr->value;
			   }

			   mt_ptr = moves.data + '\n';
			   if (*mt_ptr == c_idx_not_exist) {
			      *mt_ptr = move_ptr->value;
			   }
			}
			break;
		     case reg_char_digit:
			{
			   unsigned *mt_ptr = moves.data + '0';
			   unsigned *mt_ptr_end = moves.data + '9' + 1;

			   do {
			      if (*mt_ptr == c_idx_not_exist) {
				 *mt_ptr = move_ptr->value;
			      }
			   } while(++mt_ptr < mt_ptr_end);
			}
			break;
		     case reg_char_letter:
			{
			   unsigned *mt_ptr = moves.data + 'a';
			   unsigned *mt_ptr_end = moves.data + 'z' + 1;

			   do {
			      if (*mt_ptr == c_idx_not_exist) {
				 *mt_ptr = move_ptr->value;
			      }
			   } while(++mt_ptr < mt_ptr_end);

			   mt_ptr = moves.data + 'A';
			   mt_ptr_end = moves.data + 'Z' + 1;

			   do {
			      if (*mt_ptr == c_idx_not_exist) {
				 *mt_ptr = move_ptr->value;
			      }
			   } while(++mt_ptr < mt_ptr_end);
			}
			break;
		     default:
			assert(0);
		     }
		  }
	       }
	    } while(++move_ptr < move_ptr_end);
	 }

      } while(++state_ptr < state_ptr_end);
   }
}

/*
 * rozpozna v predanem retezci na predane pozici terminal konecnym automatem
 */
unsigned final_automata_s::recognize(char *input,unsigned &input_idx,unsigned input_length,unsigned &line_cnt)
{
   debug_assert(input != NULL);

   // - nastaveni pocatecniho stavu -
   state_idx = 0;

   // - nekonecny cyklus hledajici terminal -
   do {

      // - ziskani ref. na aktualni stav a priprava promenne pro nasledny stav -
      fa_state_s &state = states[state_idx];
      unsigned next_state_idx = c_idx_not_exist;

      // - zpracovani nasledujiciho vstupniho znaku -
      unsigned short in_char;
      if (input_idx < input_length) {
	 in_char = (unsigned char)input[input_idx];

	 //FIXME vyresit volbu vlastniho escape znaku, nebo znakuu
	 //FIXME debug test
	 /*if (in_char == '\\') {
	    assert(input_idx + 1 < input_length);
	    in_char = c_escape_char; 
	 }*/
      }
      else {
	 in_char = '\0';
      }

      // - prechod do nasledujiciho stavu -
      next_state_idx = state_moves[state_idx][in_char];

      /*
       * starsi zpusob prechazeni mezi stavy, pomalejsi, ale nepotrebuje velkou strukturu v pameti
       */
      // - ziskani mnoziny moznych prechodu -
      /*fa_state_moves_s &state_moves = state.moves;

      if (state.moves.used > 0) {
	 fa_state_move_s *state_move_ptr = state_moves.data;
	 fa_state_move_s *state_move_ptr_end = state_move_ptr + state_moves.used;

	 // - cyklus pres mnozinu prechodu a hledani odpovidajiciho pro nasledujici znak -
	 do {
	    unsigned short move_reg_char = state_move_ptr->idx;
	    
	    if (move_reg_char < c_base_char_cnt) {
	       if (move_reg_char == in_char) {
		  next_state_idx = state_move_ptr->value;
		  break;
	       }
	    }
	    else {
	       //reg_char_...
	       if (move_reg_char >= c_no_char_base) {
		  if (in_char != move_reg_char - c_no_char_base) {
		     next_state_idx = state_move_ptr->value;
		     break;
		  }
	       }
	       else {
		  switch (move_reg_char) {
		  case reg_char_white:
		     if (in_char == ' ' || in_char == '\t' || in_char == '\n') {
			next_state_idx = state_move_ptr->value;
			break;
		     }
		     break;
		  case reg_char_digit:
		     if (in_char >= '0' && in_char <= '9') {
			next_state_idx = state_move_ptr->value;
			break;
		     }
		     break;
		  case reg_char_letter:
		     if ((in_char >= 'a' && in_char <= 'z') || (in_char >= 'A' && in_char <= 'Z')) {
			next_state_idx = state_move_ptr->value;
			break;
		     }
		     break;
		  default:
		     assert(0);
		  }
	       }
	    }
	 } while(++state_move_ptr < state_move_ptr_end);
      }*/

      // - prechod do nasledujiciho stavu pokud existuje -
      if (next_state_idx != c_idx_not_exist) {
	 
	 // - ukonceni vstupu -
	 if (in_char == '\0') {
	    return states[next_state_idx].final;
	 }

	 // - pocitani radku v retezci -
	 if (in_char == '\n') {
	    line_cnt++;
	 }

	 //FIXME debug test
	 /*if (in_char == c_escape_char) {
	    input_idx += 2;
	 }
	 else {*/
	    input_idx++;
	 //}

	 state_idx = next_state_idx;
      }
      
      // - neexistuje prechod do dalsiho stavu, rozpoznany terminal je state.final -
      else {
	 return state.final;
      }
   } while(1);
}


