
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

/*
 * obsah funkce prevadejici obecne instrukce semantickeho kodu na instrukce
 * popisujici operaci vcetne typu dat nad kterymi se provadi
 *
 * v podstate jde jen o zasobnik (var_type_stack), do ktereho se ukladaji typy hodnot podle
 * provadenych operaci a generuji se operace pracujici s typy na zasobniku,
 * tj. pri spousteni uz neni potreba typova kontrola dat.
 *
 * priklad:
 * 
 * zapis: (zdrojovy semanticky program)
 * num + 10
 *
 * vstup: (obecne instrukce)
 * stt_id (unsigned)
 * stt_int
 * stt_add
 *
 * vystup: (instrukce vcetne typu dat)
 * sott_int_var idx
 * sott_int_value val
 * sott_add_int_var_unsined_value
 *
 * interpretace: (funkce)
 * stack.push(idx)
 * stack.push(val)
 * stack.push(stack.pop() + int_variables[stack.pop()])
 *
 * provadi se zde take deklarovani promennych, a kontrola zda jsou promenne deklarovany
 */

/*
 * makra zjednodusujici zapis podobnych konverzi instrukci
 */
#define UNSIGNED_OP(OP_NAME,OP_POSTFIX,STACK_END_OP,EXPAND_CODE) \
{\
   switch (var_type_stack.STACK_END_OP()) {\
   case svt_int:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_var ## OP_POSTFIX);\
      EXPAND_CODE;\
      break;\
   case svt_int_tmp:\
   case svt_int_const:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_value ## OP_POSTFIX);\
      EXPAND_CODE;\
      break;\
   case svt_int_aref:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aref ## OP_POSTFIX);\
      EXPAND_CODE;\
      break;\
   case svt_int_aaref:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aaref ## OP_POSTFIX);\
      EXPAND_CODE;\
      break;\
   default:\
      assert(0);\
   }\
}

#define UNSIGNED_PUSH_VALUE_OP() \
{\
   switch(var_type_stack.pop()) {\
   case svt_int:\
      sem_code.push_sem_instruction(sott_push_value_int_var);\
      break;\
   case svt_int_tmp:\
   case svt_int_const:\
      break;\
   case svt_int_aref:\
      sem_code.push_sem_instruction(sott_push_value_int_aref);\
      break;\
   case svt_int_aaref:\
      sem_code.push_sem_instruction(sott_push_value_int_aaref);\
      break;\
   default:\
      assert(0);\
   }\
}

#define STRING_OP(OP_NAME,OP_POSTFIX,STACK_END_OP,EXPAND_CODE) \
{\
   switch (var_type_stack.STACK_END_OP()) {\
   case svt_string:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var ## OP_POSTFIX);\
      EXPAND_CODE;\
      break;\
   case svt_string_tmp:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp ## OP_POSTFIX);\
      EXPAND_CODE;\
      break;\
   case svt_string_const:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const ## OP_POSTFIX);\
      EXPAND_CODE;\
      break;\
   case svt_string_aref:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref ## OP_POSTFIX);\
      EXPAND_CODE;\
      break;\
   case svt_string_aaref:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref ## OP_POSTFIX);\
      EXPAND_CODE;\
      break;\
   default:\
      assert(0);\
   }\
}

#define UNSIGNED_VAR_OP(OP_NAME,STACK_END_OP,EXPAND_CODE) \
{\
   switch (var_type_stack.STACK_END_OP()) {\
   case svt_int:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_var);\
      EXPAND_CODE;\
      break;\
   case svt_int_aref:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aref);\
      EXPAND_CODE;\
      break;\
   case svt_int_aaref:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aaref);\
      EXPAND_CODE;\
      break;\
   default:\
      assert(0);\
   }\
}

#define UNSIGNED_UNSIGNED_PUSH_TWO_VALUES_OP() \
{\
   switch (var_type_stack.pop()) {\
   case svt_int:\
      switch (var_type_stack.pop()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_var_int_var);\
	 break;\
      case svt_int_tmp:\
      case svt_int_const:\
	 sem_code.push_sem_instruction(sott_push_value_int_var);\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_aref_int_var);\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_aaref_int_var);\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_tmp:\
   case svt_int_const:\
      switch (var_type_stack.pop()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_var_int_value);\
	 break;\
      case svt_int_tmp:\
      case svt_int_const:\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_aref_int_value);\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_aaref_int_value);\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_aref:\
      switch (var_type_stack.pop()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_var_int_aref);\
	 break;\
      case svt_int_tmp:\
      case svt_int_const:\
	 sem_code.push_sem_instruction(sott_push_value_int_aref);\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_aref_int_aref);\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_aaref_int_aref);\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_aaref:\
      switch (var_type_stack.pop()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_var_int_aaref);\
	 break;\
      case svt_int_tmp:\
      case svt_int_const:\
	 sem_code.push_sem_instruction(sott_push_value_int_aaref); \
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_aref_int_aaref);\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_push_two_values_int_aaref_int_aaref);\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   default:\
      assert(0);\
   }\
}

#define STRING_STRING_OP(OP_NAME,STACK_END_OP,EXPAND_CODE) \
{\
   switch (var_type_stack.pop()) {\
   case svt_string:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_tmp:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_const);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_aaref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   default:\
      assert(0);\
   }\
}

#define ANY_VAR_SANY_OP(OP_NAME,STACK_END_OP,EXPAND_CODE) \
{\
   switch (var_type_stack.pop()) {\
   case svt_int:\
      switch(var_type_stack.STACK_END_OP()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_push_value_int_var);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_push_value_int_var);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_push_value_int_var);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aaref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 break;\
      }\
      break;\
   case svt_string:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_array_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_array_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_array:\
      if (var_type_stack.STACK_END_OP() == svt_int_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_int_array_array_var);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_string_array_array:\
      if (var_type_stack.STACK_END_OP() == svt_string_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_string_array_array_var);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_int_tmp:\
   case svt_int_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aaref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_tmp:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_tmp:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array_tmp:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_const);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_array_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_array_const);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_array_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_array_const);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_array_const:\
      if (var_type_stack.STACK_END_OP() == svt_int_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_int_array_array_const);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_string_array_array_const:\
      if (var_type_stack.STACK_END_OP() == svt_string_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_string_array_array_const);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_int_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_push_value_int_aref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_push_value_int_aref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_push_value_int_aref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aaref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_array_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_array_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_array_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_array_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_aaref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_push_value_int_aaref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_push_value_int_aaref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_push_value_int_aaref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aaref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_aaref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_blank_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_blank_array);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_blank_array);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_blank_array);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_blank_array);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_blank_array);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_blank_array);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   default:\
      assert(0);\
   }\
}

#define ANY_SANY_OP(OP_NAME,STACK_END_OP,EXPAND_CODE) \
{\
   switch (var_type_stack.pop()) {\
   case svt_int:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_push_value_int_var);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_tmp:\
      case svt_int_const:\
	 sem_code.push_sem_instruction(sott_push_value_int_var);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_value_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_push_value_int_var);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_push_value_int_var);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aaref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_tmp_int_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_const_int_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_array_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_tmp_string_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_const_string_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_array_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_int_array_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_const_int_array_array_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_string_array_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_const_string_array_array_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_tmp:\
   case svt_int_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_tmp:\
      case svt_int_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_value_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aaref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_tmp:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_tmp:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_tmp_int_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_const_int_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array_tmp:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_tmp_string_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_const_string_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_array_tmp);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_const);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_array_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_tmp_int_array_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_const_int_array_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_array_const);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_array_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_tmp_string_array_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_const_string_array_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_array_const);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_array_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_int_array_array_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_const_int_array_array_const);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array_array_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_string_array_array_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_const_string_array_array_const);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_push_value_int_aref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_tmp:\
      case svt_int_const:\
	 sem_code.push_sem_instruction(sott_push_value_int_aref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_value_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_push_value_int_aref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_push_value_int_aref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aaref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_array_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_tmp_int_array_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_const_int_array_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_array_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_array_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_tmp_string_array_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_const_string_array_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_array_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_aaref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int:\
	 sem_code.push_sem_instruction(sott_push_value_int_aaref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_tmp:\
      case svt_int_const:\
	 sem_code.push_sem_instruction(sott_push_value_int_aaref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_value_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aref:\
	 sem_code.push_sem_instruction(sott_push_value_int_aaref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_aaref:\
	 sem_code.push_sem_instruction(sott_push_value_int_aaref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_aaref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_aaref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_tmp:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_const:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   default:\
      assert(0);\
   }\
}

#define ANY_ARRAY_OP(OP_NAME,STACK_END_OP,EXPAND_CODE) \
{\
   switch (var_type_stack.STACK_END_OP()) {\
   case svt_string:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var);\
      EXPAND_CODE;\
      break;\
   case svt_int_array:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var);\
      EXPAND_CODE;\
      break;\
   case svt_string_array:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var);\
      EXPAND_CODE;\
      break;\
   case svt_int_array_array:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var);\
      EXPAND_CODE;\
      break;\
   case svt_string_array_array:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var);\
      EXPAND_CODE;\
      break;\
   case svt_string_tmp:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_tmp);\
      EXPAND_CODE;\
      break;\
   case svt_string_const:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_const);\
      EXPAND_CODE;\
      break;\
   case svt_string_aref:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref);\
      EXPAND_CODE;\
      break;\
   case svt_int_array_aref:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref);\
      EXPAND_CODE;\
      break;\
   case svt_string_array_aref:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref);\
      EXPAND_CODE;\
      break;\
   case svt_string_aaref:\
      sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref);\
      EXPAND_CODE;\
      break;\
   default:\
      assert(0);\
   }\
}

#define ANY_ARRAY_VAR_SANY_OP(OP_NAME,STACK_END_OP,EXPAND_CODE) \
{\
   switch (var_type_stack.pop()) {\
   case svt_int:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_push_value_int_var);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_push_value_int_var);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array:\
      if (var_type_stack.STACK_END_OP() == svt_int_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_int_array_var);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_string_array:\
      if (var_type_stack.STACK_END_OP() == svt_string_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_string_array_var);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_int_tmp:\
   case svt_int_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_tmp:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_tmp);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_tmp:\
      if (var_type_stack.STACK_END_OP() == svt_int_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_int_array_tmp);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_string_array_tmp:\
      if (var_type_stack.STACK_END_OP() == svt_string_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_string_array_tmp);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_string_const:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_const);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_const);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_const:\
      if (var_type_stack.STACK_END_OP() == svt_int_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_int_array_const);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_string_array_const:\
      if (var_type_stack.STACK_END_OP() == svt_string_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_string_array_const);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_int_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_push_value_int_aref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_push_value_int_aref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_aref:\
      if (var_type_stack.STACK_END_OP() == svt_int_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_int_array_aref);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_string_array_aref:\
      if (var_type_stack.STACK_END_OP() == svt_string_array_array) {\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_string_array_aref);\
	 EXPAND_CODE;\
      }\
      else {\
	 assert(0);\
      }\
      break;\
   case svt_int_aaref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_push_value_int_aaref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_value);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_push_value_int_aaref);\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_value);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_aaref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_blank_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_blank_array);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_blank_array);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   default:\
      assert(0);\
   }\
}

#define ANY_ARRAY_VAR_SANY_ARRAY_VAR_OP(OP_NAME,STACK_END_OP,EXPAND_CODE) \
{\
   switch (var_type_stack.pop()) {\
   case svt_string:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_array_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_array_var);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_array_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_array_var_int_array_array_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array_array:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_array_var_string_array_array_var);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_int_array_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_int_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_var_int_array_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_int_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _int_array_aref_int_array_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_array_aref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string_array:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_var_string_array_aref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_array_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_array_aref_string_array_aref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   case svt_string_aaref:\
      switch (var_type_stack.STACK_END_OP()) {\
      case svt_string:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_var_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aref_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      case svt_string_aaref:\
	 sem_code.push_sem_instruction(sott_ ## OP_NAME ## _string_aaref_string_aaref);\
	 EXPAND_CODE;\
	 break;\
      default:\
	 assert(0);\
      }\
      break;\
   default:\
      assert(0);\
   }\
}

// - zpracovani definice datoveho typu -
void p_semantic_parser_s::psi_stt_var_type_int(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   unsigned &var_type = sem_parse_data.var_type;

   var_type = svt_int;
}

void p_semantic_parser_s::psi_stt_var_type_string(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   unsigned &var_type = sem_parse_data.var_type;

   var_type = svt_string;
}

void p_semantic_parser_s::psi_stt_var_type_int_array(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   unsigned &var_type = sem_parse_data.var_type;

   var_type = svt_int_array;
}

void p_semantic_parser_s::psi_stt_var_type_string_array(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   unsigned &var_type = sem_parse_data.var_type;

   var_type = svt_string_array;
}

void p_semantic_parser_s::psi_stt_var_type_int_array_array(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   unsigned &var_type = sem_parse_data.var_type;

   var_type = svt_int_array_array;
}

void p_semantic_parser_s::psi_stt_var_type_string_array_array(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   unsigned &var_type = sem_parse_data.var_type;

   var_type = svt_string_array_array;
}

// - definovani nove promenne -
void p_semantic_parser_s::psi_stt_var_def(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   string_s &source_string = *((string_s *)sem_parse_data.source_string_ptr);
   p_lalr_stack_s &lalr_stack = sem_parse_data.lalr_stack;
   p_sem_variable_names_array_s &sem_variable_names_array = sem_parse_data.sem_variable_names_array;
   unsigned &var_type = sem_parse_data.var_type;

   p_lalr_stack_element_s &lalr_se = lalr_stack.last();
   unsigned str_length = lalr_se.ui_third - lalr_se.ui_second;
   char *str_data = source_string.data + lalr_se.ui_second;

   // - kontrola jestli uz tato promenna nebyla definovana -
   unsigned vt_idx = 0;
   bool not_exist = true;
   do {
      unsigned _idx = sem_variable_names_array[vt_idx].get_char_ptr_idx(str_length,str_data);
      if (_idx != c_idx_not_exist) {
	 not_exist = false;
	 break;
      }
   } while(++vt_idx < c_sem_var_type_cnt);
   
   // FX ERR
   if (!not_exist) {
      fprintf(stderr,"ERROR: p_semantic_parser_s::psi_stt_var_def: variable is already defined\n");
      assert(0);
   }

   // - ulozeni nazvu promenne do pole jejiho typu -
   p_sem_variable_names_s &sem_variable_names = sem_variable_names_array[var_type];
   sem_variable_names.push_blank();
   sem_variable_names.last().set(str_length,str_data);
}

// - definovani nove promenne s pripravou na prirazeni -
void p_semantic_parser_s::psi_stt_var_def_id(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   string_s &source_string = *((string_s *)sem_parse_data.source_string_ptr);
   p_lalr_stack_s &lalr_stack = sem_parse_data.lalr_stack;
   p_sem_variable_names_array_s &sem_variable_names_array = sem_parse_data.sem_variable_names_array;
   unsigned &var_type = sem_parse_data.var_type;
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   p_lalr_stack_element_s &lalr_se = lalr_stack[lalr_stack.used - 2];
   unsigned str_length = lalr_se.ui_third - lalr_se.ui_second;
   char *str_data = source_string.data + lalr_se.ui_second;

   // - kontrola jestli uz tato promenna nebyla definovana -
   unsigned vt_idx = 0;
   bool not_exist = true;
   do {
      unsigned _idx = sem_variable_names_array[vt_idx].get_char_ptr_idx(str_length,str_data);
      if (_idx != c_idx_not_exist) {
	 not_exist = false;
	 break;
      }
   } while(++vt_idx < c_sem_var_type_cnt);
   
   // FX ERR
   if (!not_exist) {
      fprintf(stderr,"ERROR: p_semantic_parser_s::psi_stt_var_def_id: variable is already defined\n");
      assert(0);
   }

   // - ulozeni nazvy promenne do pole jejiho typu -
   p_sem_variable_names_s &sem_variable_names = sem_variable_names_array[var_type];
   sem_variable_names.push_blank();
   sem_variable_names.last().set(str_length,str_data);

   // - vlozeni identifikatoru promenne na zasobnik a do kodu -
   sem_code.push_sem_instruction(sott_int_var + var_type);
   sem_code.push_unsigned(sem_variable_names.used - 1);
   var_type_stack.push(svt_int + var_type);
}

// - zpracovani instrukce pro vystup -
void p_semantic_parser_s::psi_stt_out(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   switch (var_type_stack.pop()) {
   case svt_int:
      sem_code.push_sem_instruction(sott_push_value_int_var);
      sem_code.push_sem_instruction(sott_out_int_value);
      break;
   case svt_string:
      sem_code.push_sem_instruction(sott_out_string_var);
      break;
   case svt_int_array:
      sem_code.push_sem_instruction(sott_out_int_array_var);
      break;
   case svt_string_array:
      sem_code.push_sem_instruction(sott_out_string_array_var);
      break;
   case svt_int_array_array:
      sem_code.push_sem_instruction(sott_out_int_array_array_var);
      break;
   case svt_string_array_array:
      sem_code.push_sem_instruction(sott_out_string_array_array_var);
      break;
   case svt_int_tmp:
   case svt_int_const:
      sem_code.push_sem_instruction(sott_out_int_value);
      break;
   case svt_string_tmp:
      sem_code.push_sem_instruction(sott_out_string_tmp);
      break;
   case svt_int_array_tmp:
      sem_code.push_sem_instruction(sott_out_int_array_tmp);
      break;
   case svt_string_array_tmp:
      sem_code.push_sem_instruction(sott_out_string_array_tmp);
      break;
   case svt_string_const:
      sem_code.push_sem_instruction(sott_out_string_const);
      break;
   case svt_int_aref:
      sem_code.push_sem_instruction(sott_push_value_int_aref);
      sem_code.push_sem_instruction(sott_out_int_value);
      break;
   case svt_string_aref:
      sem_code.push_sem_instruction(sott_out_string_aref);
      break;
   case svt_int_array_aref:
      sem_code.push_sem_instruction(sott_out_int_array_aref);
      break;
   case svt_string_array_aref:
      sem_code.push_sem_instruction(sott_out_string_array_aref);
      break;
   case svt_int_aaref:
      sem_code.push_sem_instruction(sott_push_value_int_aaref);
      sem_code.push_sem_instruction(sott_out_int_value);
      break;
   case svt_string_aaref:
      sem_code.push_sem_instruction(sott_out_string_aaref);
      break;
   default:
      assert(0);
   }
}

// - zpracovani podminky -
void p_semantic_parser_s::psi_stt_if_begin(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();

   sem_code.push_sem_instruction(sott_jump_if_zero);
   sem_code.push_unsigned(0);
   var_type_stack.push(sem_code.used - sizeof(unsigned));
}

void p_semantic_parser_s::psi_stt_if_end(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   *(unsigned *)(sem_code.data + var_type_stack.pop()) = sem_code.used;
}

void p_semantic_parser_s::psi_stt_if_else_end(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   *(unsigned *)(sem_code.data + var_type_stack.pop()) = sem_code.used;
}

void p_semantic_parser_s::psi_stt_if_else(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   sem_code.push_sem_instruction(sott_jump);
   sem_code.push_unsigned(0);
   *(unsigned *)(sem_code.data + var_type_stack.pop()) = sem_code.used;
   var_type_stack.push(sem_code.used - sizeof(unsigned));
}

// - zpracovani do while cyklu -
void p_semantic_parser_s::psi_stt_do_while_end(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();

   sem_code.push_sem_instruction(sott_jump_if_no_zero);
   sem_code.push_unsigned(var_type_stack.pop());
}

void p_semantic_parser_s::psi_stt_do_while_begin(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   var_type_stack.push(sem_code.used);
}

// - zpracovani ujisteni -
void p_semantic_parser_s::psi_stt_assert(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   string_s &source_string = *((string_s *)sem_parse_data.source_string_ptr);
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   p_lalr_stack_s &lalr_stack = sem_parse_data.lalr_stack;
   string_array_s &string_constants = sem_parse_data.string_constants;

   UNSIGNED_PUSH_VALUE_OP();

   // - BEWARE skaredy hack, musi sam kontrolovat vyskyt konstanty - 
   unsigned as_begin = lalr_stack.__tail(3).ui_second;
   unsigned as_length = lalr_stack.__tail(1).ui_third - as_begin;

   sem_code.push_sem_instruction(sott_string_const);

   unsigned c_idx = string_constants.get_char_ptr_idx(as_length,source_string.data + as_begin);
   if (c_idx != c_idx_not_exist) {
      sem_code.push_unsigned(c_idx);
   }
   else {
      string_constants.push_blank();
      string_constants.last().set(as_length,source_string.data + as_begin);
      sem_code.push_unsigned(string_constants.used - 1);
   }
   
   sem_code.push_sem_instruction(sott_assert_int_value_string_const);
}

// - ukonceni vyrazu -
void p_semantic_parser_s::psi_stt_exp_end(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   
   switch (var_type_stack.pop()) {
   case svt_int:
   case svt_string:
   case svt_int_array:
   case svt_string_array:
   case svt_int_array_array:
   case svt_string_array_array:
   case svt_int_tmp:
   case svt_int_const:
   case svt_string_const:
   case svt_int_array_const:
   case svt_string_array_const:
   case svt_int_array_array_const:
   case svt_string_array_array_const:
      sem_code.push_sem_instruction(sott_exp_end);
      break;
   case svt_string_tmp:
      sem_code.push_sem_instruction(sott_exp_end_string_tmp);
      break;
   case svt_int_array_tmp:
      sem_code.push_sem_instruction(sott_exp_end_int_array_tmp);
      break;
   case svt_string_array_tmp:
      sem_code.push_sem_instruction(sott_exp_end_string_array_tmp);
      break;
   case svt_int_aref:
   case svt_string_aref:
   case svt_int_array_aref:
   case svt_string_array_aref:
      sem_code.push_sem_instruction(sott_exp_end_aref);
      break;
   case svt_int_aaref:
   case svt_string_aaref:
      sem_code.push_sem_instruction(sott_exp_end_aaref);
      break;
   default:
      assert(0);
   }
}

// - zpracovani operace prirazeni -
void p_semantic_parser_s::psi_stt_set(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   ANY_VAR_SANY_OP(set,last,);
}

// - zpracovani operace zameny obsahu poli -
void p_semantic_parser_s::psi_stt_swap(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   ANY_ARRAY_VAR_SANY_ARRAY_VAR_OP(swap,last,);
}

// - zpracovani operace logicky and -
void p_semantic_parser_s::psi_stt_and(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   UNSIGNED_OP(logic_and,_int_value,pop,var_type_stack.push(svt_int_tmp));
}

// - zpraovani operace logicky or -
void p_semantic_parser_s::psi_stt_or(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   UNSIGNED_OP(logic_or,_int_value,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani operace compare -
void p_semantic_parser_s::psi_stt_compare(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   ANY_SANY_OP(compare,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani operace neg_compare
void p_semantic_parser_s::psi_stt_neg_compare(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   ANY_SANY_OP(compare,pop,
      sem_code.push_sem_instruction(sott_negation_int_value);
      var_type_stack.push(svt_int_tmp);
   );
}

// - zpracovani operace porovnani porovnani less -
void p_semantic_parser_s::psi_stt_less(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   UNSIGNED_OP(less,_int_value,pop,var_type_stack.push(svt_int_tmp));
}
	    
// - zpracovani operace porovnani porovnani greater -
void p_semantic_parser_s::psi_stt_greater(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   UNSIGNED_OP(greater,_int_value,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani operace porovnani porovnani less_equal -
void p_semantic_parser_s::psi_stt_less_equal(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   UNSIGNED_OP(less_equal,_int_value,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani operace porovnani porovnani greater_equal -
void p_semantic_parser_s::psi_stt_greater_equal(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   UNSIGNED_OP(greater_equal,_int_value,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani operace souctu -
void p_semantic_parser_s::psi_stt_add(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   UNSIGNED_OP(add,_int_value,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani operace odecteni -
void p_semantic_parser_s::psi_stt_sub(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   UNSIGNED_OP(sub,_int_value,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani konkatenace -
void p_semantic_parser_s::psi_stt_conc(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   STRING_STRING_OP(conc,pop,var_type_stack.push(svt_string_tmp));
}

// - zpracovani post incrementace -
void p_semantic_parser_s::psi_stt_post_inc(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_VAR_OP(post_inc,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani post decrementace -
void p_semantic_parser_s::psi_stt_post_dec(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_VAR_OP(post_dec,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani pre incrementace  -
void p_semantic_parser_s::psi_stt_pre_inc(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_VAR_OP(pre_inc,last,);
}

// - zpracovani pre decrementace -
void p_semantic_parser_s::psi_stt_pre_dec(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_VAR_OP(pre_dec,last,);
}

// - zpracovani inverze (zaporna hodnota) cisla -
void p_semantic_parser_s::psi_stt_invertion(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   sem_code.push_sem_instruction(sott_invertion_int_value);
   var_type_stack.push(svt_int_tmp);
}

// - zpracovani vyvoreni podretezce retezce -
void p_semantic_parser_s::psi_stt_substr(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_UNSIGNED_PUSH_TWO_VALUES_OP();
   
   switch (var_type_stack.pop()) {
   case svt_string:
      sem_code.push_sem_instruction(sott_substr_string_var_int_value_int_value);
      var_type_stack.push(svt_string_tmp);
      break;
   case svt_string_tmp:
      sem_code.push_sem_instruction(sott_substr_string_tmp_int_value_int_value);
      var_type_stack.push(svt_string_tmp);
      break;
   case svt_string_const:
      sem_code.push_sem_instruction(sott_substr_string_const_int_value_int_value);
      var_type_stack.push(svt_string_tmp);
      break;
   case svt_string_aref:
      sem_code.push_sem_instruction(sott_substr_string_aref_int_value_int_value);
      var_type_stack.push(svt_string_tmp);
      break;
   case svt_string_aaref:
      sem_code.push_sem_instruction(sott_substr_string_aaref_int_value_int_value);
      var_type_stack.push(svt_string_tmp);
      break;
   default:
      assert(0);
   }
}

// - zformatovani retezce podle parametru -
void p_semantic_parser_s::psi_stt_format(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   if (var_type_stack.pop() == svt_int_array_const) {
      switch (var_type_stack.pop()) {
      case svt_string:
	 sem_code.push_sem_instruction(sott_format_string_var_int_array_const);
	 var_type_stack.push(svt_string_tmp);
	 break;
      case svt_string_tmp:
	 sem_code.push_sem_instruction(sott_format_string_tmp_int_array_const);
	 var_type_stack.push(svt_string_tmp);
	 break;
      case svt_string_const:
	 sem_code.push_sem_instruction(sott_format_string_const_int_array_const);
	 var_type_stack.push(svt_string_tmp);
	 break;
      case svt_string_aref:
	 sem_code.push_sem_instruction(sott_format_string_aref_int_array_const);
	 var_type_stack.push(svt_string_tmp);
	 break;
      case svt_string_aaref:
	 sem_code.push_sem_instruction(sott_format_string_aaref_int_array_const);
	 var_type_stack.push(svt_string_tmp);
	 break;
      default:
	 assert(0);
      }
   }
   else {
      assert(0);
   }
}

// - zpracovani nacteni retezce elementu pravidla -
void p_semantic_parser_s::psi_stt_get_rb(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   sem_code.push_sem_instruction(sott_get_rb_int_value);
   var_type_stack.push(svt_string_tmp);
}

// - zpracovani instrukce vracejici cislo katualniho radku ve zdrojovem programu -
void p_semantic_parser_s::psi_stt_get_line_cnt(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   sem_code.push_sem_instruction(sott_get_line_cnt);
   var_type_stack.push(svt_int_tmp);
}

// - zpracovani konverze na unsigned -
void p_semantic_parser_s::psi_stt_convert_int(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   STRING_OP(convert_int,_int_value,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani konverze na string -
void p_semantic_parser_s::psi_stt_convert_string(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   sem_code.push_sem_instruction(sott_convert_string_int_value);
   var_type_stack.push(svt_string_tmp);
}

// - zpracovani identifikatoru -
void p_semantic_parser_s::psi_stt_id(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   string_s &source_string = *((string_s *)sem_parse_data.source_string_ptr);
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   p_lalr_stack_s &lalr_stack = sem_parse_data.lalr_stack;
   p_sem_variable_names_array_s &sem_variable_names_array = sem_parse_data.sem_variable_names_array;

   p_lalr_stack_element_s &lalr_se = lalr_stack.last();
   unsigned str_length = lalr_se.ui_third - lalr_se.ui_second;
   char *str_data = source_string.data + lalr_se.ui_second;

   // - kontrola jestli uz tato promenna nebyla definovana -
   unsigned vt_idx = 0;
   unsigned v_idx;

   do {
      v_idx = sem_variable_names_array[vt_idx].get_char_ptr_idx(str_length,str_data);
      if (v_idx != c_idx_not_exist) {
	 break;
      }
   } while(++vt_idx < c_sem_var_type_cnt);

   // FX ERR
   if (v_idx == c_idx_not_exist) {
      fprintf(stderr,"ERROR: p_semantic_parser_s::psi_stt_id: identifier is not defined\n");
      assert(0);
   }

   sem_code.push_sem_instruction(sott_int_var + vt_idx);
   sem_code.push_unsigned(v_idx);
   var_type_stack.push(svt_int + vt_idx);
}

// - zpracovani celociselne konstanty v oct formatu -
void p_semantic_parser_s::psi_stt_const_oct_int(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   string_s &source_string = *((string_s *)sem_parse_data.source_string_ptr);
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   p_lalr_stack_s &lalr_stack = sem_parse_data.lalr_stack;
   ui_array_s &int_constants = sem_parse_data.int_constants;

   p_lalr_stack_element_s &lalr_se = lalr_stack.last();
   char bu_char = source_string.data[lalr_se.ui_third];

   source_string.data[lalr_se.ui_third] = '\0';
   unsigned num = strtol(source_string.data + lalr_se.ui_second,(char **)NULL,8);
   source_string.data[lalr_se.ui_third] = bu_char;
   
   int_constants.push(num);
   var_type_stack.push(svt_int_const);
}

// - zpracovani celociselne konstanty -
void p_semantic_parser_s::psi_stt_const_dec_int(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   string_s &source_string = *((string_s *)sem_parse_data.source_string_ptr);
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   p_lalr_stack_s &lalr_stack = sem_parse_data.lalr_stack;
   ui_array_s &int_constants = sem_parse_data.int_constants;

   p_lalr_stack_element_s &lalr_se = lalr_stack.last();
   char bu_char = source_string.data[lalr_se.ui_third];

   source_string.data[lalr_se.ui_third] = '\0';
   unsigned num = atoi(source_string.data + lalr_se.ui_second);
   source_string.data[lalr_se.ui_third] = bu_char;

   int_constants.push(num);
   var_type_stack.push(svt_int_const);
}

// - zpr. celociselne konstanty v hex. zapisu -
void p_semantic_parser_s::psi_stt_const_hex_int(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   string_s &source_string = *((string_s *)sem_parse_data.source_string_ptr);
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   p_lalr_stack_s &lalr_stack = sem_parse_data.lalr_stack;
   ui_array_s &int_constants = sem_parse_data.int_constants;

   p_lalr_stack_element_s &lalr_se = lalr_stack.last();
   char bu_char = source_string.data[lalr_se.ui_third];

   source_string.data[lalr_se.ui_third] = '\0';
   unsigned num = strtol(source_string.data + lalr_se.ui_second,(char **)NULL,16);
   source_string.data[lalr_se.ui_third] = bu_char;

   int_constants.push(num);
   var_type_stack.push(svt_int_const);
}
	    
// - zpracovani celociselne konstanty ve znakovem zapisu -
void p_semantic_parser_s::psi_stt_const_char_int(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   string_s &source_string = *((string_s *)sem_parse_data.source_string_ptr);
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   p_lalr_stack_s &lalr_stack = sem_parse_data.lalr_stack;
   ui_array_s &int_constants = sem_parse_data.int_constants;

   p_lalr_stack_element_s &lalr_se = lalr_stack.last();
   char *char_ptr = source_string.data + lalr_se.ui_second + 1;
   unsigned num = 0;

   if (*char_ptr == '\\') {
      switch (*(++char_ptr)) {
      case '\'':
	 num = (unsigned)'\'';
	 break;
      case 't':
	 num = (unsigned)'\t';
	 break;
      case 'n':
	 num = (unsigned)'\n';
	 break;
      case '0':
	 num = (unsigned)'\0';
	 break;
      case '}':
	 num = (unsigned)'}';
	 break;
      case '\\':
	 num = (unsigned)'\\';
	 break;
      default:

         // FX ERR
         fprintf(stderr,"ERROR: p_semantic_parser_s::psi_stt_const_char_int: undefined escape sequence in character constant\n");
         assert(0);
      }
   }
   else {
      num = (unsigned)*char_ptr;
   }

   int_constants.push(num);
   var_type_stack.push(svt_int_const);
}

// - zpracovani retezcove konstanty -
void p_semantic_parser_s::psi_stt_const_str(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   string_s &source_string = *((string_s *)sem_parse_data.source_string_ptr);
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   p_lalr_stack_s &lalr_stack = sem_parse_data.lalr_stack;
   string_array_s &string_constants = sem_parse_data.string_constants;

   p_lalr_stack_element_s &lalr_se = lalr_stack.last();

   string_constants.push_blank();
   string_constants.last().set_format((lalr_se.ui_third - lalr_se.ui_second) - 2,source_string.data + lalr_se.ui_second + 1);
   var_type_stack.push(svt_string_const);
}

// - zpracovani definice prazdneho pole -
void p_semantic_parser_s::psi_stt_blank_array(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   var_type_stack.push(svt_blank_array);
}

// - zpracovani definice pole -
void p_semantic_parser_s::psi_stt_array_end(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   p_sem_const_array_types_s &sem_const_array_types = sem_parse_data.sem_const_array_types;

   var_type_stack.push(sem_const_array_types.pop());
}

// - zpracovani pocatku definice pole -
void p_semantic_parser_s::psi_stt_array_start(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   p_sem_const_array_types_s &sem_const_array_types = sem_parse_data.sem_const_array_types;

   sem_const_array_types.push(c_idx_not_exist);
}

// - zpracovani elementu pole -
void p_semantic_parser_s::psi_stt_array_element(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   p_sem_const_array_types_s &sem_const_array_types = sem_parse_data.sem_const_array_types;
   ui_array_s &int_constants = sem_parse_data.int_constants;
   string_array_s & string_constants = sem_parse_data.string_constants;
   ui_arrays_s &int_array_constants = sem_parse_data.int_array_constants;
   string_arrays_s &string_array_constants = sem_parse_data.string_array_constants;
   ui_arrays_array_s &int_array_array_constants = sem_parse_data.int_array_array_constants;
   string_arrays_array_s &string_array_array_constants = sem_parse_data.string_array_array_constants;

   unsigned &const_array_type = sem_const_array_types.last();
   unsigned array_element_type = var_type_stack.pop();

   switch (array_element_type) {
   case svt_int_const:
      if (const_array_type != svt_int_array_const) {
	 if (const_array_type == c_idx_not_exist) {
	    const_array_type = svt_int_array_const;
	    int_array_constants.push_blank();
	    int_array_constants.last().used = 0;
	 }
	 else {
	    assert(0);
	 }
      }

      int_array_constants.last().push(int_constants.pop());
      break;
   case svt_string_const:
      {
	 if (const_array_type != svt_string_array_const) {
	    if (const_array_type == c_idx_not_exist) {
	       const_array_type = svt_string_array_const;
	       string_array_constants.push_blank();
	       string_array_constants.last().used = 0;
	    }
	    else {
	       assert(0);
	    }
	 }
	 
	 string_array_s &string_array_constant = string_array_constants.last();

	 string_array_constant.push_blank();
	 string_array_constant.last().swap(string_constants.pop());
      }
      break;
   case svt_int_array_const:
      {
	 if (const_array_type != svt_int_array_array_const) {
	    if (const_array_type == c_idx_not_exist) {
	       const_array_type = svt_int_array_array_const;
	       int_array_array_constants.push_blank();
	       int_array_array_constants.last().used = 0;
	    }
	    else {
	       assert(0);
	    }
	 }

	 ui_arrays_s &int_array_array_constant = int_array_array_constants.last();

	 int_array_array_constant.push_blank();
	 int_array_array_constant.last().swap(int_array_constants.pop());
      }
      break;
   case svt_string_array_const:
      {
	 if (const_array_type != svt_string_array_array_const) {
	    if (const_array_type == c_idx_not_exist) {
	       const_array_type = svt_string_array_array_const;
	       string_array_array_constants.push_blank();
	       string_array_array_constants.last().used = 0;
	    }
	    else {
	       assert(0);
	    }
	 }

	 string_arrays_s &string_array_array_constant = string_array_array_constants.last();

	 string_array_array_constant.push_blank();
	 string_array_array_constant.last().swap(string_array_constants.pop());
      }
      break;
   default:
      assert(0);
   }
}

// - ziskani elementu pole -
void p_semantic_parser_s::psi_stt_get_element(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();

   switch (var_type_stack.pop()) {
   case svt_int_array:
      var_type_stack.push(svt_int_aref);
      break;
   case svt_string_array:
      var_type_stack.push(svt_string_aref);
      break;
   case svt_int_array_array:
      var_type_stack.push(svt_int_array_aref);
      break;
   case svt_string_array_array:
      var_type_stack.push(svt_string_array_aref);
      break;
   case svt_int_array_aref:
      var_type_stack.push(svt_int_aaref);
      break;
   case svt_string_array_aref:
      var_type_stack.push(svt_string_aaref);
      break;
   default:
      assert(0);
   }
}

// - zpracovani zjisteni velikosti -
void p_semantic_parser_s::psi_stt_size_of(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   ANY_ARRAY_OP(size_of,pop,var_type_stack.push(svt_int_tmp));
}
	    
// - zpracovani vlozeni prvku do pole -
void p_semantic_parser_s::psi_stt_push_to(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   ANY_ARRAY_VAR_SANY_OP(push_to,last,);
}

// - zpracovani vyjmuti prvku z pole -
void p_semantic_parser_s::psi_stt_pop_from(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   switch (var_type_stack.pop()) {
   case svt_int_array:
      sem_code.push_sem_instruction(sott_pop_from_int_array_var);
      var_type_stack.push(svt_int_tmp);
      break;
   case svt_string_array:
      sem_code.push_sem_instruction(sott_pop_from_string_array_var);
      var_type_stack.push(svt_string_tmp);
      break;
   case svt_int_array_array:
      sem_code.push_sem_instruction(sott_pop_from_int_array_array_var);
      var_type_stack.push(svt_int_array_tmp);
      break;
   case svt_string_array_array:
      sem_code.push_sem_instruction(sott_pop_from_string_array_array_var);
      var_type_stack.push(svt_string_array_tmp);
      break;
   case svt_int_array_aref:
      sem_code.push_sem_instruction(sott_pop_from_int_array_aref);
      var_type_stack.push(svt_int_tmp);
      break;
   case svt_string_array_aref:
      sem_code.push_sem_instruction(sott_pop_from_string_array_aref);
      var_type_stack.push(svt_string_tmp);
      break;
   default:
      assert(0);
   }
}

// - zpracovani odstraneni x prvku z vrcholu pole -
void p_semantic_parser_s::psi_stt_remove_from(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();

   switch (var_type_stack.last()) {
   case svt_int_array:
      sem_code.push_sem_instruction(sott_remove_from_int_array_var);
      break;
   case svt_string_array:
      sem_code.push_sem_instruction(sott_remove_from_string_array_var);
      break;
   case svt_int_array_array:
      sem_code.push_sem_instruction(sott_remove_from_int_array_array_var);
      break;
   case svt_string_array_array:
      sem_code.push_sem_instruction(sott_remove_from_string_array_array_var);
      break;
   case svt_int_array_aref:
      sem_code.push_sem_instruction(sott_remove_from_int_array_aref);
      break;
   case svt_string_array_aref:
      sem_code.push_sem_instruction(sott_remove_from_string_array_aref);
      break;
   default:
      assert(0);
   }
}

// - zpracovani ziskani posledniho elementu pole -
void p_semantic_parser_s::psi_stt_last_of(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   switch (var_type_stack.pop()) {
   case svt_int_array:
      sem_code.push_sem_instruction(sott_last_of_int_array_var);
      var_type_stack.push(svt_int_aref);
      break;
   case svt_string_array:
      sem_code.push_sem_instruction(sott_last_of_string_array_var);
      var_type_stack.push(svt_string_aref);
      break;
   case svt_int_array_array:
      sem_code.push_sem_instruction(sott_last_of_int_array_array_var);
      var_type_stack.push(svt_int_array_aref);
      break;
   case svt_string_array_array:
      sem_code.push_sem_instruction(sott_last_of_string_array_array_var);
      var_type_stack.push(svt_string_array_aref);
      break;
   case svt_int_array_aref:
      sem_code.push_sem_instruction(sott_last_of_int_array_aref);
      var_type_stack.push(svt_int_aaref);
      break;
   case svt_string_array_aref:
      sem_code.push_sem_instruction(sott_last_of_string_array_aref);
      var_type_stack.push(svt_string_aaref);
      break;
   default:
      assert(0);
   }
}

// - zpracovani ziskani elementu pole indexovaneho od konce -
void p_semantic_parser_s::psi_stt_tail_of(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   UNSIGNED_PUSH_VALUE_OP();
   switch (var_type_stack.pop()) {
   case svt_int_array:
      sem_code.push_sem_instruction(sott_tail_of_int_array_var_int_value);
      var_type_stack.push(svt_int_aref);
      break;
   case svt_string_array:
      sem_code.push_sem_instruction(sott_tail_of_string_array_var_int_value);
      var_type_stack.push(svt_string_aref);
      break;
   case svt_int_array_array:
      sem_code.push_sem_instruction(sott_tail_of_int_array_array_var_int_value);
      var_type_stack.push(svt_int_array_aref);
      break;
   case svt_string_array_array:
      sem_code.push_sem_instruction(sott_tail_of_string_array_array_var_int_value);
      var_type_stack.push(svt_string_array_aref);
      break;
   case svt_int_array_aref:
      sem_code.push_sem_instruction(sott_tail_of_int_array_aref_int_value);
      var_type_stack.push(svt_int_aaref);
      break;
   case svt_string_array_aref:
      sem_code.push_sem_instruction(sott_tail_of_string_array_aref_int_value);
      var_type_stack.push(svt_string_aaref);
      break;
   default:
      assert(0);
   }
}

// - zpracovani get_idx -
void p_semantic_parser_s::psi_stt_get_idx(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;

   ANY_ARRAY_VAR_SANY_OP(get_idx,pop,var_type_stack.push(svt_int_tmp));
}

// - zpracovani get_next_idx -
void p_semantic_parser_s::psi_stt_get_next_idx(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   //TODO nekdy dodelat :)
   printf("TODO - stt_next_idx\n");
   assert(0);
}

// - ulozeni konstanty do kodu -
void p_semantic_parser_s::psi_stt_const_apply(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   ui_array_s &var_type_stack = sem_parse_data.var_type_stack;
   ui_array_s &int_constants = sem_parse_data.int_constants;
   string_array_s & string_constants = sem_parse_data.string_constants;
   ui_arrays_s &int_array_constants = sem_parse_data.int_array_constants;
   string_arrays_s &string_array_constants = sem_parse_data.string_array_constants;
   ui_arrays_array_s &int_array_array_constants = sem_parse_data.int_array_array_constants;
   string_arrays_array_s &string_array_array_constants = sem_parse_data.string_array_array_constants;

   switch (var_type_stack.pop()) {
   case svt_int_const:
      {
	 sem_code.push_sem_instruction(sott_int_value);
	 sem_code.push_unsigned(int_constants.pop());
	 var_type_stack.push(svt_int_const);
      }
      break;
   case svt_string_const:
      {
	 sem_code.push_sem_instruction(sott_string_const);

	 unsigned c_idx = string_constants.get_idx(string_constants.last());
	 if (c_idx != string_constants.used - 1) {
	    string_constants.used--;
	    sem_code.push_unsigned(c_idx);
	 }
	 else {
	    sem_code.push_unsigned(string_constants.used - 1);
	 }

	 var_type_stack.push(svt_string_const);
      }
      break;
   case svt_int_array_const:
      {
	 sem_code.push_sem_instruction(sott_int_array_const);

	 unsigned c_idx = int_array_constants.get_idx(int_array_constants.last());
	 if (c_idx != int_array_constants.used - 1) {
	    int_array_constants.used--;
	    sem_code.push_unsigned(c_idx);
	 }
	 else {
	    sem_code.push_unsigned(int_array_constants.used - 1);
	 }

	 var_type_stack.push(svt_int_array_const);
      }
      break;
   case svt_string_array_const:
      {
	 sem_code.push_sem_instruction(sott_string_array_const);

	 unsigned c_idx = string_array_constants.get_idx(string_array_constants.last());
	 if (c_idx != string_array_constants.used - 1) {
	    string_array_constants.used--;
	    sem_code.push_unsigned(c_idx);
	 }
	 else {
	    sem_code.push_unsigned(string_array_constants.used - 1);
	 }

	 var_type_stack.push(svt_string_array_const);
      }
      break;
   case svt_int_array_array_const:
      {
	 sem_code.push_sem_instruction(sott_int_array_array_const);

	 unsigned c_idx = int_array_array_constants.get_idx(int_array_array_constants.last());
	 if (c_idx != int_array_array_constants.used - 1) {
	    int_array_array_constants.used--;
	    sem_code.push_unsigned(c_idx);
	 }
	 else {
	    sem_code.push_unsigned(int_array_array_constants.used - 1);
	 }

	 var_type_stack.push(svt_int_array_array_const);
      }
      break;
   case svt_string_array_array_const:
      {
	 sem_code.push_sem_instruction(sott_string_array_array_const);

	 unsigned c_idx = string_array_array_constants.get_idx(string_array_array_constants.last());
	 if (c_idx != string_array_array_constants.used - 1) {
	    string_array_array_constants.used--;
	    sem_code.push_unsigned(c_idx);
	 }
	 else {
	    sem_code.push_unsigned(string_array_array_constants.used - 1);
	 }

	 var_type_stack.push(svt_string_array_array_const);
      }
      break;
   case svt_blank_array:
      var_type_stack.push(svt_blank_array);
      break;
   default:
      assert(0);
   }
}

