
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

// TODO swap instrukce '<=>' prekladace semantiky

// --- definitions ---

/*
 * direktivy popisujici zda se konecny automat rozpoznavajici soubor
 * popisujici prekladac, a semanticky prekladac maji znovu vytvorit, nebo nacist ze
 * souboru.
 */
#define p_creat_descr_s_load_final_automata_set_CREATE 1
#define p_semantic_parser_s_load_or_create_CREATE 1

/*
 * terminaly nacitane ze souboru popisujiciho jazyk
 */
enum {
   rt_space = 0,
   rt_comment,
   rt_sec_comment,
   rt_input_end,
   rt_init_code,
   rt_terminals,
   rt_nonterminals,
   rt_rules,
   rt_div_0,
   rt_div_1,
   rt_identifier,
   rt_nonterm_id,
   rt_code
};

/*
 * regularni vyrazy popisujici terminaly v souboru pravidel
 */
const unsigned test_reg_exp_cnt = 13;
const char *test_reg_exps[test_reg_exp_cnt] = {
   "w.w*",//"\\w\\w\\*",
   "'$'.!'\\n'*.'\\n'",//"$\\!\\n\\*\\n",
   "'#'.!'\\n'*.'\\n'",//"#\\!\\n\\*\\n",
   "'\\0'",//"\\0",
   "\"init_code:\"",//"init_code:",
   "\"terminals:\"",//"terminals:",
   "\"nonterminals:\"",//"nonterminals:",
   "\"rules:\"",//"rules:",
   "\"->\"",//"->",
   "\"->>\"",//"->>",
   "('_'+l).('_'+l+d)*",//"\\[_\\l\\]\\[_\\l\\d\\]\\*",
   "'<'.('_'+l).('_'+l+d)*.'>'",//"<\\[_\\l\\]\\[_\\l\\d\\]\\*>",
   "'{'.((|\\\\}|)+('\\\\'.!'\\0'))*.'}'"//"'{'.!'}'*.'}'"//"{\\!}\\*}"
};

/*
 * indexy terminalu definujicich specialni funkcnost v prekladaci
 * ktt_skip - ignorovane prekladacem
 * ktt_end - terminal ukoncujici rozklad
 */
enum {
   ktt_skip,
   ktt_end
};

/*
 * retezce popisujici terminaly s vyznamem pro prekladac
 * ingnorovanych terminalu by melo byt mozne definovat vic, to je umozneno
 * zapisem libovolneho poctu podtrzitek pred a za slovem SKIP
 */
const unsigned c_key_terminal_cnt = 2;
const char *key_terminals[c_key_terminal_cnt] = {
   "'_'*.\"SKIP\".'_'*",//"_\\*SKIP_\\*",
   "\"_END_\""//"_END_"
};

/*
 * c_lalr_table_reduce_base - popisuje hranici za niz se jedna o akci REDUCE 
 * (zaznamy v tabulce maji typ unsigned)
 * c_chars_needed_for_number_string - hodnota popisujici s kolika znaky se ma pocitat
 * pri nacitani zapisu cisla ze vstupu
 */
const unsigned c_lalr_table_reduce_base = 0x80000000;
const unsigned c_chars_needed_for_number_string = 16;

/* 
 * retezec popisujici semanticky prekladac
 * zapisuje se stejnym zpusobem jako soubor definujici jazyk, s tim rozdilem ze 
 * se misto semantickeho kodu pouziva pouze nazev jedne instrukce, nebo
 * prazdny kod
 */
const char *semantic_parser_string = 
"init_code: {}\n"

"terminals:\n"
"  id			   {('_'+l).('_'+l+d)*}\n"//{\\[_\\l\\]\\[_\\l\\d\\]\\*}\n"

"  oct_int		   {'0'.<07>.<07>*}\n"//{0\\[01234567\\]\\[01234567\\]\\*}\n"
"  dec_int		   {d.d*}\n"//{\\d\\d\\*}\n"
"  hex_int		   {\"0x\".(d+<af>)*}\n"//{0x\\[\\dabcdef\\]\\*}\n"
"  char_int		   {'\\''.(|\\\\\'|+('\\\\'.!'\\0')).'\\''}\n"//{'\\''.!'\\''.'\\''}\n"//{'\\!''}\n"
"  str			   {'\"'.(|\\\\\"|+('\\\\'.!'\\0'))*.'\"'}\n"//{'\"'.!'\"'*.'\"'}\n"//{\"\\!\"\\*\"}\n"

"  semicolon		   {';'}\n"//{;}\n"
"  comma		   {','}\n"//{,}\n"
"  dot			   {'.'}\n"//{.}\n"
"  le_br		   {'['}\n"//{[}\n"
"  re_br		   {']'}\n"//{]}\n"
"  lr_br		   {'('}\n"//{(}\n"
"  rr_br		   {')'}\n"//{)}\n"
"  ls_br		   {'<'}\n"//{<}\n"
"  rs_br		   {'>'}\n"//{>}\n"
"  empty		   {\"empty\"}\n"//{empty}\n"
"  array		   {\"array\"}\n"//{array}\n"
"  size			   {\"size\"}\n"//{size}\n"
"  push			   {\"push\"}\n"//{push}\n"
"  pop			   {\"pop\"}\n"//{pop}\n"
"  remove		   {\"remove\"}\n"//{remove}\n"
"  last			   {\"last\"}\n"//{last}\n"
"  tail			   {\"tail\"}\n"//{tail}\n"
"  get_idx		   {\"get_idx\"}\n"//{get_idx}\n"
"  get_next_idx		   {\"get_next_idx\"}\n"//{get_next_idx}\n"

"  null			   {\"null\"}\n"//{null}\n"

"  int			   {\"int\"}\n"//{int}\n"
"  string		   {\"string\"}\n"//{string}\n"
"  int_array		   {\"int[]\"}\n"//{int[]}\n"
"  string_array		   {\"string[]\"}\n"//{string[]}\n"
"  int_array_array	   {\"int[][]\"}\n"//{int[][]}\n"
"  string_array_array	   {\"string[][]\"}\n"//{string[][]}\n"

"  out			   {\"out\"}\n"//{out}\n"

"  if			   {\"if\"}\n"//{if}\n"
"  fi			   {\"fi\"}\n"//{fi}\n"
"  else			   {\"else\"}\n"//{else}\n"
"  do			   {\"do\"}\n"//{do}\n"
"  while		   {\"while\"}\n"//{while}\n"
"  assert		   {\"assert\"}\n"//{assert}\n"

"  set			   {'='}\n"//{=}\n"
"  swap			   {\"<=>\"}\n"//{<=>}\n"
"  and			   {\"and\"}\n"//{and}\n"
"  or			   {\"or\"}\n"//{or}\n"

"  equal		   {\"==\"}\n"//{==}\n"
"  not_equal		   {\"!=\"}\n"//{!=}\n"
"  less			   {'<'}\n"//{<}\n"
"  greater		   {'>'}\n"//{>}\n"
"  less_equal		   {\"<=\"}\n"//{<=}\n"
"  greater_equal	   {\">=\"}\n"//{>=}\n"

"  add			   {'+'}\n"//{+}\n"
"  sub			   {'-'}\n"//{-}\n"
"  increment		   {\"++\"}\n"//{++}\n"
"  decrement		   {\"--\"}\n"//{--}\n"

"  substr		   {\"substr\"}\n"//{substr}\n"
"  format		   {\"format\"}\n"//{format}\n"

"  to_int		   {\"to_int\"}\n"//{to_int}\n"
"  to_string		   {\"to_string\"}\n"//{to_string}\n"

"  rule_body		   {\"rule_body\"}\n"//{rule_body}\n"
"  line_cnt		   {\"line_cnt\"}\n"//{line_cnt}\n"

"  _SKIP_		   {w.w*}\n"//{\\w\\w\\*}\n"
"  _SKIP__		   {'$'.!'\\n'*.'\\n'}\n"//{$\\!\\n\\*\\n}\n"
"  _SKIP___		   {'#'.!'\\n'*.'\\n'}\n"//{#\\!\\n\\*\\n}\n"
"  _END_		   {'\\0'}\n"//{\\0}\n"

"nonterminals:\n"
"  <start>\n"
"  <cmd_list>\n"
"  <cmd>\n"

"  <var_type>\n"
"  <var_list>\n"
"  <var_def>\n"
"  <var_def_set>\n"
"  <var_def_id>\n"

"  <out_list>\n"

"  <if_cond>\n"
"  <if_cont>\n"
"  <if_else>\n"
"  <do>\n"

"  <exp>\n"
"  <D>\n"
"  <C>\n"
"  <B>\n"
"  <A>\n"
"  <array_start>\n"
"  <array_elements>\n"

"  <const>\n"

"rules:\n"
"  <start> ->		<cmd_list> _END_ ->>		    {}\n"
"  <cmd_list> ->	<cmd_list> <cmd> ->>		    {}\n"
"  <cmd_list> ->	<cmd> ->>			    {}\n"
"  <cmd> ->		null ->>			    {}\n"

"  <cmd> ->		<var_type> <var_list> semicolon ->> {}\n"
"  <var_type> ->	int ->>				    {var_type_int}\n"
"  <var_type> ->	string ->>			    {var_type_string}\n"
"  <var_type> ->	int_array ->>			    {var_type_int_array}\n"
"  <var_type> ->	string_array ->>		    {var_type_string_array}\n"
"  <var_type> ->	int_array_array ->>		    {var_type_int_array_array}\n"
"  <var_type> ->	string_array_array ->>		    {var_type_string_array_array}\n"

"  <var_list> ->	<var_list> comma <var_def> ->>	    {}\n"
"  <var_list> ->	<var_def> ->>			    {}\n"
"  <var_def> ->		id ->>				    {var_def}\n"
"  <var_def> ->		<var_def_set> ->>		    {exp_end}\n"
"  <var_def_set> ->	<var_def_id> <exp> ->>		    {set}\n"
"  <var_def_id> ->	id set ->>			    {var_def_id}\n"

"  <cmd> ->		out lr_br <out_list> rr_br semicolon ->>  {}\n"
"  <out_list> ->	<out_list> comma <exp> ->>		  {out}\n"
"  <out_list> ->	<exp> ->>				  {out}"

"  <cmd> ->		<if_cond> <cmd_list> <if_cont> ->>  {}\n"
"  <if_cond> ->		if <exp> ->>			    {if_begin}\n"
"  <if_cont> ->		fi ->>				    {if_end}\n"
"  <if_cont> ->		<if_else> <cmd_list> fi ->>	    {if_else_end}\n"
"  <if_else> ->		else ->>			    {if_else}\n"

"  <cmd> ->		<do> <cmd_list> while <exp> semicolon ->> {do_while_end}\n"
"  <do> ->		do ->>					  {do_while_begin}\n"

"  <cmd> ->		assert lr_br <exp> rr_br semicolon ->>	  {assert}\n"

"  <cmd> ->		<exp> semicolon ->>		    {exp_end}\n"

"  <exp> ->		<D> ->>				    {}\n"

"  <D> ->		<D> set <D> ->>			    {set}\n"
"  <D> ->		<D> swap <D> ->>		    {swap}\n"
"  <D> ->		<D> and <D> ->>			    {and}\n"
"  <D> ->		<D> or <D> ->>			    {or}\n"
"  <D> ->		<C> ->>				    {}\n"

"  <C> ->		<C> equal <C> ->>		    {compare}\n"
"  <C> ->		<C> not_equal <C> ->>		    {neg_compare}\n"
"  <C> ->		<C> less <C> ->>		    {less}\n"
"  <C> ->		<C> greater <C> ->>		    {greater}\n"
"  <C> ->		<C> less_equal <C> ->>		    {less_equal}\n"
"  <C> ->		<C> greater_equal <C> ->>	    {greater_equal}\n"
"  <C> ->		<B> ->>				    {}\n"

"  <B> ->		<B> add <B> ->>			    {add}\n"
"  <B> ->		<B> sub <B> ->>			    {sub}\n"

"  <B> ->		<B> dot <B> ->>			    {conc}\n"
"  <B> ->		<A> ->>				    {}\n"

"  <A> ->		<A> increment ->>		    {post_inc}\n"
"  <A> ->		<A> decrement ->>		    {post_dec}\n"
"  <A> ->		increment <A> ->>		    {pre_inc}\n"
"  <A> ->		decrement <A> ->>		    {pre_dec}\n"
"  <A> ->		sub <A> ->>			    {invertion}\n"

"  <A> ->		substr lr_br <D> comma <D> comma <D> rr_br ->>	{substr}\n"
"  <A> ->		format lr_br <D> comma <D> rr_br ->>		{format}\n"

"  <A> ->		rule_body lr_br <D> rr_br ->>		  {get_rb}\n"
"  <A> ->		line_cnt lr_br rr_br ->>		  {get_line_cnt}\n"

"  <A> ->		to_int lr_br <D> comma <D> rr_br ->>   {convert_int}\n"
"  <A> ->		to_string lr_br <D> rr_br ->>	       {convert_string}\n"

"  <A> ->		id ->>				 {id}\n"
"  <const> ->		oct_int ->>			 {const_oct_int}\n"
"  <const> ->		dec_int ->>			 {const_dec_int}\n"
"  <const> ->		hex_int ->>			 {const_hex_int}\n"
"  <const> ->		char_int ->>			 {const_char_int}\n"
"  <const> ->		str ->>				 {const_str}\n"

"  <const> ->		empty lr_br rr_br ->>		 {blank_array}\n"

"  <const> ->		<array_start> <array_elements> rr_br ->>  {array_end}\n"
"  <array_start> ->	array lr_br ->>				  {array_start}\n"
"  <array_elements> ->	<array_elements> comma <const> ->>	  {array_element}\n"
"  <array_elements> ->	<const> ->>				  {array_element}\n"

"  <A> ->		<A> le_br <D> re_br ->>				      {get_element}\n"
"  <A> ->		size lr_br <D> rr_br ->>			      {size_of}\n"
"  <A> ->		push lr_br <D> comma <D> rr_br ->>		      {push_to}\n"
"  <A> ->		pop lr_br <D> rr_br ->>				      {pop_from}\n"
"  <A> ->		remove lr_br <D> comma <D> rr_br ->>		      {remove_from}\n"
"  <A> ->		<A> last ->>					      {last_of}\n"
"  <A> ->		<A> tail le_br <D> re_br ->>			      {tail_of}\n"
"  <A> ->		get_idx lr_br <D> comma <D> rr_br ->>		      {get_idx}\n"
"  <A> ->		get_next_idx lr_br <D> comma <D> comma <D> rr_br ->>  {get_next_idx}\n"

"  <A> ->		lr_br <D> rr_br ->>		    {}\n"
"  <A> ->		<const> ->>			    {const_apply}\n"

"\n";

/* 
 * pocet typu promenych semantickych podprogramu
 * patri mezi ne:
 *    int
 *    string
 *    int_array
 *    string_array
 *    int_array_array
 *    string_array_array
 */
const unsigned c_sem_var_type_cnt = 6;

/*
 * identifikatory typu semantickych promennych, s rozlisenim
 * podle jejich reprezentace tj. 
 * bez_pripony - variable
 * "_tmp" - docasna promenna na zasobniku (u slozitejsich typu, se ukladaji do zasobniku za promenne)
 * "_const" - jsou zapsane v retezci bytoveho kodu
 * "_aref" - reference na tento typ promenne do pole techto typu 
 *	    (tj. na zasobniku je zase nejaka forma pole a celociselna hodnota reprezentujici index v tomto poli)
 * "_aaref" - reference na promennou do pole poli techto typu
 *	    (podone jako nahore s dvemi celociselnymi hodnotami)
 *
 * spec. svt_blank_array - prazdne pole jakehokoli typu.
 */
enum {
   svt_int = 0,
   svt_string,
   svt_int_array,
   svt_string_array,
   svt_int_array_array,
   svt_string_array_array,

   svt_int_tmp,
   svt_string_tmp,
   svt_int_array_tmp,
   svt_string_array_tmp,

   svt_int_const,
   svt_string_const,
   svt_int_array_const,
   svt_string_array_const,
   svt_int_array_array_const,
   svt_string_array_array_const,

   svt_int_aref,
   svt_string_aref,
   svt_int_array_aref,
   svt_string_array_aref,

   svt_int_aaref,
   svt_string_aaref,

   svt_blank_array
};

/*
 * retezce popisujici terminaly semantickych pravidel
 * slouzi k rozpoznani instrukci v souboru popisujicim semanticky jazyk
 */
const unsigned c_sem_terminal_cnt = 61;
const char *sem_terminals[c_sem_terminal_cnt] = {
   "var_type_int",
   "var_type_string",
   "var_type_int_array",
   "var_type_string_array",
   "var_type_int_array_array",
   "var_type_string_array_array",
   "var_def",
   "var_def_id",

   "out",

   "if_begin",
   "if_end",
   "if_else_end",
   "if_else",
   "do_while_end",
   "do_while_begin",
   "assert",

   "exp_end",
   "set",
   "swap",
   "and",
   "or",
   "compare",
   "neg_compare",
   "less",
   "greater",
   "less_equal",
   "greater_equal",
   "add",
   "sub",
   "conc",
   "post_inc",
   "post_dec",
   "pre_inc",
   "pre_dec",
   "invertion",
   
   "substr",
   "format",

   "get_rb",
   "get_line_cnt",

   "convert_int",
   "convert_string",

   "id",

   "const_oct_int",
   "const_dec_int",
   "const_hex_int",
   "const_char_int",
   "const_str",

   "blank_array",
   "array_end",
   "array_start",
   "array_element",

   "get_element",
   "size_of",
   "push_to",
   "pop_from",
   "remove_from",
   "last_of",
   "tail_of",
   "get_idx",
   "get_next_idx",

   "const_apply"
};

/*
 * odkazy na funkce implementujici semanticke podprogramy, prekladace semantickych podprogramu
 * odpovidaji vyse uvedenym jmenum
 */

void(*parse_sem_instr_callers[c_sem_terminal_cnt])(buffer_s&,p_sem_parse_data_s&) = {
   p_semantic_parser_s::psi_stt_var_type_int,
   p_semantic_parser_s::psi_stt_var_type_string,
   p_semantic_parser_s::psi_stt_var_type_int_array,
   p_semantic_parser_s::psi_stt_var_type_string_array,
   p_semantic_parser_s::psi_stt_var_type_int_array_array,
   p_semantic_parser_s::psi_stt_var_type_string_array_array,
   p_semantic_parser_s::psi_stt_var_def,
   p_semantic_parser_s::psi_stt_var_def_id,

   p_semantic_parser_s::psi_stt_out,

   p_semantic_parser_s::psi_stt_if_begin,
   p_semantic_parser_s::psi_stt_if_end,
   p_semantic_parser_s::psi_stt_if_else_end,
   p_semantic_parser_s::psi_stt_if_else,
   p_semantic_parser_s::psi_stt_do_while_end,
   p_semantic_parser_s::psi_stt_do_while_begin,
   p_semantic_parser_s::psi_stt_assert,

   p_semantic_parser_s::psi_stt_exp_end,
   p_semantic_parser_s::psi_stt_set,
   p_semantic_parser_s::psi_stt_swap,
   p_semantic_parser_s::psi_stt_and,
   p_semantic_parser_s::psi_stt_or,
   p_semantic_parser_s::psi_stt_compare,
   p_semantic_parser_s::psi_stt_neg_compare,
   p_semantic_parser_s::psi_stt_less,
   p_semantic_parser_s::psi_stt_greater,
   p_semantic_parser_s::psi_stt_less_equal,
   p_semantic_parser_s::psi_stt_greater_equal,
   p_semantic_parser_s::psi_stt_add,
   p_semantic_parser_s::psi_stt_sub,
   p_semantic_parser_s::psi_stt_conc,
   p_semantic_parser_s::psi_stt_post_inc,
   p_semantic_parser_s::psi_stt_post_dec,
   p_semantic_parser_s::psi_stt_pre_inc,
   p_semantic_parser_s::psi_stt_pre_dec,
   p_semantic_parser_s::psi_stt_invertion,

   p_semantic_parser_s::psi_stt_substr,
   p_semantic_parser_s::psi_stt_format,

   p_semantic_parser_s::psi_stt_get_rb,
   p_semantic_parser_s::psi_stt_get_line_cnt,

   p_semantic_parser_s::psi_stt_convert_int,
   p_semantic_parser_s::psi_stt_convert_string,

   p_semantic_parser_s::psi_stt_id,
   p_semantic_parser_s::psi_stt_const_oct_int,
   p_semantic_parser_s::psi_stt_const_dec_int,
   p_semantic_parser_s::psi_stt_const_hex_int,
   p_semantic_parser_s::psi_stt_const_char_int,
   p_semantic_parser_s::psi_stt_const_str,

   p_semantic_parser_s::psi_stt_blank_array,
   p_semantic_parser_s::psi_stt_array_end,
   p_semantic_parser_s::psi_stt_array_start,
   p_semantic_parser_s::psi_stt_array_element,

   p_semantic_parser_s::psi_stt_get_element,
   p_semantic_parser_s::psi_stt_size_of,
   p_semantic_parser_s::psi_stt_push_to,
   p_semantic_parser_s::psi_stt_pop_from,
   p_semantic_parser_s::psi_stt_remove_from,
   p_semantic_parser_s::psi_stt_last_of,
   p_semantic_parser_s::psi_stt_tail_of,
   p_semantic_parser_s::psi_stt_get_idx,
   p_semantic_parser_s::psi_stt_get_next_idx,

   p_semantic_parser_s::psi_stt_const_apply
};

/*
 * indentifikatory vystupnich terminalu semantickeho prekladace (instrukci)
 * jedna se o stejne instrukce semantickeho jazyka, ktere byly popsany vyse, s
 * tim rozdilem ze u techto je uz znamo s jakymi dat. typy budou operovat
 * z duvodu zrychleni vypoctu. (nehorazne naroste pocet instrukci)
 *
 * konvence "sott_" - semantic otput terminal type
 * typ instrukce
 * vycet typu s kterymi se bude pracovat(kod realizujici instrukci pote vi kde najit data, nemusi se zjistovat typ
 *    to se provede uz pri prekladu semantickeho programu)
 */
enum {
   
   // - vypis obsahu na stdout -
   sott_out_string_var = 0,
   sott_out_int_array_var,
   sott_out_string_array_var,
   sott_out_int_array_array_var,
   sott_out_string_array_array_var,
   sott_out_int_value,
   sott_out_string_tmp,
   sott_out_int_array_tmp,
   sott_out_string_array_tmp,
   sott_out_string_const,
   sott_out_string_aref,
   sott_out_int_array_aref,
   sott_out_string_array_aref,
   sott_out_string_aaref,

   // - instrukce skoku -
   sott_jump,
   sott_jump_if_zero,
   sott_jump_if_no_zero,

   // - instrukce testovani spravnosti -
   sott_assert_int_value_string_const,

   // - odstraneni docasnych promennych ze zasobniku po vypoctu -
   sott_exp_end,
   sott_exp_end_string_tmp,
   sott_exp_end_int_array_tmp,
   sott_exp_end_string_array_tmp,
   sott_exp_end_aref,
   sott_exp_end_aaref,

   // - prirazeni -
   sott_set_int_var_int_value,
   sott_set_string_var_string_var,
   sott_set_string_var_string_tmp,
   sott_set_string_var_string_const,
   sott_set_string_var_string_aref,
   sott_set_string_var_string_aaref,
   sott_set_int_array_var_int_array_var,
   sott_set_int_array_var_int_array_tmp,
   sott_set_int_array_var_int_array_const,
   sott_set_int_array_var_int_array_aref,
   sott_set_int_array_var_blank_array,
   sott_set_string_array_var_string_array_var,
   sott_set_string_array_var_string_array_tmp,
   sott_set_string_array_var_string_array_const,
   sott_set_string_array_var_string_array_aref,
   sott_set_string_array_var_blank_array,
   sott_set_int_array_array_var_int_array_array_var,
   sott_set_int_array_array_var_int_array_array_const,
   sott_set_int_array_array_var_blank_array,
   sott_set_string_array_array_var_string_array_array_var,
   sott_set_string_array_array_var_string_array_array_const,
   sott_set_string_array_array_var_blank_array,
   sott_set_int_aref_int_value,
   sott_set_string_aref_string_var,
   sott_set_string_aref_string_tmp,
   sott_set_string_aref_string_const,
   sott_set_string_aref_string_aref,
   sott_set_string_aref_string_aaref,
   sott_set_int_array_aref_int_array_var,
   sott_set_int_array_aref_int_array_tmp,
   sott_set_int_array_aref_int_array_const,
   sott_set_int_array_aref_int_array_aref,
   sott_set_int_array_aref_blank_array,
   sott_set_string_array_aref_string_array_var,
   sott_set_string_array_aref_string_array_tmp,
   sott_set_string_array_aref_string_array_const,
   sott_set_string_array_aref_string_array_aref,
   sott_set_string_array_aref_blank_array,
   sott_set_int_aaref_int_value,
   sott_set_string_aaref_string_var,
   sott_set_string_aaref_string_tmp,
   sott_set_string_aaref_string_const,
   sott_set_string_aaref_string_aref,
   sott_set_string_aaref_string_aaref,

   // - prohozeni hodnot poli a retezcu -
   sott_swap_string_var_string_var,
   sott_swap_string_var_string_aref,
   sott_swap_string_var_string_aaref,
   sott_swap_string_aref_string_var,
   sott_swap_string_aref_string_aref,
   sott_swap_string_aref_string_aaref,
   sott_swap_string_aaref_string_var,
   sott_swap_string_aaref_string_aref,
   sott_swap_string_aaref_string_aaref,
   sott_swap_int_array_var_int_array_var,
   sott_swap_int_array_var_int_array_aref,
   sott_swap_int_array_aref_int_array_var,
   sott_swap_int_array_aref_int_array_aref,
   sott_swap_string_array_var_string_array_var,
   sott_swap_string_array_var_string_array_aref,
   sott_swap_string_array_aref_string_array_var,
   sott_swap_string_array_aref_string_array_aref,
   sott_swap_int_array_array_var_int_array_array_var,
   sott_swap_string_array_array_var_string_array_array_var,

   // - logicky and -
   sott_logic_and_int_var_int_value,
   sott_logic_and_int_value_int_value,
   sott_logic_and_int_aref_int_value,
   sott_logic_and_int_aaref_int_value,

   // - logicky or -
   sott_logic_or_int_var_int_value,
   sott_logic_or_int_value_int_value,
   sott_logic_or_int_aref_int_value,
   sott_logic_or_int_aaref_int_value,

   // - porovani hodnot (vysledek je int 0 nebo 1) -
   sott_compare_int_var_int_value,
   sott_compare_string_var_string_var,
   sott_compare_string_var_string_tmp,
   sott_compare_string_var_string_const,
   sott_compare_string_var_string_aref,
   sott_compare_string_var_string_aaref,
   sott_compare_int_array_var_int_array_var,
   sott_compare_int_array_var_int_array_tmp,
   sott_compare_int_array_var_int_array_const,
   sott_compare_int_array_var_int_array_aref,
   sott_compare_string_array_var_string_array_var,
   sott_compare_string_array_var_string_array_tmp,
   sott_compare_string_array_var_string_array_const,
   sott_compare_string_array_var_string_array_aref,
   sott_compare_int_array_array_var_int_array_array_var,
   sott_compare_int_array_array_var_int_array_array_const,
   sott_compare_string_array_array_var_string_array_array_var,
   sott_compare_string_array_array_var_string_array_array_const,
   sott_compare_int_value_int_value,
   sott_compare_string_tmp_string_var,
   sott_compare_string_tmp_string_tmp,
   sott_compare_string_tmp_string_const,
   sott_compare_string_tmp_string_aref,
   sott_compare_string_tmp_string_aaref,
   sott_compare_int_array_tmp_int_array_var,
   sott_compare_int_array_tmp_int_array_tmp,
   sott_compare_int_array_tmp_int_array_const,
   sott_compare_int_array_tmp_int_array_aref,
   sott_compare_string_array_tmp_string_array_var,
   sott_compare_string_array_tmp_string_array_tmp,
   sott_compare_string_array_tmp_string_array_const,
   sott_compare_string_array_tmp_string_array_aref,
   sott_compare_string_const_string_var,
   sott_compare_string_const_string_tmp,
   sott_compare_string_const_string_const,
   sott_compare_string_const_string_aref,
   sott_compare_string_const_string_aaref,
   sott_compare_int_array_const_int_array_var,
   sott_compare_int_array_const_int_array_tmp,
   sott_compare_int_array_const_int_array_const,
   sott_compare_int_array_const_int_array_aref,
   sott_compare_string_array_const_string_array_var,
   sott_compare_string_array_const_string_array_tmp,
   sott_compare_string_array_const_string_array_const,
   sott_compare_string_array_const_string_array_aref,
   sott_compare_int_array_array_const_int_array_array_var,
   sott_compare_int_array_array_const_int_array_array_const,
   sott_compare_string_array_array_const_string_array_array_var,
   sott_compare_string_array_array_const_string_array_array_const,
   sott_compare_int_aref_int_value,
   sott_compare_string_aref_string_var,
   sott_compare_string_aref_string_tmp,
   sott_compare_string_aref_string_const,
   sott_compare_string_aref_string_aref,
   sott_compare_string_aref_string_aaref,
   sott_compare_int_array_aref_int_array_var,
   sott_compare_int_array_aref_int_array_tmp,
   sott_compare_int_array_aref_int_array_const,
   sott_compare_int_array_aref_int_array_aref,
   sott_compare_string_array_aref_string_array_var,
   sott_compare_string_array_aref_string_array_tmp,
   sott_compare_string_array_aref_string_array_const,
   sott_compare_string_array_aref_string_array_aref,
   sott_compare_int_aaref_int_value,
   sott_compare_string_aaref_string_var,
   sott_compare_string_aaref_string_tmp,
   sott_compare_string_aaref_string_const,
   sott_compare_string_aaref_string_aref,
   sott_compare_string_aaref_string_aaref,

   // - test zda je prvni hodnota mensi nez druha -
   sott_less_int_var_int_value,
   sott_less_int_value_int_value,
   sott_less_int_aref_int_value,
   sott_less_int_aaref_int_value,

   // - test zda je prvni hodnota vetsi nez druha -
   sott_greater_int_var_int_value,
   sott_greater_int_value_int_value,
   sott_greater_int_aref_int_value,
   sott_greater_int_aaref_int_value,

   // - test zda je prvni hodnota mensi, nebo stejna jako druha hodnota -
   sott_less_equal_int_var_int_value,
   sott_less_equal_int_value_int_value,
   sott_less_equal_int_aref_int_value,
   sott_less_equal_int_aaref_int_value,

   // - test zda je prvni hodnota vetsi, nebo stejna jako druha hodnota -
   sott_greater_equal_int_var_int_value,
   sott_greater_equal_int_value_int_value,
   sott_greater_equal_int_aref_int_value,
   sott_greater_equal_int_aaref_int_value,

   // - secteni dvou hodnot -
   sott_add_int_var_int_value,
   sott_add_int_value_int_value,
   sott_add_int_aref_int_value,
   sott_add_int_aaref_int_value,

   // - odecteni dvou hodnot -
   sott_sub_int_var_int_value,
   sott_sub_int_value_int_value,
   sott_sub_int_aref_int_value,
   sott_sub_int_aaref_int_value,

   // - konkatenace dvou hodnot (retezcu) -
   sott_conc_string_var_string_var,
   sott_conc_string_var_string_tmp,
   sott_conc_string_var_string_const,
   sott_conc_string_var_string_aref,
   sott_conc_string_var_string_aaref,
   sott_conc_string_tmp_string_var,
   sott_conc_string_tmp_string_tmp,
   sott_conc_string_tmp_string_const,
   sott_conc_string_tmp_string_aref,
   sott_conc_string_tmp_string_aaref,
   sott_conc_string_const_string_var,
   sott_conc_string_const_string_tmp,
   sott_conc_string_const_string_const,
   sott_conc_string_const_string_aref,
   sott_conc_string_const_string_aaref,
   sott_conc_string_aref_string_var,
   sott_conc_string_aref_string_tmp,
   sott_conc_string_aref_string_const,
   sott_conc_string_aref_string_aref,
   sott_conc_string_aref_string_aaref,
   sott_conc_string_aaref_string_var,
   sott_conc_string_aaref_string_tmp,
   sott_conc_string_aaref_string_const,
   sott_conc_string_aaref_string_aref,
   sott_conc_string_aaref_string_aaref,

   // - inkrementace promenne po ulozeni jeji hodnoty na zasobnik -
   sott_post_inc_int_var,
   sott_post_inc_int_aref,
   sott_post_inc_int_aaref,

   // - dekrementace promenne po ulozeni jeji hodnoty na zasobnik -
   sott_post_dec_int_var,
   sott_post_dec_int_aref,
   sott_post_dec_int_aaref,

   // - inkrementace promenne a jeji ulozeni na zasobnik -
   sott_pre_inc_int_var,
   sott_pre_inc_int_aref,
   sott_pre_inc_int_aaref,

   // - dekremenyace promenne a jeji ulozeni na zasobnik -
   sott_pre_dec_int_var,
   sott_pre_dec_int_aref,
   sott_pre_dec_int_aaref,
   
   // - inverze(tj. -) a negace(tj. !) hodnoty na zasobniku -
   sott_invertion_int_value,
   sott_negation_int_value,

   // - ziskani podretezce -
   sott_substr_string_var_int_value_int_value,
   sott_substr_string_tmp_int_value_int_value,
   sott_substr_string_const_int_value_int_value,
   sott_substr_string_aref_int_value_int_value,
   sott_substr_string_aaref_int_value_int_value,

   // - zformatovani retezce podle escape znaku popsanych hodnotami na zasobniku -
   sott_format_string_var_int_array_const,
   sott_format_string_tmp_int_array_const,
   sott_format_string_const_int_array_const,
   sott_format_string_aref_int_array_const,
   sott_format_string_aaref_int_array_const,

   // - ziskani retezce popisujiciho terminal pravidla na pozici urcene hodnotou na zasobniku -
   sott_get_rb_int_value,

   // - ziska cislo radku na kterem se nachazi ve zdrojovem programu -
   sott_get_line_cnt,

   // - zmena formatu promennych tj string -> int a naopak -
   sott_convert_int_string_var_int_value,
   sott_convert_int_string_tmp_int_value,
   sott_convert_int_string_const_int_value,
   sott_convert_int_string_aref_int_value,
   sott_convert_int_string_aaref_int_value,
   sott_convert_string_int_value,

   // - instrukce zapisujici popis pozice promenne na zasobnik -
   sott_int_var,
   sott_string_var,
   sott_int_array_var,
   sott_string_array_var,
   sott_int_array_array_var,
   sott_string_array_array_var,

   // - zapis constant popsanych v codu na zasobnik -
   sott_int_value,
   sott_string_const,
   sott_int_array_const,
   sott_string_array_const,
   sott_int_array_array_const,
   sott_string_array_array_const,

   /*
    * ulozi ciselnou hodnotu int promenne na zasobnik(pouziva se pred nekterymi instrukcemi
    * k redukci poctu variant instrukci v zavislosti na typu hdonot)
    */
   sott_push_value_int_var,
   sott_push_value_int_aref,
   sott_push_value_int_aaref,

   // - podobne jako predchozi, slouzi pro ulozeni dvou hodnot najednou -
   sott_push_two_values_int_var_int_var,
   sott_push_two_values_int_aref_int_var,
   sott_push_two_values_int_aaref_int_var,
   sott_push_two_values_int_var_int_value,
   sott_push_two_values_int_aref_int_value,
   sott_push_two_values_int_aaref_int_value,
   sott_push_two_values_int_var_int_aref,
   sott_push_two_values_int_aref_int_aref,
   sott_push_two_values_int_aaref_int_aref,
   sott_push_two_values_int_var_int_aaref,
   sott_push_two_values_int_aref_int_aaref,
   sott_push_two_values_int_aaref_int_aaref,

   // - ulozi na zasobnik velikost retezce, nebo poli popsanych na zasobniku -
   sott_size_of_string_var,
   sott_size_of_int_array_var,
   sott_size_of_string_array_var,
   sott_size_of_int_array_array_var,
   sott_size_of_string_array_array_var,
   sott_size_of_string_tmp,
   sott_size_of_string_const,
   sott_size_of_string_aref,
   sott_size_of_int_array_aref,
   sott_size_of_string_array_aref,
   sott_size_of_string_aaref,

   // - vkladani hodnoty do pole -
   sott_push_to_int_array_var_int_value,
   sott_push_to_string_array_var_string_var,
   sott_push_to_string_array_var_string_tmp,
   sott_push_to_string_array_var_string_const,
   sott_push_to_string_array_var_string_aref,
   sott_push_to_string_array_var_string_aaref,
   sott_push_to_int_array_array_var_int_array_var,
   sott_push_to_int_array_array_var_int_array_tmp,
   sott_push_to_int_array_array_var_int_array_const,
   sott_push_to_int_array_array_var_int_array_aref,
   sott_push_to_int_array_array_var_blank_array,
   sott_push_to_string_array_array_var_string_array_var,
   sott_push_to_string_array_array_var_string_array_tmp,
   sott_push_to_string_array_array_var_string_array_const,
   sott_push_to_string_array_array_var_string_array_aref,
   sott_push_to_string_array_array_var_blank_array,
   sott_push_to_int_array_aref_int_value,
   sott_push_to_string_array_aref_string_var,
   sott_push_to_string_array_aref_string_tmp,
   sott_push_to_string_array_aref_string_const,
   sott_push_to_string_array_aref_string_aref,
   sott_push_to_string_array_aref_string_aaref,

   // - vyzvednuti hodnoty z vrcholu pole -
   sott_pop_from_int_array_var,
   sott_pop_from_string_array_var,
   sott_pop_from_int_array_array_var,
   sott_pop_from_string_array_array_var,
   sott_pop_from_int_array_aref,
   sott_pop_from_string_array_aref,
 
   // - odstraneni x hodnot z vrcholu zasobniku -
   sott_remove_from_int_array_var,
   sott_remove_from_string_array_var,
   sott_remove_from_int_array_array_var,
   sott_remove_from_string_array_array_var,
   sott_remove_from_int_array_aref,
   sott_remove_from_string_array_aref,

   // - ziskani reference na posledni prvek v poli -
   sott_last_of_int_array_var,
   sott_last_of_string_array_var,
   sott_last_of_int_array_array_var,
   sott_last_of_string_array_array_var,
   sott_last_of_int_array_aref,
   sott_last_of_string_array_aref,

   // - ziska referenci na x-ty prvek v poli -
   sott_tail_of_int_array_var_int_value,
   sott_tail_of_string_array_var_int_value,
   sott_tail_of_int_array_array_var_int_value,
   sott_tail_of_string_array_array_var_int_value,
   sott_tail_of_int_array_aref_int_value,
   sott_tail_of_string_array_aref_int_value,

   // - ziskani indexu hodnoty v poli stejneho typu -
   sott_get_idx_int_array_var_int_value,
   sott_get_idx_string_array_var_string_var,
   sott_get_idx_string_array_var_string_tmp,
   sott_get_idx_string_array_var_string_const,
   sott_get_idx_string_array_var_string_aref,
   sott_get_idx_string_array_var_string_aaref,
   sott_get_idx_int_array_array_var_int_array_var,
   sott_get_idx_int_array_array_var_int_array_tmp,
   sott_get_idx_int_array_array_var_int_array_const,
   sott_get_idx_int_array_array_var_int_array_aref,
   sott_get_idx_int_array_array_var_blank_array,
   sott_get_idx_string_array_array_var_string_array_var,
   sott_get_idx_string_array_array_var_string_array_tmp,
   sott_get_idx_string_array_array_var_string_array_const,
   sott_get_idx_string_array_array_var_string_array_aref,
   sott_get_idx_string_array_array_var_blank_array,
   sott_get_idx_int_array_aref_int_value,
   sott_get_idx_string_array_aref_string_var,
   sott_get_idx_string_array_aref_string_tmp,
   sott_get_idx_string_array_aref_string_const,
   sott_get_idx_string_array_aref_string_aref,
   sott_get_idx_string_array_aref_string_aaref
};

/*
 * seznam odkazu na funkce popisujici jednotlive instrukce semantickych podprogramu,
 * odpovidaji vyse uvedenemu seznamu identifikatoru intrukci
 */

const unsigned c_sem_parser_instruction_cnt = 351;
void(*parse_instr_callers[c_sem_parser_instruction_cnt])(p_parse_data_s&) = {
   parser_s::pi_sott_out_string_var,
   parser_s::pi_sott_out_int_array_var,
   parser_s::pi_sott_out_string_array_var,
   parser_s::pi_sott_out_int_array_array_var,
   parser_s::pi_sott_out_string_array_array_var,
   parser_s::pi_sott_out_int_value,
   parser_s::pi_sott_out_string_tmp,
   parser_s::pi_sott_out_int_array_tmp,
   parser_s::pi_sott_out_string_array_tmp,
   parser_s::pi_sott_out_string_const,
   parser_s::pi_sott_out_string_aref,
   parser_s::pi_sott_out_int_array_aref,
   parser_s::pi_sott_out_string_array_aref,
   parser_s::pi_sott_out_string_aaref,

   parser_s::pi_sott_jump,
   parser_s::pi_sott_jump_if_zero,
   parser_s::pi_sott_jump_if_no_zero,

   parser_s::pi_sott_assert_int_value_string_const,

   parser_s::pi_sott_exp_end,
   parser_s::pi_sott_exp_end_string_tmp,
   parser_s::pi_sott_exp_end_int_array_tmp,
   parser_s::pi_sott_exp_end_string_array_tmp,
   parser_s::pi_sott_exp_end_aref,
   parser_s::pi_sott_exp_end_aaref,

   parser_s::pi_sott_set_int_var_int_value,
   parser_s::pi_sott_set_string_var_string_var,
   parser_s::pi_sott_set_string_var_string_tmp,
   parser_s::pi_sott_set_string_var_string_const,
   parser_s::pi_sott_set_string_var_string_aref,
   parser_s::pi_sott_set_string_var_string_aaref,
   parser_s::pi_sott_set_int_array_var_int_array_var,
   parser_s::pi_sott_set_int_array_var_int_array_tmp,
   parser_s::pi_sott_set_int_array_var_int_array_const,
   parser_s::pi_sott_set_int_array_var_int_array_aref,
   parser_s::pi_sott_set_int_array_var_blank_array,
   parser_s::pi_sott_set_string_array_var_string_array_var,
   parser_s::pi_sott_set_string_array_var_string_array_tmp,
   parser_s::pi_sott_set_string_array_var_string_array_const,
   parser_s::pi_sott_set_string_array_var_string_array_aref,
   parser_s::pi_sott_set_string_array_var_blank_array,
   parser_s::pi_sott_set_int_array_array_var_int_array_array_var,
   parser_s::pi_sott_set_int_array_array_var_int_array_array_const,
   parser_s::pi_sott_set_int_array_array_var_blank_array,
   parser_s::pi_sott_set_string_array_array_var_string_array_array_var,
   parser_s::pi_sott_set_string_array_array_var_string_array_array_const,
   parser_s::pi_sott_set_string_array_array_var_blank_array,
   parser_s::pi_sott_set_int_aref_int_value,
   parser_s::pi_sott_set_string_aref_string_var,
   parser_s::pi_sott_set_string_aref_string_tmp,
   parser_s::pi_sott_set_string_aref_string_const,
   parser_s::pi_sott_set_string_aref_string_aref,
   parser_s::pi_sott_set_string_aref_string_aaref,
   parser_s::pi_sott_set_int_array_aref_int_array_var,
   parser_s::pi_sott_set_int_array_aref_int_array_tmp,
   parser_s::pi_sott_set_int_array_aref_int_array_const,
   parser_s::pi_sott_set_int_array_aref_int_array_aref,
   parser_s::pi_sott_set_int_array_aref_blank_array,
   parser_s::pi_sott_set_string_array_aref_string_array_var,
   parser_s::pi_sott_set_string_array_aref_string_array_tmp,
   parser_s::pi_sott_set_string_array_aref_string_array_const,
   parser_s::pi_sott_set_string_array_aref_string_array_aref,
   parser_s::pi_sott_set_string_array_aref_blank_array,
   parser_s::pi_sott_set_int_aaref_int_value,
   parser_s::pi_sott_set_string_aaref_string_var,
   parser_s::pi_sott_set_string_aaref_string_tmp,
   parser_s::pi_sott_set_string_aaref_string_const,
   parser_s::pi_sott_set_string_aaref_string_aref,
   parser_s::pi_sott_set_string_aaref_string_aaref,

   parser_s::pi_sott_swap_string_var_string_var,
   parser_s::pi_sott_swap_string_var_string_aref,
   parser_s::pi_sott_swap_string_var_string_aaref,
   parser_s::pi_sott_swap_string_aref_string_var,
   parser_s::pi_sott_swap_string_aref_string_aref,
   parser_s::pi_sott_swap_string_aref_string_aaref,
   parser_s::pi_sott_swap_string_aaref_string_var,
   parser_s::pi_sott_swap_string_aaref_string_aref,
   parser_s::pi_sott_swap_string_aaref_string_aaref,
   parser_s::pi_sott_swap_int_array_var_int_array_var,
   parser_s::pi_sott_swap_int_array_var_int_array_aref,
   parser_s::pi_sott_swap_int_array_aref_int_array_var,
   parser_s::pi_sott_swap_int_array_aref_int_array_aref,
   parser_s::pi_sott_swap_string_array_var_string_array_var,
   parser_s::pi_sott_swap_string_array_var_string_array_aref,
   parser_s::pi_sott_swap_string_array_aref_string_array_var,
   parser_s::pi_sott_swap_string_array_aref_string_array_aref,
   parser_s::pi_sott_swap_int_array_array_var_int_array_array_var,
   parser_s::pi_sott_swap_string_array_array_var_string_array_array_var,

   parser_s::pi_sott_logic_and_int_var_int_value,
   parser_s::pi_sott_logic_and_int_value_int_value,
   parser_s::pi_sott_logic_and_int_aref_int_value,
   parser_s::pi_sott_logic_and_int_aaref_int_value,

   parser_s::pi_sott_logic_or_int_var_int_value,
   parser_s::pi_sott_logic_or_int_value_int_value,
   parser_s::pi_sott_logic_or_int_aref_int_value,
   parser_s::pi_sott_logic_or_int_aaref_int_value,

   parser_s::pi_sott_compare_int_var_int_value,
   parser_s::pi_sott_compare_string_var_string_var,
   parser_s::pi_sott_compare_string_var_string_tmp,
   parser_s::pi_sott_compare_string_var_string_const,
   parser_s::pi_sott_compare_string_var_string_aref,
   parser_s::pi_sott_compare_string_var_string_aaref,
   parser_s::pi_sott_compare_int_array_var_int_array_var,
   parser_s::pi_sott_compare_int_array_var_int_array_tmp,
   parser_s::pi_sott_compare_int_array_var_int_array_const,
   parser_s::pi_sott_compare_int_array_var_int_array_aref,
   parser_s::pi_sott_compare_string_array_var_string_array_var,
   parser_s::pi_sott_compare_string_array_var_string_array_tmp,
   parser_s::pi_sott_compare_string_array_var_string_array_const,
   parser_s::pi_sott_compare_string_array_var_string_array_aref,
   parser_s::pi_sott_compare_int_array_array_var_int_array_array_var,
   parser_s::pi_sott_compare_int_array_array_var_int_array_array_const,
   parser_s::pi_sott_compare_string_array_array_var_string_array_array_var,
   parser_s::pi_sott_compare_string_array_array_var_string_array_array_const,
   parser_s::pi_sott_compare_int_value_int_value,
   parser_s::pi_sott_compare_string_tmp_string_var,
   parser_s::pi_sott_compare_string_tmp_string_tmp,
   parser_s::pi_sott_compare_string_tmp_string_const,
   parser_s::pi_sott_compare_string_tmp_string_aref,
   parser_s::pi_sott_compare_string_tmp_string_aaref,
   parser_s::pi_sott_compare_int_array_tmp_int_array_var,
   parser_s::pi_sott_compare_int_array_tmp_int_array_tmp,
   parser_s::pi_sott_compare_int_array_tmp_int_array_const,
   parser_s::pi_sott_compare_int_array_tmp_int_array_aref,
   parser_s::pi_sott_compare_string_array_tmp_string_array_var,
   parser_s::pi_sott_compare_string_array_tmp_string_array_tmp,
   parser_s::pi_sott_compare_string_array_tmp_string_array_const,
   parser_s::pi_sott_compare_string_array_tmp_string_array_aref,
   parser_s::pi_sott_compare_string_const_string_var,
   parser_s::pi_sott_compare_string_const_string_tmp,
   parser_s::pi_sott_compare_string_const_string_const,
   parser_s::pi_sott_compare_string_const_string_aref,
   parser_s::pi_sott_compare_string_const_string_aaref,
   parser_s::pi_sott_compare_int_array_const_int_array_var,
   parser_s::pi_sott_compare_int_array_const_int_array_tmp,
   parser_s::pi_sott_compare_int_array_const_int_array_const,
   parser_s::pi_sott_compare_int_array_const_int_array_aref,
   parser_s::pi_sott_compare_string_array_const_string_array_var,
   parser_s::pi_sott_compare_string_array_const_string_array_tmp,
   parser_s::pi_sott_compare_string_array_const_string_array_const,
   parser_s::pi_sott_compare_string_array_const_string_array_aref,
   parser_s::pi_sott_compare_int_array_array_const_int_array_array_var,
   parser_s::pi_sott_compare_int_array_array_const_int_array_array_const,
   parser_s::pi_sott_compare_string_array_array_const_string_array_array_var,
   parser_s::pi_sott_compare_string_array_array_const_string_array_array_const,
   parser_s::pi_sott_compare_int_aref_int_value,
   parser_s::pi_sott_compare_string_aref_string_var,
   parser_s::pi_sott_compare_string_aref_string_tmp,
   parser_s::pi_sott_compare_string_aref_string_const,
   parser_s::pi_sott_compare_string_aref_string_aref,
   parser_s::pi_sott_compare_string_aref_string_aaref,
   parser_s::pi_sott_compare_int_array_aref_int_array_var,
   parser_s::pi_sott_compare_int_array_aref_int_array_tmp,
   parser_s::pi_sott_compare_int_array_aref_int_array_const,
   parser_s::pi_sott_compare_int_array_aref_int_array_aref,
   parser_s::pi_sott_compare_string_array_aref_string_array_var,
   parser_s::pi_sott_compare_string_array_aref_string_array_tmp,
   parser_s::pi_sott_compare_string_array_aref_string_array_const,
   parser_s::pi_sott_compare_string_array_aref_string_array_aref,
   parser_s::pi_sott_compare_int_aaref_int_value,
   parser_s::pi_sott_compare_string_aaref_string_var,
   parser_s::pi_sott_compare_string_aaref_string_tmp,
   parser_s::pi_sott_compare_string_aaref_string_const,
   parser_s::pi_sott_compare_string_aaref_string_aref,
   parser_s::pi_sott_compare_string_aaref_string_aaref,

   parser_s::pi_sott_less_int_var_int_value,
   parser_s::pi_sott_less_int_value_int_value,
   parser_s::pi_sott_less_int_aref_int_value,
   parser_s::pi_sott_less_int_aaref_int_value,

   parser_s::pi_sott_greater_int_var_int_value,
   parser_s::pi_sott_greater_int_value_int_value,
   parser_s::pi_sott_greater_int_aref_int_value,
   parser_s::pi_sott_greater_int_aaref_int_value,

   parser_s::pi_sott_less_equal_int_var_int_value,
   parser_s::pi_sott_less_equal_int_value_int_value,
   parser_s::pi_sott_less_equal_int_aref_int_value,
   parser_s::pi_sott_less_equal_int_aaref_int_value,

   parser_s::pi_sott_greater_equal_int_var_int_value,
   parser_s::pi_sott_greater_equal_int_value_int_value,
   parser_s::pi_sott_greater_equal_int_aref_int_value,
   parser_s::pi_sott_greater_equal_int_aaref_int_value,

   parser_s::pi_sott_add_int_var_int_value,
   parser_s::pi_sott_add_int_value_int_value,
   parser_s::pi_sott_add_int_aref_int_value,
   parser_s::pi_sott_add_int_aaref_int_value,

   parser_s::pi_sott_sub_int_var_int_value,
   parser_s::pi_sott_sub_int_value_int_value,
   parser_s::pi_sott_sub_int_aref_int_value,
   parser_s::pi_sott_sub_int_aaref_int_value,

   parser_s::pi_sott_conc_string_var_string_var,
   parser_s::pi_sott_conc_string_var_string_tmp,
   parser_s::pi_sott_conc_string_var_string_const,
   parser_s::pi_sott_conc_string_var_string_aref,
   parser_s::pi_sott_conc_string_var_string_aaref,
   parser_s::pi_sott_conc_string_tmp_string_var,
   parser_s::pi_sott_conc_string_tmp_string_tmp,
   parser_s::pi_sott_conc_string_tmp_string_const,
   parser_s::pi_sott_conc_string_tmp_string_aref,
   parser_s::pi_sott_conc_string_tmp_string_aaref,
   parser_s::pi_sott_conc_string_const_string_var,
   parser_s::pi_sott_conc_string_const_string_tmp,
   parser_s::pi_sott_conc_string_const_string_const,
   parser_s::pi_sott_conc_string_const_string_aref,
   parser_s::pi_sott_conc_string_const_string_aaref,
   parser_s::pi_sott_conc_string_aref_string_var,
   parser_s::pi_sott_conc_string_aref_string_tmp,
   parser_s::pi_sott_conc_string_aref_string_const,
   parser_s::pi_sott_conc_string_aref_string_aref,
   parser_s::pi_sott_conc_string_aref_string_aaref,
   parser_s::pi_sott_conc_string_aaref_string_var,
   parser_s::pi_sott_conc_string_aaref_string_tmp,
   parser_s::pi_sott_conc_string_aaref_string_const,
   parser_s::pi_sott_conc_string_aaref_string_aref,
   parser_s::pi_sott_conc_string_aaref_string_aaref,

   parser_s::pi_sott_post_inc_int_var,
   parser_s::pi_sott_post_inc_int_aref,
   parser_s::pi_sott_post_inc_int_aaref,

   parser_s::pi_sott_post_dec_int_var,
   parser_s::pi_sott_post_dec_int_aref,
   parser_s::pi_sott_post_dec_int_aaref,

   parser_s::pi_sott_pre_inc_int_var,
   parser_s::pi_sott_pre_inc_int_aref,
   parser_s::pi_sott_pre_inc_int_aaref,

   parser_s::pi_sott_pre_dec_int_var,
   parser_s::pi_sott_pre_dec_int_aref,
   parser_s::pi_sott_pre_dec_int_aaref,
   
   parser_s::pi_sott_invertion_int_value,
   parser_s::pi_sott_negation_int_value,

   parser_s::pi_sott_substr_string_var_int_value_int_value,
   parser_s::pi_sott_substr_string_tmp_int_value_int_value,
   parser_s::pi_sott_substr_string_const_int_value_int_value,
   parser_s::pi_sott_substr_string_aref_int_value_int_value,
   parser_s::pi_sott_substr_string_aaref_int_value_int_value,

   parser_s::pi_sott_format_string_var_int_array_const,
   parser_s::pi_sott_format_string_tmp_int_array_const,
   parser_s::pi_sott_format_string_const_int_array_const,
   parser_s::pi_sott_format_string_aref_int_array_const,
   parser_s::pi_sott_format_string_aaref_int_array_const,

   parser_s::pi_sott_get_rb_int_value,

   parser_s::pi_sott_get_line_cnt,

   parser_s::pi_sott_convert_int_string_var_int_value,
   parser_s::pi_sott_convert_int_string_tmp_int_value,
   parser_s::pi_sott_convert_int_string_const_int_value,
   parser_s::pi_sott_convert_int_string_aref_int_value,
   parser_s::pi_sott_convert_int_string_aaref_int_value,
   parser_s::pi_sott_convert_string_int_value,

   parser_s::pi_sott_int_var,
   parser_s::pi_sott_string_var,
   parser_s::pi_sott_int_array_var,
   parser_s::pi_sott_string_array_var,
   parser_s::pi_sott_int_array_array_var,
   parser_s::pi_sott_string_array_array_var,

   parser_s::pi_sott_int_value,
   parser_s::pi_sott_string_const,
   parser_s::pi_sott_int_array_const,
   parser_s::pi_sott_string_array_const,
   parser_s::pi_sott_int_array_array_const,
   parser_s::pi_sott_string_array_array_const,

   parser_s::pi_sott_push_value_int_var,
   parser_s::pi_sott_push_value_int_aref,
   parser_s::pi_sott_push_value_int_aaref,

   parser_s::pi_sott_push_two_values_int_var_int_var,
   parser_s::pi_sott_push_two_values_int_aref_int_var,
   parser_s::pi_sott_push_two_values_int_aaref_int_var,
   parser_s::pi_sott_push_two_values_int_var_int_value,
   parser_s::pi_sott_push_two_values_int_aref_int_value,
   parser_s::pi_sott_push_two_values_int_aaref_int_value,
   parser_s::pi_sott_push_two_values_int_var_int_aref,
   parser_s::pi_sott_push_two_values_int_aref_int_aref,
   parser_s::pi_sott_push_two_values_int_aaref_int_aref,
   parser_s::pi_sott_push_two_values_int_var_int_aaref,
   parser_s::pi_sott_push_two_values_int_aref_int_aaref,
   parser_s::pi_sott_push_two_values_int_aaref_int_aaref,

   parser_s::pi_sott_size_of_string_var,
   parser_s::pi_sott_size_of_int_array_var,
   parser_s::pi_sott_size_of_string_array_var,
   parser_s::pi_sott_size_of_int_array_array_var,
   parser_s::pi_sott_size_of_string_array_array_var,
   parser_s::pi_sott_size_of_string_tmp,
   parser_s::pi_sott_size_of_string_const,
   parser_s::pi_sott_size_of_string_aref,
   parser_s::pi_sott_size_of_int_array_aref,
   parser_s::pi_sott_size_of_string_array_aref,
   parser_s::pi_sott_size_of_string_aaref,

   parser_s::pi_sott_push_to_int_array_var_int_value,
   parser_s::pi_sott_push_to_string_array_var_string_var,
   parser_s::pi_sott_push_to_string_array_var_string_tmp,
   parser_s::pi_sott_push_to_string_array_var_string_const,
   parser_s::pi_sott_push_to_string_array_var_string_aref,
   parser_s::pi_sott_push_to_string_array_var_string_aaref,
   parser_s::pi_sott_push_to_int_array_array_var_int_array_var,
   parser_s::pi_sott_push_to_int_array_array_var_int_array_tmp,
   parser_s::pi_sott_push_to_int_array_array_var_int_array_const,
   parser_s::pi_sott_push_to_int_array_array_var_int_array_aref,
   parser_s::pi_sott_push_to_int_array_array_var_blank_array,
   parser_s::pi_sott_push_to_string_array_array_var_string_array_var,
   parser_s::pi_sott_push_to_string_array_array_var_string_array_tmp,
   parser_s::pi_sott_push_to_string_array_array_var_string_array_const,
   parser_s::pi_sott_push_to_string_array_array_var_string_array_aref,
   parser_s::pi_sott_push_to_string_array_array_var_blank_array,
   parser_s::pi_sott_push_to_int_array_aref_int_value,
   parser_s::pi_sott_push_to_string_array_aref_string_var,
   parser_s::pi_sott_push_to_string_array_aref_string_tmp,
   parser_s::pi_sott_push_to_string_array_aref_string_const,
   parser_s::pi_sott_push_to_string_array_aref_string_aref,
   parser_s::pi_sott_push_to_string_array_aref_string_aaref,

   parser_s::pi_sott_pop_from_int_array_var,
   parser_s::pi_sott_pop_from_string_array_var,
   parser_s::pi_sott_pop_from_int_array_array_var,
   parser_s::pi_sott_pop_from_string_array_array_var,
   parser_s::pi_sott_pop_from_int_array_aref,
   parser_s::pi_sott_pop_from_string_array_aref,
 
   parser_s::pi_sott_remove_from_int_array_var,
   parser_s::pi_sott_remove_from_string_array_var,
   parser_s::pi_sott_remove_from_int_array_array_var,
   parser_s::pi_sott_remove_from_string_array_array_var,
   parser_s::pi_sott_remove_from_int_array_aref,
   parser_s::pi_sott_remove_from_string_array_aref,

   parser_s::pi_sott_last_of_int_array_var,
   parser_s::pi_sott_last_of_string_array_var,
   parser_s::pi_sott_last_of_int_array_array_var,
   parser_s::pi_sott_last_of_string_array_array_var,
   parser_s::pi_sott_last_of_int_array_aref,
   parser_s::pi_sott_last_of_string_array_aref,

   parser_s::pi_sott_tail_of_int_array_var_int_value,
   parser_s::pi_sott_tail_of_string_array_var_int_value,
   parser_s::pi_sott_tail_of_int_array_array_var_int_value,
   parser_s::pi_sott_tail_of_string_array_array_var_int_value,
   parser_s::pi_sott_tail_of_int_array_aref_int_value,
   parser_s::pi_sott_tail_of_string_array_aref_int_value,

   parser_s::pi_sott_get_idx_int_array_var_int_value,
   parser_s::pi_sott_get_idx_string_array_var_string_var,
   parser_s::pi_sott_get_idx_string_array_var_string_tmp,
   parser_s::pi_sott_get_idx_string_array_var_string_const,
   parser_s::pi_sott_get_idx_string_array_var_string_aref,
   parser_s::pi_sott_get_idx_string_array_var_string_aaref,
   parser_s::pi_sott_get_idx_int_array_array_var_int_array_var,
   parser_s::pi_sott_get_idx_int_array_array_var_int_array_tmp,
   parser_s::pi_sott_get_idx_int_array_array_var_int_array_const,
   parser_s::pi_sott_get_idx_int_array_array_var_int_array_aref,
   parser_s::pi_sott_get_idx_int_array_array_var_blank_array,
   parser_s::pi_sott_get_idx_string_array_array_var_string_array_var,
   parser_s::pi_sott_get_idx_string_array_array_var_string_array_tmp,
   parser_s::pi_sott_get_idx_string_array_array_var_string_array_const,
   parser_s::pi_sott_get_idx_string_array_array_var_string_array_aref,
   parser_s::pi_sott_get_idx_string_array_array_var_blank_array,
   parser_s::pi_sott_get_idx_int_array_aref_int_value,
   parser_s::pi_sott_get_idx_string_array_aref_string_var,
   parser_s::pi_sott_get_idx_string_array_aref_string_tmp,
   parser_s::pi_sott_get_idx_string_array_aref_string_const,
   parser_s::pi_sott_get_idx_string_array_aref_string_aref,
   parser_s::pi_sott_get_idx_string_array_aref_string_aaref,
};

// -- funkce struktury p_sem_variable_names_s --

/*
 * vyhleda v poli retezcu index retezce reprezentovaneho jeho delkou a ukazatelem na char
 */
unsigned p_sem_variable_names_s::get_char_ptr_idx(unsigned a_length,char *a_data)
{
   if (used == 0) return c_idx_not_exist;

   string_s *ptr = data;
   string_s *ptr_end = ptr + used;

   do {
      if (ptr->compare_char_ptr(a_length,a_data)) {
	 return ptr - data;
      }
   } while(++ptr < ptr_end);

   return c_idx_not_exist;
}

// -- funkce struktury p_terminals_s --

/*
 * ziska index terminalu, ktery ma jmeno shodne s predanym retezcem
 */
unsigned p_terminals_s::get_char_ptr_idx(unsigned a_length,char *a_data)
{
   if (used == 0) return c_idx_not_exist;
   
   p_terminal_s *ptr = data;
   p_terminal_s *ptr_end = ptr + used;
   
   do {
      if (ptr->idx.compare_char_ptr(a_length,a_data)) {
	 return ptr - data;
      }
   } while(++ptr < ptr_end);

   return c_idx_not_exist;
}

// -- funkce struktury p_nonterminals_s --

/*
 * ziska index nonterminalu, ktery ma jmeno shodne s predanym retezcem
 */
unsigned p_nonterminals_s::get_char_ptr_idx(unsigned a_length,char *a_data)
{
   if (used == 0) return c_idx_not_exist;
   
   p_nonterminal_s *ptr = data;
   p_nonterminal_s *ptr_end = ptr + used;
   
   do {
      if (ptr->name.compare_char_ptr(a_length,a_data)) {
	 return ptr - data;
      }
   } while(++ptr < ptr_end);

   return c_idx_not_exist;
}

// -- funkce struktury p_kernels_s --

/*
 * nalezne index jadra, ktere ma s predanym jadrem shodne data ve strukture popisujici pravidla
 * jadra
 */
unsigned p_kernels_s::get_idx_by_rule_dots(p_kernel_s &a_value)
{
   if (used == 0) return c_idx_not_exist;

   p_kernel_s *ptr = data;
   p_kernel_s *ptr_end = ptr + used;

   do {
      if (ptr->rule_dots == a_value.rule_dots) {
	 return ptr - data;
      }
   } while(++ptr < ptr_end);

   return c_idx_not_exist;
}

// -- funkce struktury p_creat_descr_s --

/*
 * vytvori, nebo nacte ze souboru konecne automaty potrebne pro
 * rozklad souboru pravidel popisujicich jazyk
 * do mnoziny automatu patri:
 *    rule_file_fa - rozpoznava terminaly v souboru pravidel
 *    key_terminals_sa - dodatecne rozpoznava terminaly se specialnim vyznamem pro prekladac (klicovym vyzname)
 */
inline bool p_creat_descr_s::load_final_automata_set(char *file_name)
{
#ifndef p_creat_descr_s_load_final_automata_set_CREATE
   return final_automata_set.load_file(file_name);
#else
   final_automata_set.rule_file_fa.clear();

   ushort_reg_exps_s test_us_reg_exps;
   test_us_reg_exps.init();

   test_us_reg_exps.load_char_ptrs_reg_exps(test_reg_exp_cnt,(char **)test_reg_exps);
   final_automata_set.rule_file_fa.create(test_us_reg_exps);

   test_us_reg_exps.load_char_ptrs_reg_exps(c_key_terminal_cnt,(char **)key_terminals);
   final_automata_set.key_terminals_fa.create(test_us_reg_exps);
   test_us_reg_exps.clear();

   return final_automata_set.save_file(file_name);
#endif
}

inline bool p_creat_descr_s::load_final_automata_set_new(const char *file_name)
{
#ifndef p_creat_descr_s_load_final_automata_set_CREATE
   return final_automata_set.load_file(file_name);
#else
   fa_states_array_s states_array;
   states_array.init();

   reg_parser_s reg_parser;
   reg_parser.init();

   // - zpracovani reg. vyrazu urcenych pro cteni souboru pravidel -
   {
      unsigned re_idx = 0;
      do {
	 string_s reg_exp_string;
	 reg_exp_string.size = strlen(test_reg_exps[re_idx]) + 1;
	 reg_exp_string.data = (char *)test_reg_exps[re_idx];

	 reg_parser.process_reg_exp(reg_exp_string);

	 states_array.push_blank();
	 states_array.last().swap(reg_parser.states);
      } while(++re_idx < test_reg_exp_cnt);
   }

   final_automata_set.rule_file_fa.clear();
   final_automata_set.rule_file_fa.create_new(states_array);

   states_array.used = 0;

   // - zpracovani reg. vyrazu rozpoznavajicich klicove terminaly -
   {
      unsigned re_idx = 0;
      do {
	 string_s reg_exp_string;
	 reg_exp_string.size = strlen(key_terminals[re_idx]) + 1;
	 reg_exp_string.data = (char *)key_terminals[re_idx];

	 reg_parser.process_reg_exp(reg_exp_string);

	 states_array.push_blank();
	 states_array.last().swap(reg_parser.states);
      } while(++re_idx < c_key_terminal_cnt);
   }

   final_automata_set.key_terminals_fa.clear();
   final_automata_set.key_terminals_fa.create_new(states_array);

   reg_parser.clear();
   states_array.clear();

   return final_automata_set.save_file(file_name);
#endif
}

/*
 * naplni strukturu p_creat_descr_s (parser_creation_description) ze souboru popisujiciho
 * jazyk (tj. nalezne inicializacni sem. kod, terminaly, neterminaly, pravidla (vcetne semantickeho kodu)). 
 */
bool p_creat_descr_s::load_from_rule_char_ptr(unsigned a_length,char *a_data)
{
   terminals.used = 0;
   nonterminals.used = 0;
   rules.used = 0;

   // - konecny automat nacitajici terminaly ze souboru pravidel -
   final_automata_s &final_automata = final_automata_set.rule_file_fa;

   unsigned input_idx = 0;
   unsigned line_cnt = 0;
   unsigned input_length = a_length;
   unsigned ret_term;

   while((ret_term = final_automata.recognize(a_data,input_idx,input_length,line_cnt)) != rt_init_code);
   
   // - nacteni inicializacniho semantickeho kodu -
   {
      do {
	 unsigned old_input_idx = input_idx;

	 ret_term = final_automata.recognize(a_data,input_idx,input_length,line_cnt);
	 
	 // - preskakuj bile mista, a comentare -
	 if (ret_term <= rt_sec_comment) {
	    continue;
	 }
	 
         // FX ERR
         if (ret_term != rt_code) {
            fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: expected code terminal symbol\n");
            assert(0);
         }

	 init_sem_prog.set(input_idx - old_input_idx - 2,a_data + old_input_idx + 1);
	 break;

      } while(1);
   }

   while((ret_term = final_automata.recognize(a_data,input_idx,input_length,line_cnt)) != rt_terminals);

   // - zpracovani terminalu -
   {
      unsigned terminal_state = 0;
      p_terminal_s new_terminal;
      new_terminal.init();

      do {
	 unsigned old_input_idx = input_idx;

	 // - dokud neni vracen terminal rt_nonterminals -
	 if ((ret_term = final_automata.recognize(a_data,input_idx,input_length,line_cnt)) == rt_nonterminals) {
	    break;
	 }

	 // - preskakuj bile mista -
	 if (ret_term == rt_space || ret_term == rt_comment) {
	    continue;
	 }

	 // - nacteni jmena terminalu -
	 if (terminal_state == 0) {

            // FX ERR
            if (ret_term != rt_identifier) {
               fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: expected identifier terminal symbol\n");
               assert(0);
            }

	    new_terminal.idx.set(input_idx - old_input_idx,a_data + old_input_idx);

	    terminal_state = 1;
	 }

	 // - regularni vyraz popisujici terminal -
	 else {

            // FX ERR
            if (ret_term != rt_code) {
               fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: expected code terminal symbol\n");
               assert(0);
            }

	    new_terminal.value.set(input_idx - old_input_idx - 2,a_data + old_input_idx + 1);

            // FX ERR
            if (terminals.get_idx(new_terminal) != c_idx_not_exist) {
               fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: new terminal is already defined\n");
               assert(0);
            }

	    terminals.push_blank();
	    terminals.last().swap(new_terminal);

	    terminal_state = 0;
	 }
      } while(1);

      new_terminal.clear();
   }

   // - zpracovani nonterminalu -
   {
      p_nonterminal_s new_nonterminal;
      new_nonterminal.init();

      do {
	 unsigned old_input_idx = input_idx;

	 // - dokud neni vracen terminal rt_rules -
	 if ((ret_term = final_automata.recognize(a_data,input_idx,input_length,line_cnt)) == rt_rules) {
	    break;
	 }

	 // - preskoc bila mista -
	 if (ret_term == rt_space || ret_term == rt_comment) {
	    continue;
	 }
	 
         // FX ERR
         if (ret_term != rt_nonterm_id) {
            fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: expected nonterminal terminal symbol\n");
            assert(0);
         }


	 // - nacteni jmena neterminalniho symbolu -
	 new_nonterminal.name.set(input_idx - old_input_idx,a_data + old_input_idx);

         // FX ERR
         if (nonterminals.get_idx(new_nonterminal) != c_idx_not_exist) {
            fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: nonterminal symbol is already defined\n");
            assert(0);
         }

	 nonterminals.push_blank();
	 nonterminals.last().swap(new_nonterminal);

      } while(1);

      new_nonterminal.clear();
   }

   // - zpracovani pravidel -
   {
      unsigned rule_state = 0;
      p_rule_s new_rule;
      new_rule.init();

      do {
	 unsigned old_input_idx = input_idx;

	 // - dokud neni vracen konec vstupu -
	 if ((ret_term = final_automata.recognize(a_data,input_idx,input_length,line_cnt)) == rt_input_end) {
	    break;
	 }

	 if (ret_term == rt_space || ret_term == rt_comment) {
	    continue;
	 }

	 if (rule_state < 2) {

	    // - nacteni hlavy pravidla -
	    if (rule_state == 0) {

               // FX ERR
               if (ret_term != rt_nonterm_id) {
                  fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: rule head expect nonteminal symbol\n");
                  assert(0);
               }

	       unsigned idx = nonterminals.get_char_ptr_idx(input_idx - old_input_idx,a_data + old_input_idx);

               // FX ERR
               if (idx == c_idx_not_exist) {
                  fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: nonterminal used as rule head was not defined\n");
                  assert(0);
               }


	       new_rule.head = terminals.used + idx;
	       rule_state = 1;
	    }

	    // - potvrzeni oddeleni hlavy pravidla a jeho tela -
	    else {

               // FX ERR
               if (ret_term != rt_div_0) {
                  fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: wrong separator of rule head from its body\n");
                  assert(0);
               }

	       new_rule.body.used = 0;
	       rule_state = 2;
	    }
	 }
	 else {
	    
	    // - nacteni tela pravidla -
	    if (rule_state == 2) {
	       if (ret_term == rt_identifier) {
		  unsigned idx = terminals.get_char_ptr_idx(input_idx - old_input_idx,a_data + old_input_idx);

                  // FX ERR
                  if (idx == c_idx_not_exist) {
                     fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: cannot resolve terminal symbol from rule body\n");
                     assert(0);
                  }

		  new_rule.body.push(idx);
	       }
	       else if (ret_term == rt_nonterm_id) {
		  unsigned idx = nonterminals.get_char_ptr_idx(input_idx - old_input_idx,a_data + old_input_idx);

                  // FX ERR
                  if (idx == c_idx_not_exist) {
                     fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: cannot resolve nonterminal symbol from rule body\n");
                     assert(0);
                  }

		  new_rule.body.push(terminals.used + idx);
	       }

	       // - oddeleni tela pravidla a jeho semantickeho kodu -
	       else {

                  // FX ERR
                  if (ret_term != rt_div_1 || new_rule.body.used <= 0) {
                     fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: wrong delimiter of rule body from its code, or rule body is empty\n");
                     assert(0);
                  }

		  rule_state = 3;
	       }
	    }

	    // - nacteni semantickeho kodu pravidla -
	    else {

               // FX ERR
               if (ret_term != rt_code) {
                  fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: expected code terminal\n");
                  assert(0);
               }

	       new_rule.sem_prog.set(input_idx - old_input_idx - 2,a_data + old_input_idx + 1);

               // FX ERR
               if (rules.get_idx(new_rule) != c_idx_not_exist) {
                  fprintf(stderr,"ERROR: p_creat_descr_s::load_from_rule_char_ptr: new rule is already defined\n");
                  assert(0);
               }

	       rules.push_blank();
	       rules.last().swap(new_rule);

	       rule_state = 0;
	    }
	 }

      } while(1);

      new_rule.clear();
   }

   //FIXME debug vypis
   /*printf("terminals:\n");
   terminals.print();
   printf("nonterminals:\n");
   nonterminals.print();
   printf("rules:\n");
   rules.print();*/

   return true;
}

/*
 * naplni srukturu p_creat_descr_s daty ze souboru pravidel jazyka
 */
inline bool p_creat_descr_s::load_from_rule_string(string_s &rule_string)
{
   return load_from_rule_char_ptr(rule_string.size - 1,rule_string.data);
}

/*
 * nalezne mezi terminaly klicove terminaly a ulozi je do jejich seznamu
 * TODO mozna by to slo zadratovat primo do nacitani souboru pravidel
 */
void p_creat_descr_s::find_key_terminals(unsigned &end_terminal,ui_array_s &skip_terminals)
{

   // FX ERR
   if (terminals.used == 0) {
      fprintf(stderr,"ERROR: p_creat_descr_s::find_key_terminals: no terminal symbols was defined\n");
      assert(0);
   }

   end_terminal = c_idx_not_exist;
   skip_terminals.used = 0;

   final_automata_s &final_automata = final_automata_set.key_terminals_fa;

   unsigned t_idx = 0;
   do {
      string_s &string = terminals[t_idx].idx;

      unsigned input_idx = 0;
      unsigned line_cnt = 0;
      unsigned input_length = string.size - 1;
      unsigned ret_term;

      ret_term = final_automata.recognize(string.data,input_idx,input_length,line_cnt);

      if (ret_term != c_idx_not_exist) {
	 if (ret_term == ktt_skip) {
	    skip_terminals.push(t_idx);
	 }
	 else if (ret_term == ktt_end) {

            // FX ERR FIXME
            if (end_terminal != c_idx_not_exist) {
               fprintf(stderr,"ERROR: p_creat_descr_s::find_key_terminals: ending terminal is already defined\n");
               assert(0);
            }

	    end_terminal = t_idx;
	 }
      }

   } while(++t_idx < terminals.used);
}

/*
 * vytvori konecny automat urceny pro rozpoznavani terminalu ze souboru pravidel
 */
void p_creat_descr_s::create_final_automata(final_automata_s &final_automata)
{

   // FX ERR
   if (terminals.used == 0) {
      fprintf(stderr,"ERROR: p_creat_descr_s::create_final_automata: no terminal symbols was defined\n");
      assert(0);
   }

   ushort_reg_exps_s ushort_reg_exps;
   ushort_reg_exps.init_size(terminals.used);

   {
      unsigned t_idx = 0;
      do {
	 ushort_reg_exps.push_blank();
	 ushort_reg_exps.last().load_char_ptr_reg_exp(terminals[t_idx].value.data);
      } while(++t_idx < terminals.used);
   }

   final_automata.clear();
   final_automata.create(ushort_reg_exps);
   
   ushort_reg_exps.clear();
}

void p_creat_descr_s::create_final_automata_new(final_automata_s &final_automata)
{

   // FX ERR
   if (terminals.used == 0) {
      fprintf(stderr,"ERROR: p_creat_descr_s::create_final_automata_new: no terminal symbols was defined\n");
      assert(0);
   }

   fa_states_array_s states_array;
   states_array.init();

   reg_parser_s reg_parser;
   reg_parser.init();

   {
      unsigned t_idx = 0;
      do {
	 reg_parser.process_reg_exp(terminals[t_idx].value);

	 states_array.push_blank();
	 states_array.last().swap(reg_parser.states);
      } while(++t_idx < terminals.used);
   }

   final_automata.clear();
   final_automata.create_new(states_array);

   reg_parser.clear();
   states_array.clear();
}

/*
 * spocita mnozinu firsts pro neterminaly
 * tj. mnozinu elementu se kterymi muze zacinat jakykoli literal rozepsatelny
 * z tohoto neterminalu
 */
void p_creat_descr_s::compute_firsts()
{

   // FX ERR
   if (terminals.used == 0 || nonterminals.used == 0 || rules.used == 0) {
      fprintf(stderr,"ERROR: p_creat_descr_s::compute_firsts: conditions for computing firsts sets are not met\n");
      assert(0);
   }

   firsts.used = 0;
   first_rules.used = 0;

   p_first_set_s first_set;
   first_set.init();

   p_rule_idxs_sets_s first_rule_idxs_sets;
   first_rule_idxs_sets.init();

   // - vypocet mnozin firsts pro nonterminalni symboly
   {
      unsigned non_idx = 0;

      do {
	 first_set.used = 0;
	 first_rule_idxs_sets.used = 0;

	 first_set.push(non_idx + terminals.used);
	 first_rule_idxs_sets.push_clear();

	 // - cyklus pres elementy v mnozine first -
	 unsigned fs_idx = 0;
	 do {
	    
	    // - pokud je element nonterminal -
	    if (first_set[fs_idx] >= terminals.used) {
	       unsigned r_idx = 0;

	       // - cyklus pres vsechny pravidla -
	       do {
		  p_rule_s &rule_ref = rules[r_idx];

		  // - kdyz je hlava pravidla stejna jako kontrolovany nonterminal -
		  if (rule_ref.head == first_set[fs_idx]) {
		     unsigned first_body = rule_ref.body[0];

		     unsigned first_body_idx = first_set.get_idx(first_body);
		     // - kdyz jeste neni element v mnozine first_set -
		     if (first_body_idx == c_idx_not_exist) {
			first_set.push(first_body);
			first_rule_idxs_sets.push_clear();
			first_rule_idxs_sets.last().push(r_idx);
		     }
		     else {
			first_rule_idxs_sets[first_body_idx].push(r_idx);
		     }
		  }
	       } while(++r_idx < rules.used);
	    }
	 } while(++fs_idx < first_set.used);

	 // - vlozeni mnoziny first nonterminalu do seznamu -
	 firsts.push_blank();
	 firsts.last().swap(first_set);

	 first_rules.push_blank();
	 first_rules.last().swap(first_rule_idxs_sets);
      } while(++non_idx < nonterminals.used);
   }

   first_set.clear();
   first_rule_idxs_sets.clear();
}

/*
 * spocita mnozinu follow jednoho neterminalniho symbolu
 */
bool p_creat_descr_s::compute_follows_of_nonterminal(unsigned nonterm_idx,bb_array_s &follows_created,ui_array_s &nonterm_used,p_follow_set_s &ret_follows)
{
   bool clean = true;
   ret_follows.used = 0;
   
   unsigned r_idx = 0;

   // - cyklus pres vsechny pravidla -
   do {
      p_rule_s &rule = rules[r_idx];
      unsigned rb_idx = 0;

      // - cyklus pres pravou stranu pravidla -
      do {
	 
	 // - kdyz je nalezen sledovany nonterminal -
	 if (nonterm_idx == rule.body[rb_idx]) {

	    // - pokud se nachazi uvnitr tela pravidla -
	    if (rb_idx < rule.body.used - 1) {
	       unsigned element = rule.body[rb_idx + 1];

	       // - kdyz je element terminal vloz jej do mnoziny follow -
	       if (element < terminals.used) {
		  if (ret_follows.get_idx(element) == c_idx_not_exist) {
		     ret_follows.push(element);
		  }
	       }

	       // - kdyz je element nonterminal vloz jeho first mnozinu do mnoziny follow -
	       else {
		  ret_follows.add_to_set(firsts[element - terminals.used]);
	       }
	    }

	    // - kdyz se nachazi na konci pravidla -
	    else {

	       // - pokud hlava pravidla jeste nebyla pouzita na rozepsani pouzij ji -
	       if (nonterm_used.get_idx(rule.head) == c_idx_not_exist) {

		  // - jestli jiz byla spocitana mnozina follow hlavy vloz ji do mnoziny follow 
		  // jinak ji spocitej a spocitej a zaznamenej vypocet jeji mnoziny follow pokud
		  // probehl ciste (tj. bez omezeni pocitani nekterych neterminalu) -
		  if (!follows_created[rule.head - terminals.used]) {
		     nonterm_used.push(rule.head);

		     bool this_clean = compute_follows_of_nonterminal(rule.head,follows_created,nonterm_used,follows[rule.head - terminals.used]);

		     if (this_clean) {
			follows_created[rule.head - terminals.used] = true;
		     }
		     else {
			clean = false;
		     }
		  }
		  
		  ret_follows.add_to_set(follows[rule.head - terminals.used]);
	       }

	       // - kdyz uz byla pouzita hlava pravidla na rozepsani nepokracuj ale oznac vypocet za necisty -
	       else {
		  clean = false;
	       }
	    }
	 }
      } while(++rb_idx < rule.body.used);
   } while(++r_idx < rules.used);

   return clean;
}

/*
 * spocita mnozinu follow ke vsem neterminalnim symbolum
 * tj. mnozinu obsahujici seznam elementu, ktere mohou v nejakem literalu
 * rozepsanem z pocatecniho symbolu gramatiky nasledovat za timto symbolem
 */
void p_creat_descr_s::compute_follows()
{

   // FX ERR
   if (terminals.used == 0 || nonterminals.used == 0 || rules.used == 0 || firsts.used == 0) {
      fprintf(stderr,"ERROR: p_creat_descr_s::compute_follows: conditions for computing of follows sets are not met\n");
      assert(0);
   }

   follows.used = 0;
   follows.copy_resize(nonterminals.used);

   {
      p_follow_set_s follow_set;
      follow_set.init();

      follows.fill(follow_set);

      follow_set.clear();
   }

   bb_array_s follows_created;
   follows_created.init_size(nonterminals.used);
   follows_created.fill(false);

   ui_array_s nonterm_used;
   nonterm_used.init();

   unsigned non_idx = 0;

   // - cyklus pres vsechny neterminaly -
   do {
      if (!follows_created[non_idx]) {
	 nonterm_used.used = 0;
	 nonterm_used.push(terminals.used + non_idx);

	 compute_follows_of_nonterminal(terminals.used + non_idx,follows_created,nonterm_used,follows[non_idx]);
      }

   } while(++non_idx < nonterminals.used);

   nonterm_used.clear();
   follows_created.clear();

   //follows.print();
}

/*
 * spocita jadra rozkladu prekladace podle nactenych pravidel
 * s vyuzitim mnozin firsts, first_rules a follows
 */
void p_creat_descr_s::compute_kernels()
{

   // FX ERR
   if (terminals.used == 0 || nonterminals.used == 0 || rules.used == 0 || firsts.used == 0 || follows.used == 0) {
      fprintf(stderr,"ERROR: p_creat_descr_s::compute_kernels: conditions for compuring parse kernels are not met\n");
      assert(0);
   }

   kernels.used = 0;

   // - vytvoreni prvniho jadra rozkladu obsahujiciho prvni pravidlo s iteratorem pred prvnim elementem pravidla -
   {
      kernels.push_clear();
      p_kernel_rule_dots_s &rule_dots = kernels.last().rule_dots;

      rule_dots.push_blank();
      rule_dots.last().set(0,0);
   }

   p_first_set_s terminal_first;
   terminal_first.init_size(1);
   terminal_first.used = 1;

   p_kernel_s new_kernel;
   new_kernel.init();

   unsigned k_idx = 0;

   // - cyklus pres jiz vytvorena jadra -
   do {

      // - pokud jadro obsahuje nejake pravidla -
      if (kernels[k_idx].rule_dots.used != 0) {
	 p_kernel_rule_dots_s *rule_dots = &kernels[k_idx].rule_dots;

	 unsigned rd_idx = 0;

	 // - cyklus pres vsechny pravidla jadra -
	 do {
	    p_kernel_rule_dot_s *rule_dot = rule_dots->data + rd_idx;

	    // - kdyz iterator neni na konci tela pravidla -
	    if (rule_dot->ui_second < rules[rule_dot->ui_first].body.used) {
	       unsigned element = rules[rule_dot->ui_first].body[rule_dot->ui_second];
	      
	       if (element < terminals.used) {
		  terminal_first.data[0] = element;
	       }

	       p_first_set_s &element_first = (element < terminals.used)?terminal_first:firsts[element - terminals.used];

	       if (element_first.used != 0) {
		  unsigned ef_idx = 0;

		  // - cyklus pres vsechny cleny v mnozine element_first -
		  do {
		     
		     // - vyvor nove jadro rozkladu -
		     p_kernel_rule_dots_s &nk_rule_dots = new_kernel.rule_dots;
		     nk_rule_dots.used = 0;

		     // - vloz vsechny pravidla ktera generuji element v mnozine first s iteratorem za timto
		     // prvnim elementem -
		     if (element >= terminals.used) {
			ui_array_s &ef_rules = first_rules[element - terminals.used][ef_idx];

			nk_rule_dots.copy_resize(ef_rules.used + 1);

			if (ef_rules.used != 0) {
			   unsigned _idx = 0;

			   do {
			      nk_rule_dots.data[_idx].ui_first = ef_rules[_idx];
			      nk_rule_dots.data[_idx].ui_second = 1;
			   } while(++_idx < ef_rules.used);
			   
			   nk_rule_dots.used = ef_rules.used;
			}
		     }

		     // - pokud je prvkem v mnozine first samotny element zkopiruj jeho pravidlo z aktualniho jadra
		     // a posun iterator za nej -
		     if (ef_idx == 0) {
		       
			//FIXME debug vypis
			//printf("%d -- %d\n",k_idx,element_first[ef_idx]);

			nk_rule_dots.push(kernels[k_idx].rule_dots[rd_idx]);
			nk_rule_dots.last().ui_second++;
		     }

		     // - do mnoziny prechodu vloz prechod ze zpracovavaneho jadra se zpracovavanym elementem 
		     // ve first -
		     p_kernel_gotos_s &gotos = new_kernel.gotos;
		     gotos.used = 0;
		     gotos.push_blank();
		     gotos.data[0].set(k_idx,element_first[ef_idx]);

		     // - prohledej jiz vygenerovane jadra -
		     unsigned tmp_k_idx = kernels.get_idx_by_rule_dots(new_kernel);

		     // - pokud neexistuje jadro se stejnou mnozinou rule_dots uloz nove -
		     if (tmp_k_idx == c_idx_not_exist) {
			kernels.push_blank();
			kernels.last().swap(new_kernel);

			// - obnoveni ukazatelu po mozne realokaci pole -
			rule_dots = &kernels[k_idx].rule_dots;
			rule_dot = rule_dots->data + rd_idx;
		     }

		     // - pokud jiz existuje takove jadro pridej do jeho mnoziny gotos vygenerovany prechod -
		     else {
			kernels[tmp_k_idx].gotos.push(gotos[0]);
		     }

		  } while(++ef_idx < element_first.used);
	       }
	    }
	 } while(++rd_idx < rule_dots->used);
      }
   } while(++k_idx < kernels.used);

   new_kernel.clear();
   terminal_first.clear();

   //FIXME debug vypis informaci o prave vytvorenych jadrech rozkladu
   /*{
      unsigned k_idx = 0;
      do {
	 printf("kernel %d:\n",k_idx);

	 p_kernel_rule_dots_s &rule_dots = kernels[k_idx].rule_dots;
	 p_kernel_gotos_s &gotos = kernels[k_idx].gotos;

	 unsigned rd_idx = 0;
	 do {
	    p_rule_s &rule = rules[rule_dots[rd_idx].ui_first];
	    
	    printf("%s -> ",nonterminals[rule.head - terminals.used].name.data);

	    unsigned rb_idx = 0;
	    do {
	       if (rb_idx == rule_dots[rd_idx].ui_second) {
		  printf("# ");
	       }

	       if (rule.body[rb_idx] < terminals.used) {
		  printf("%s ",terminals[rule.body[rb_idx]].idx.data);
	       }
	       else {
		  printf("%s ",nonterminals[rule.body[rb_idx] - terminals.used].name.data);
	       }
	    } while(++rb_idx < rule.body.used);

	    putchar('\n');

	 } while(++rd_idx < rule_dots.used);

	 putchar('\n');

	 if (gotos.used != 0) {
	    unsigned g_idx = 0;
	    do {
	       printf("from: %d ",gotos[g_idx].ui_first);

	       if (gotos[g_idx].ui_second < terminals.used) {
		  printf("with %s\n",terminals[gotos[g_idx].ui_second].idx.data);
	       }
	       else {
		  printf("with %s\n",nonterminals[gotos[g_idx].ui_second - terminals.used].name.data);
	       }
	    } while(++g_idx < gotos.used);
	 }

	 putchar('\n');

      } while(++k_idx < kernels.used);
   }*/
}

/*
 * vytvori rozkladovou tabulku na zaklade jader rozkladu
 */
void p_creat_descr_s::create_lalr_table(p_lalr_table_s &lalr_table)
{

   // FX ERR
   if (terminals.used == 0 || nonterminals.used == 0 || firsts.used == 0 || follows.used == 0 || kernels.used == 0) {
      fprintf(stderr,"ERROR: p_creat_descr_s::create_lalr_table: cannot create LALR table: conditions are not met\n");
      assert(0);
   }

   // - vytvori prazdnou tabulku -
   lalr_table.clear();
   lalr_table.resize(terminals.used + nonterminals.used,kernels.used);
   lalr_table.fill(c_idx_not_exist);

   {
      unsigned k_idx = 0;

      // - cyklus pres vsechny jadra rozkladu -
      do {
	 p_kernel_gotos_s &gotos = kernels[k_idx].gotos;

	 // - pokud pole gotos obsahuje nejake elementy -
	 if (gotos.used != 0) {
	    unsigned g_idx = 0;
	    
	    // - cyklus pres vsechny prvky v poli gotos -
	    do {
	       unsigned &value = lalr_table.value(gotos[g_idx].ui_second,gotos[g_idx].ui_first);

	       // - pokud je element nonterminal zapis do tabulky akci shift, jinak zapis do tabulky prechodu
	       // cislo stavu, na ktery se presunout po redukci-
	       if (gotos[g_idx].ui_second < terminals.used) {

                  // FX ERR
                  if (value != c_idx_not_exist) {
                     fprintf(stderr,"ERROR: p_creat_descr_s::create_lalr_table: SLR(1) parse table conflict\n");
                     assert(0);
                  }

		  value = k_idx;
	       }
	       else {

                  // FX ERR
                  if (value != c_idx_not_exist) {
                     fprintf(stderr,"ERROR: p_creat_descr_s::create_lalr_table: SLR(1) parse table conflict\n");
                     assert(0);
                  }

		  value = k_idx;
	       }
	    } while(++g_idx < gotos.used);
	 }
      } while(++k_idx < kernels.used);
   }

   {
      unsigned k_idx = 0;

      do {

	 p_kernel_rule_dots_s &rule_dots = kernels[k_idx].rule_dots;
	 unsigned rd_idx = 0;
	 
	 // - cyklus pres vsechny pravidla v jadre  -
	 do {
	    p_kernel_rule_dot_s &rule_dot = rule_dots[rd_idx];

	    // - pokud se iterator nachazi na konci tela pravidla -
	    if (rules[rule_dot.ui_first].body.used == rule_dot.ui_second) {

	       p_follow_set_s &head_follow = follows[rules[rule_dot.ui_first].head - terminals.used];

	       // - pokud neni follow mnozina hlavy pravidla prazdna -
	       if (head_follow.used != 0) {
		  unsigned hf_idx = 0;

		  // - cyklus pres vsechny prvky v mnozine head_follow -
		  do {
		     
		     // - pokud je element terminalni symbol -
		     if (head_follow[hf_idx] < terminals.used) {
			bool good = true;
			unsigned rd2_idx = 0;

			// - Kontrola zda neexistuje v jadre takove pravidlo, ktere by umoznovalo akci shift tj. redukce by byla predcasna. Pokud ne good == true -
			do {
			   p_kernel_rule_dot_s &rule_dot2 = rule_dots[rd2_idx];

			   if (rules[rule_dot2.ui_first].body.used > rule_dot2.ui_second) {
			      if (head_follow[hf_idx] == rules[rule_dot2.ui_first].body[rule_dot2.ui_second]) {
				 good = false;
				 break;
			      }
			   }
			} while(++rd2_idx < rule_dots.used);

			// - pokud good zapise do tabulky redukci podle pravidla rule_dots[rd_idx] -
			if (good) {
			   unsigned &value = lalr_table.value(head_follow[hf_idx],k_idx);

                           // FX ERR
                           if (value != c_idx_not_exist) {
                              fprintf(stderr,"ERROR: p_creat_descr_s::create_lalr_table: SLR(1) parse table conflict\n");
                              assert(0);
                           }

			   value = c_lalr_table_reduce_base + rule_dots[rd_idx].ui_first;
			}
			else {
			   unsigned &value = lalr_table.value(head_follow[hf_idx],k_idx);

                           // FX ERR
                           if (value == c_idx_not_exist) {
                              fprintf(stderr,"ERROR: p_creat_descr_s::create_lalr_table: cannot resolve target rule head for shift action\n");
                              assert(0);
                           }

			}
		     }
		  } while(++hf_idx < head_follow.used);
	       }
	    }
	 } while(++rd_idx < rule_dots.used);
      } while(++k_idx < kernels.used);
   }

   //FIXME debug vypis prave vytvorene rozkladove tabulky na stdout
   /*{
      const unsigned tab_size = 5;

      ui_array_s tabs;
      tabs.init();

      // prvni mezera
      {
	 unsigned _idx = 0;
	 do {
	    putchar(' ');
	 } while(++_idx < tab_size);

	 tabs.push(tab_size);
      }

      unsigned t_idx = 0;
      do {
	 printf("%s",terminals[t_idx].idx.data);

	 unsigned term_length = terminals[t_idx].idx.size - 1;
	 unsigned ntl = (term_length/tab_size + 1) * tab_size;

	 if (ntl > term_length) {
	    unsigned _idx = 0;
	    do {
	       putchar(' ');
	    } while(++_idx < ntl - term_length);
	 }
	 
	 tabs.push(ntl);
      } while(++t_idx < terminals.used);

      unsigned n_idx = 0;
      do {
	 printf("%s",nonterminals[n_idx].name.data);

	 unsigned nonterm_length = nonterminals[n_idx].name.size - 1;
	 unsigned nnl = (nonterm_length/tab_size + 1) * tab_size;

	 if (nnl > nonterm_length) {
	    unsigned _idx = 0;
	    do {
	       putchar(' ');
	    } while(++_idx < nnl - nonterm_length);
	 }

	 tabs.push(nnl);
      } while(++n_idx < nonterminals.used);
      
      putchar('\n');

      char txt_buff[256];
      unsigned tb_length;

      unsigned l_idx = 0;
      do {
	 tb_length = sprintf(txt_buff,"%d",l_idx);
	 printf("%s",txt_buff);

	 unsigned c_idx = 0;
	 do {
	    {
	       unsigned next_tab = tabs[c_idx];
	       unsigned _idx = tb_length;
	       do {
		  putchar(' ');
	       } while(++_idx < next_tab);
	    }
	    
	    unsigned value = lalr_table.value(c_idx,l_idx);
	    
	    if (value != c_idx_not_exist) {
	       if (c_idx < terminals.used) {
		  if (value < c_lalr_table_reduce_base) {
		     tb_length = sprintf(txt_buff,"s%d",value);
		  }
		  else {
		     tb_length = sprintf(txt_buff,"r%d",value - c_lalr_table_reduce_base);
		  }
	       }
	       else {
		     tb_length = sprintf(txt_buff,"%d",value);
	       }

	       printf("%s",txt_buff);
	    }
	    else {
	       tb_length = 0;
	    }

	 } while(++c_idx < lalr_table.x_size);

	 putchar('\n');
      } while(++l_idx < lalr_table.y_size);

      tabs.clear();
   }*/
}

// -- funkce struktury p_semantic_parser_s --

/*
 * podle konfigurace nacte ze souboru, nebo znovu vytvori prekladac semantickych
 * podprogramu z retezce
 */
bool p_semantic_parser_s::load_or_create(p_creat_descr_s &creat_descr)
{
#ifndef p_semantic_parser_s_load_or_create_CREATE
   return load_file("tmp_files/sem_parser.dat");
#else
   clear();

   // - nacte terminaly, neterminaly a pravidla z retezce -
   if (!(creat_descr.load_from_rule_char_ptr(strlen(semantic_parser_string),(char *)semantic_parser_string))) {
      return false;
   }

   creat_descr.find_key_terminals(end_terminal,skip_terminals);
   creat_descr.create_final_automata_new(final_automata);
   creat_descr.compute_firsts();
   creat_descr.compute_follows();
   creat_descr.compute_kernels();
   creat_descr.create_lalr_table(lalr_table);

   // - vytvoreni popisu pravidel do semantickeho parseru -
   {
      
      // - konecny automat pro rozpoznani instrukci pravidel semantickeho programu -
      final_automata_s sem_rule_fa;
      sem_rule_fa.init();

      // - vytvoreni automatu -
      {
	 ushort_reg_exps_s reg_exps;
	 reg_exps.init();

	 reg_exps.load_char_ptrs_reg_exps(c_sem_terminal_cnt,(char **)sem_terminals);
	 sem_rule_fa.create(reg_exps);
	 reg_exps.clear();
      }

      p_rules_s &rules = creat_descr.rules;
      sem_rule_descrs.used = 0;

      // - vyvoreni popisu pravidel -
      if (rules.used != 0) {
	 sem_rule_descrs.copy_resize(rules.used);

	 unsigned r_idx = 0;
	 do {
	    p_sem_rule_descr_s &sem_rule_descr = sem_rule_descrs.data[r_idx];
	    p_rule_s &rule = rules[r_idx];

	    sem_rule_descr.ui_first = rule.head;
	    sem_rule_descr.ui_second = rule.body.used;

	    unsigned input_idx = 0;
	    unsigned line_cnt = 0;
	    unsigned input_length = rule.sem_prog.size - 1;

	    // - celociselny semanticky kod pravidla podle hodnoty vracene automatem -
	    sem_rule_descr.ui_third = sem_rule_fa.recognize(rule.sem_prog.data,input_idx,input_length,line_cnt);

	 } while(++r_idx < rules.used);
	 sem_rule_descrs.used = rules.used;
      }

      sem_rule_fa.clear();
   }

   return save_file("tmp_files/sem_parser.dat");
#endif
}

/*
 * vlozeni funkci popisujici semanticke podprogramy, prekladace semantickych podprogramu
 * z jineho souboru pro prehlednost
 */
%%include parse_sem_instr.cc

/*
 * prelozi semanticky program do toku instrukci, interpretovatelnych pri rozkladu 
 * jazyka
 */
void p_semantic_parser_s::parse_sem_string(buffer_s &sem_code,p_sem_parse_data_s &sem_parse_data)
{
   // - ziskani reference na prekladany program -
   string_s &source_string = *((string_s *)sem_parse_data.source_string_ptr);

   // - vyprazdneni semantickeho kodu -
   sem_code.used = 0;

   // - vychozi nastaveni lalr_stavoveho zasobniku -
   p_lalr_stack_s &lalr_stack = sem_parse_data.lalr_stack;
   lalr_stack.used = 0;
   lalr_stack.push_blank();
   lalr_stack.last().ui_first = 0;

   // - vychozi nastaveni zasobniku typu promennych -
   sem_parse_data.var_type_stack.used = 0;

   // - vychozi nastaveni zasobniku pro ulozeni popisu definovanych poli -
   sem_parse_data.sem_const_array_types.used = 0;

   // - promenna popisujici hodnotu definovaneho typu -
   sem_parse_data.var_type = svt_int;

   // - promenne popisujici stav konecneho lexikalniho automatu -
   unsigned old_input_idx = 0;
   unsigned input_idx = 0;
   unsigned line_cnt = 1;
   unsigned input_length = source_string.size - 1;
   unsigned ret_term = c_idx_not_exist;

   do {
      
      // - rozpoznani dalsiho terminalu na vstupu -
      while (ret_term == c_idx_not_exist) {
	 old_input_idx = input_idx;

	 ret_term = final_automata.recognize(source_string.data,input_idx,input_length,line_cnt);

         // FX ERR
         if (ret_term == c_idx_not_exist) {
            fprintf(stderr,"ERROR: p_semantic_parser_s::parse_sem_string: cannot recognize terminal symbol\n");
            assert(0);
         }

	 if (skip_terminals.get_idx(ret_term) != c_idx_not_exist) {
	    ret_term = c_idx_not_exist;
	 }
      }

      // - nalezeni akce v tabulce akci -
      unsigned action = lalr_table.value(ret_term,lalr_stack.last().ui_first);
      
      // FX ERR
      if (action == c_idx_not_exist) {
         fprintf(stderr,"ERROR: p_semantic_parser_s::parse_sem_string: cannot find parse action\n");
         assert(0);
      }

      // - akce SHIFT -
      if (action  < c_lalr_table_reduce_base) {

	 // - kdyz byl prijmut koncovy terminal ukonci rozklad -
	 if (ret_term == end_terminal) {
	    break;
	 }

	 // - vlozi na zasobnik novy stav a pozici terminalu ve zdrojovem retezci -
	 lalr_stack.push_blank();
	 lalr_stack.last().set(action,old_input_idx,input_idx);

	 ret_term = c_idx_not_exist;
      }

      // - akce REDUCE -
      else {
	 action -= c_lalr_table_reduce_base;
	
	 // - zpracovani instrukce semantickeho kodu -
	 if (sem_rule_descrs[action].ui_third != c_idx_not_exist) {
	    parse_sem_instr_callers[sem_rule_descrs[action].ui_third](sem_code,sem_parse_data);
	 }

	 p_sem_rule_descr_s &rule_descr = sem_rule_descrs[action];
	 lalr_stack.used -= rule_descr.ui_second;

	 unsigned goto_val = lalr_table.value(rule_descr.ui_first,lalr_stack.last().ui_first);

	 // - vlozeni noveho stavu na zasobnik po redukci -
	 lalr_stack.push_blank();
	 lalr_stack.last().ui_first = goto_val;
      }

   } while(1);

   // - kontrola vyskytu chyby -

   // FX ERR
   if (sem_parse_data.var_type_stack.used != 0) {
      fprintf(stderr,"ERROR: p_semantic_parser_s::parse_sem_string: internal error - variable type stack is not empty\n");
      assert(0);
   }
}

// -- funkce struktury program_generator_s --

bool program_generator_s::create_from_rule_string(string_s &rule_string)
{
   //#define RUN_TEST

   if (rule_string.size == 0) {
      return false;
   }

   clear();

   p_creat_descr_s creat_descr;
   creat_descr.init();

   // - nacteni konecnych automatu vyuzivanych ke zpracovani souboru pravidel -
   creat_descr.load_final_automata_set_new("tmp_files/creat_desct_automatas.sav");

   // - nacteni terminalu, neterminalu a pravidel z retezce -
   if (!creat_descr.load_from_rule_string(rule_string)) {
      return false;
   }

   p_terminals_s &terminals = creat_descr.terminals;
   p_nonterminals_s &nonterminals = creat_descr.nonterminals;
   p_rules_s &rules = creat_descr.rules;

   ui_arrays_s head_rules_array;

   // - vytvoreni seznamu indexu pravidel podle hlavy pravidla a pravdepodobnosti rozgenerovani -
   {
      head_rules_array.init_size(nonterminals.used);
      head_rules_array.used = nonterminals.used;

      unsigned r_idx = 0;
      do {
	 p_rule_s &rule = rules[r_idx];
	 ui_array_s &head_rules = head_rules_array[rule.head - terminals.used];

	 // - ulozeni indexu pravidla, nasobne podle poctu vyskytu -
	 unsigned rule_cnt = atoi(rule.sem_prog.data);
	 do {
	    head_rules.push(r_idx);
	 } while(--rule_cnt > 0);

      } while(++r_idx < rules.used);
   }

   // - generovani nahodneho programu podle gramatiky -

#ifdef RUN_TEST
   ui_array_s gen_stack;
   gen_stack.init();

   unsigned repeat = 50;
   do {

      printf("\n -------------------------------- \n");

      // - umisteni prvniho neterminalu na zasobnik -
      gen_stack.push(terminals.used);

      unsigned der_cnt = 0;

      // - prace nad rozkladovym zasobnikem -
      do {
	 unsigned stack_element = gen_stack.pop();

	 if (stack_element < terminals.used) {
	    printf("%s ",terminals[stack_element].value.data);
	 }
	 else {
	    ui_array_s &head_rules = head_rules_array[stack_element - terminals.used];
	    unsigned rule_idx;

	    // - vyber pravidla s omezenim, na pozici 0 se nachazeji ukoncovaci pravidla -
	    if (der_cnt > 500) {
	       rule_idx = head_rules[0];
	    }
	    else {
	       rule_idx = head_rules[rand()%head_rules.used];
	       der_cnt++;
	    }
	    
	    p_rule_s &rule = rules[rule_idx];

	    unsigned bi_idx = rule.body.used;
	    do {
	       bi_idx--;
	       gen_stack.push(rule.body[bi_idx]);
	    } while(bi_idx > 0);
	 }

      } while(gen_stack.used > 0);

      putchar('\n');
   } while(--repeat > 0);

   gen_stack.clear();

#else
   // - generovani zdrojoveho souboru popisujiciho generator -

   // - hlaviskove soubory -
   printf("\n#include <stdio.h>\n");
   printf("#include <stdlib.h>\n\n");

   printf("const unsigned terminal_cnt = %d;\n",terminals.used);

   // - terminals strings creation -
   {
      printf("const char *terminal_strings[] = {\n");

      unsigned t_idx = 0;
      do {
	 printf("   \"%s\",\n",terminals[t_idx].value.data);
      } while(++t_idx < terminals.used);

      printf("};\n\n");
   }

   // - rule body lengths and positions -
   {
      ui_array_s rule_positions;
      rule_positions.init();

      unsigned rule_position = 0;
      printf("const unsigned rule_sizes[%d] = { ",rules.used);

      unsigned r_idx = 0;
      do {
	 p_rule_s &rule = rules[r_idx];
      
	 printf("%d, ",rule.body.used);

	 rule_positions.push(rule_position);
	 rule_position += rule.body.used;
      } while(++r_idx < rules.used);

      printf("};\n");

      printf("const unsigned rule_positions[%d] = { ",rules.used);

      unsigned rp_idx = 0;
      do {
	 printf("%d, ",rule_positions[rp_idx]);
      } while(++rp_idx < rule_positions.used);

      printf("};\n");

      rule_positions.clear();
   }

   // - rule bodies -
   {
      printf("const unsigned rule_bodies[] = {\n");
      
      unsigned r_idx = 0;
      do {
	 printf("   ");

	 ui_array_s &body = rules[r_idx].body;
	 unsigned b_idx = 0;
	 do {
	    printf("%d, ",body[b_idx]);
	 } while(++b_idx < body.used);

	 printf("\n");
      } while(++r_idx < rules.used);

      printf("};\n\n");
   }

   // - head rule cnts and positions -
   {
      ui_array_s hr_positions;
      hr_positions.init();

      unsigned hr_position = 0;
      printf("const unsigned head_rule_cnts[%d] = { ",head_rules_array.used);

      unsigned h_idx = 0;
      do {
	 printf("%d, ",head_rules_array[h_idx].used);

	 hr_positions.push(hr_position);
	 hr_position += head_rules_array[h_idx].used;
      } while(++h_idx < head_rules_array.used);

      printf("};\n");

      printf("const unsigned head_rule_positions[%d] = { ",head_rules_array.used);

      unsigned hrp_idx = 0;
      do {
	 printf("%d, ",hr_positions[hrp_idx]);
      } while(++hrp_idx < hr_positions.used);

      printf("};\n");

      hr_positions.clear();
   }

   // - head rules array creation -
   {
      printf("const unsigned head_rules_array[] = {\n");
      
      unsigned h_idx = 0;
      do {
	 printf("   ");
      
	 ui_array_s &head_rules = head_rules_array[h_idx];
	 unsigned hr_idx = 0;
	 do {
	    printf("%d, ",head_rules[hr_idx]);
	 } while(++hr_idx < head_rules.used);

	 printf("\n");
      } while(++h_idx < head_rules_array.used);

      printf("};\n\n");
   }
#endif

   head_rules_array.clear();
   creat_descr.clear();
   return true;
}

// -- funkce struktury parser_s --

/*
 * vytvori popis prekladace z nacteneho souboru pravidel
 * tj. vytvori:
 *    inicializacni semanticky kod
 *    popis pravidel vcetne semantickych kodu
 *    konecny automat - rozpoznavajici terminaly ve zdrojovem programu
 *    rozkladovou tabulku - pro syntaktickou analyzu
 */
bool parser_s::create_from_rule_string(string_s &rule_string)
{
   if (rule_string.size == 0) {
      return false;
   }

   clear();

   p_creat_descr_s creat_descr;
   creat_descr.init();

   // - nacteni konecnych automatu vyuzivanych ke zpracovani souboru pravidel -
   creat_descr.load_final_automata_set_new("tmp_files/creat_desct_automatas.sav");

   // - nacteni terminalu, neterminalu a pravidel z retezce -
   if (!creat_descr.load_from_rule_string(rule_string)) {
      return false;
   }

   // - ulozeni do parseru pocet terminalu gramatiky -
   terminal_cnt = creat_descr.terminals.used;

   creat_descr.find_key_terminals(end_terminal,skip_terminals);
   //INFO zmena zpusobu prekladu popisu tvaru terminalnich symbolu
   //creat_descr.create_final_automata(final_automata);
   creat_descr.create_final_automata_new(final_automata);
   creat_descr.compute_firsts();
   creat_descr.compute_follows();
   creat_descr.compute_kernels();
   creat_descr.create_lalr_table(lalr_table);

   // - vytvoreni popisu pravidel do parseru -
   {
      // - pole retezcu urcene pro ulozeni semantickych kodu pravidel -
      string_array_s rule_sem_progs;
      rule_sem_progs.init_size(creat_descr.rules.used);

      p_rules_s &rules = creat_descr.rules;
      rule_descrs.used = 0;

      if (rules.used != 0) {
	 rule_descrs.copy_resize(rules.used);

	 unsigned r_idx = 0;

	 // - cyklus pres vsechny pravidla programu -
	 do {
	    p_rule_descr_s &rule_descr = rule_descrs.data[r_idx];
	    p_rule_s &rule = rules[r_idx];

	    rule_descr.ui_first = rule.head;
	    rule_descr.ui_second = rule.body.used;
	    
	    // - zkopiruje semanticky program pravidla -
	    rule_sem_progs.data[r_idx].swap(rule.sem_prog);

	 } while(++r_idx < rules.used);
	 rule_descrs.used = rules.used;
	 rule_sem_progs.used = rules.used;
      }

      // - inicializacni semanticky program -
      string_s init_sem_prog;
      init_sem_prog.init();
      init_sem_prog.swap(creat_descr.init_sem_prog);

      // - data semantickeho prekladace vyuzivana pri rozkladu -
      p_sem_parse_data_s sem_parse_data;
      sem_parse_data.init();

      // - nacte semanticky prekladac ze souboru -
      p_semantic_parser_s semantic_parser;
      semantic_parser.init();
      semantic_parser.load_or_create(creat_descr);

      // - upraveni velikosti pole poli promennych podle poctu datovych typu sem prekladace a jejich vytvoreni -
      {
	 p_sem_variable_names_array_s &sem_variable_names_array = sem_parse_data.sem_variable_names_array;
	 sem_variable_names_array.copy_resize(c_sem_var_type_cnt);

	 unsigned vt_idx = 0;
	 do {
	    sem_variable_names_array.push_blank();
	 } while(++vt_idx < c_sem_var_type_cnt);
      }

      // - preklad inicializacniho semantickeho programu -
      sem_parse_data.source_string_ptr = (pointer)&init_sem_prog;
      semantic_parser.parse_sem_string(init_sem_code,sem_parse_data);

      // - cyklus pres vsechny semanticke programy pravidel a jejich preklad -
      unsigned rsc_idx = 0;

      do {

	 // - prelozeni semantickeho programu -
	 sem_parse_data.source_string_ptr = (pointer)&rule_sem_progs[rsc_idx];
	 semantic_parser.parse_sem_string(rule_descrs[rsc_idx].sem_code,sem_parse_data);
      } while(++rsc_idx < rule_sem_progs.used);

      // - ulozeni poctu semantickych promennych jednotlivych typu pro pouziti parserem -
      sem_variable_cnts.used = 0;
      sem_variable_cnts.copy_resize(c_sem_var_type_cnt);
      
      {
	 p_sem_variable_names_array_s &sem_variable_names_array = sem_parse_data.sem_variable_names_array;

	 unsigned svn_idx = 0;
	 do {
	    sem_variable_cnts.push(sem_variable_names_array[svn_idx].used);
	 } while(++svn_idx < c_sem_var_type_cnt);
      }

      // - ziskani hodnot konstant, semantickych programu, krome integeru ty jsou napevno v kodu -
      string_constants.swap(sem_parse_data.string_constants);
      int_array_constants.swap(sem_parse_data.int_array_constants);
      string_array_constants.swap(sem_parse_data.string_array_constants);
      int_array_array_constants.swap(sem_parse_data.int_array_array_constants);
      string_array_array_constants.swap(sem_parse_data.string_array_array_constants);

      // - uvolneni docasnych promennych -
      sem_parse_data.clear();
      semantic_parser.clear();
      init_sem_prog.clear();
      rule_sem_progs.clear();
   }

   creat_descr.clear();
   return true;
}

/*
 * vlozeni funkci vykonavajicich instrukce semantickych podprogramu
 */
%%include run_sem_instr.cc

/*
 * spusti semanticky program predany parametrem
 * tj. vytvori iterator ukazujici na pozici v kodu, nacita jednotlive instrukce
 * a spousti podprogramy k nim prirazene
 */
void parser_s::run_semantic_code(p_parse_data_s &parse_data)
{
   // - ziskani reference na semanticky kod -
   buffer_s &sem_code = *(buffer_s *)parse_data.sem_code_ptr;

   // FX ERR
   if (sem_code.used == 0) {
      fprintf(stderr,"ERROR: parser_s::run_semantic_code: semantic code is empty\n");
      assert(0);
   }

   // - priprava zasobniku pro spusteni semantickeho kodu -
   parse_data.sem_run_stack.used = 0;

   // - pruchod pres vsechny instrukce a jejich spousteni -
   unsigned &sc_idx = parse_data.sc_idx;
   sc_idx = 0;

   do {
      parse_instr_callers[sem_code.get_sem_instruction(sc_idx)](parse_data);
   } while(sc_idx < sem_code.used);

   // FX ERR
   if (parse_data.sem_run_stack.used != 0) {
      fprintf(stderr,"ERROR: parser_s::run_semantic_code: internal error - semantic code running stack is not empty\n");
      assert(0);
   }
}

/*
 * provede rozklad zdrojoveho retezce
 * vytvori zasobnik pro ulozeni aktualni pozice rozkladu a vlozi na nej 0 - stav nula zacatek rozkladu
 * nalezne akci v rozkl. tabulce podle stavu na zasobniku a vstupniho terminalu
 * SHIFT - presune terminal, tj. pouze stav a pozici terminalu v zdrojovem retezci na zasobnik
 *	 - pokud je terminal tzv. end_terminal rozklad se ukonci
 * REDUCE - redukuje podle pravidla v tabulce, spusti semanticky kod prirazeny k pravidlu 
 *        - nalezne novy stav po redukci a ulozi jej na zasobnik
 * provadi dokud se rozklad neukonci
 */
void parser_s::parse_source_string(string_s &source_string)
{
   // - data potrebna pro rozklad programu -
   p_parse_data_s parse_data;
   parse_data.init();

   // - prirazeni ukazatele na zdrojovy retezec a parser k parse datum -
   parse_data.source_string_ptr = (pointer)&source_string;
   parse_data.parser_ptr = (pointer)this;

   // - vychozi nastaveni lalr_stavoveho zasobniku -
   p_lalr_stack_s &lalr_stack = parse_data.lalr_stack;
   lalr_stack.push_blank();
   lalr_stack.last().ui_first = 0;

   // - reference na promennou popisujici akci nalezenou v rozkladove tabulce -
   unsigned &action = parse_data.action;

   // - reference na promennou poctu radku nactenych ze zdrojoveho souboru -
   unsigned &line_cnt = parse_data.line_cnt;
   line_cnt = 1;

   // - pripraveni promennych pro semanticke programy -
   parse_data.int_variables.copy_resize(sem_variable_cnts[svt_int]);
   parse_data.int_variables.used = sem_variable_cnts[svt_int];

   parse_data.string_variables.copy_resize(sem_variable_cnts[svt_string]);
   parse_data.string_variables.used = sem_variable_cnts[svt_string];

   parse_data.int_array_variables.copy_resize(sem_variable_cnts[svt_int_array]);
   parse_data.int_array_variables.used = sem_variable_cnts[svt_int_array];

   parse_data.string_array_variables.copy_resize(sem_variable_cnts[svt_string_array]);
   parse_data.string_array_variables.used = sem_variable_cnts[svt_string_array];

   parse_data.int_array_array_variables.copy_resize(sem_variable_cnts[svt_int_array_array]);
   parse_data.int_array_array_variables.used = sem_variable_cnts[svt_int_array_array];

   parse_data.string_array_array_variables.copy_resize(sem_variable_cnts[svt_string_array_array]);
   parse_data.string_array_array_variables.used = sem_variable_cnts[svt_string_array_array];

   // - spusteni inicializacniho semantickeho programu -
   if (init_sem_code.used != 0) {
      parse_data.action = c_idx_not_exist;
      parse_data.sem_code_ptr = (pointer)&init_sem_code;
      run_semantic_code(parse_data);
   }

   // - promenne popisujici stav konecneho lexikalniho automatu -
   unsigned old_input_idx = 0;
   unsigned input_idx = 0;
   //unsigned line_cnt = 1;
   unsigned input_length = source_string.size - 1;
   unsigned ret_term = c_idx_not_exist;

   do {
      
      // - rozpoznani dalsiho terminalu na vstupu -
      while (ret_term == c_idx_not_exist) {
	 old_input_idx = input_idx;

	 ret_term = final_automata.recognize(source_string.data,input_idx,input_length,line_cnt);

         // FX ERR
         if (ret_term == c_idx_not_exist) {
            fprintf(stderr,"ERROR: parser_s::parse_source_string: unrecognized terminal symbol\n");
            assert(0);
         }

	 if (skip_terminals.get_idx(ret_term) != c_idx_not_exist) {
	    ret_term = c_idx_not_exist;
	 }
      }

      // - nalezeni akce v tabulce akci -
      action = lalr_table.value(ret_term,lalr_stack.last().ui_first);

      // FX ERR
      if (action == c_idx_not_exist) {
         fprintf(stderr,"ERROR: parser_s::parse_source_string: cannot find proper parse action\n");
         assert(0);
      }

      // - akce SHIFT -
      if (action  < c_lalr_table_reduce_base) {

	 // - kdyz byl prijmut koncovy terminal ukonci rozklad -
	 if (ret_term == end_terminal) {
	    break;
	 }

	 // - vlozi na zasobnik novy stav a pozici terminalu ve zdrojovem retezci -
	 lalr_stack.push_blank();
	 lalr_stack.last().set(action,old_input_idx,input_idx);

	 ret_term = c_idx_not_exist;
      }

      // - akce REDUCE -
      else {
	 action -= c_lalr_table_reduce_base;

	 // - spusteni semantickeho kodu prirazeneho k pravidlu -
	 {
	    buffer_s &sem_code = rule_descrs[action].sem_code;

	    if (sem_code.used != 0) {
	       parse_data.sem_code_ptr = (pointer)&sem_code;
	       run_semantic_code(parse_data);
	    }
	 }

	 p_rule_descr_s &rule_descr = rule_descrs[action];
	 lalr_stack.used -= rule_descr.ui_second;

	 unsigned goto_val = lalr_table.value(rule_descr.ui_first,lalr_stack.last().ui_first);

	 lalr_stack.push_blank();
	 lalr_stack.last().ui_first = goto_val;
      }
   } while(1);

   parse_data.clear();
}

/*
 * vygeneruje do posloupnosti znaku struktury buffer_s
 * zdrojovy kod prekladace podle popisu tohoto prekladace ve strukture parser_s
 * tento kod je zapsan v jazyce c++
 */
void parser_s::create_cc_source(buffer_s &cc_source)
{

%%include cc_creator.cc 

}

