
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

/*
 * seznam podprogramu spoustenych jednotlivymi instrukce semantickeho kodu.
 * kazdy funguje nasledujicim zpusobem
 * - vyzvedne ze zasobniku potrebny pocet operandu
 * - provede pozadovanou operaci
 * - ulozi vysledek (za predpokladu ze to je jeho ukolem) na zasobnik
 */

/*
 * nasledujici makra usnadnuji ziskani ukazatelu na reference do poli.
 * napr array[0][1]
 */

// - ziskani ukazatele na unsigned aref podle hodnot na zasobniku -
#define GET_UNSIGNED_AREF(TRG_PTR,STACK_OFFSET) \
{\
   unsigned a_idx = sem_run_stack.__tail(STACK_OFFSET);\
   ui_array_s &unsigned_array = int_array_variables[sem_run_stack.__tail(STACK_OFFSET + 1)];\
   assert(a_idx < unsigned_array.used);\
\
   TRG_PTR = &unsigned_array[a_idx];\
}

// - ziskani ukazatele na unsigned aref podle hodnot na zasobniku a odstraneni dat ze zasobniku -
#define POP_UNSIGNED_AREF(TRG_PTR) \
{\
   GET_UNSIGNED_AREF(TRG_PTR,0);\
   sem_run_stack.used -= 2;\
}

// - ziskani ukazatele na string_s aref podle hodnot na zasobniku - 
#define GET_STRING_AREF(TRG_PTR,STACK_OFFSET) \
{\
   unsigned a_idx = sem_run_stack.__tail(STACK_OFFSET);\
   string_array_s &string_array = string_array_variables[sem_run_stack.__tail(STACK_OFFSET + 1)];\
   assert(a_idx < string_array.used);\
\
   TRG_PTR = &string_array[a_idx];\
}

// - ziskani ukazatele na string_s aref podle hodnot na zasobniku a odstraneni dat ze zasobniku - 
#define POP_STRING_AREF(TRG_PTR) \
{\
   GET_STRING_AREF(TRG_PTR,0);\
   sem_run_stack.used -= 2;\
}

// - ziska ukazatele na unsigned pole aref podle hodnot na zasobniku -
#define GET_UNSIGNED_ARRAY_AREF(TRG_PTR,STACK_OFFSET) \
{\
   unsigned a_idx = sem_run_stack.__tail(STACK_OFFSET);\
   ui_arrays_s &unsigned_array_array = int_array_array_variables[sem_run_stack.__tail(STACK_OFFSET + 1)];\
   assert(a_idx < unsigned_array_array.used);\
\
   TRG_PTR = &unsigned_array_array[a_idx];\
}

// - ziska ukazatele na unsigned pole aref podle hodnot na zasobniku a odstrani data ze zasobniku -
#define POP_UNSIGNED_ARRAY_AREF(TRG_PTR) \
{\
   GET_UNSIGNED_ARRAY_AREF(TRG_PTR,0);\
   sem_run_stack.used -= 2;\
}

// - ziska ukazatele na string pole aref podle hodnot na zasobniku -
#define GET_STRING_ARRAY_AREF(TRG_PTR,STACK_OFFSET) \
{\
   unsigned a_idx = sem_run_stack.__tail(STACK_OFFSET);\
   string_arrays_s &string_array_array = string_array_array_variables[sem_run_stack.__tail(STACK_OFFSET + 1)];\
   assert(a_idx < string_array_array.used);\
\
   TRG_PTR = &string_array_array[a_idx];\
}

// - ziska ukazatele na string pole aref podle hodnot na zasobniku a odstrani data ze zasobniku-
#define POP_STRING_ARRAY_AREF(TRG_PTR) \
{\
   GET_STRING_ARRAY_AREF(TRG_PTR,0);\
   sem_run_stack.used -= 2;\
}

// - ziskani ukazatele na unsigned aaref podle hodnot na zasobniku -
#define GET_UNSIGNED_AAREF(TRG_PTR,STACK_OFFSET) \
{\
   unsigned sa_idx = sem_run_stack.__tail(STACK_OFFSET);\
   unsigned fa_idx = sem_run_stack.__tail(STACK_OFFSET + 1);\
   ui_arrays_s &unsigned_array_array = int_array_array_variables[sem_run_stack.__tail(STACK_OFFSET + 2)];\
   assert(fa_idx < unsigned_array_array.used);\
   ui_array_s &unsigned_array = unsigned_array_array[fa_idx];\
   assert(sa_idx < unsigned_array.used);\
\
   TRG_PTR = &unsigned_array[sa_idx];\
}

// - ziskani ukazatele na unsigned aaref podle hodnot na zasobniku a odstraneni dat ze zasobniku - 
#define POP_UNSIGNED_AAREF(TRG_PTR) \
{\
   GET_UNSIGNED_AAREF(TRG_PTR,0);\
   sem_run_stack.used -= 3;\
}

// - ziskani ukazatele na string_s aaref podle hodnot na zasobniku -
#define GET_STRING_AAREF(TRG_PTR,STACK_OFFSET) \
{\
   unsigned sa_idx = sem_run_stack.__tail(STACK_OFFSET);\
   unsigned fa_idx = sem_run_stack.__tail(STACK_OFFSET + 1);\
   string_arrays_s &string_array_array = string_array_array_variables[sem_run_stack.__tail(STACK_OFFSET + 2)];\
   assert(fa_idx < string_array_array.used);\
   string_array_s &string_array = string_array_array[fa_idx];\
   assert(sa_idx < string_array.used);\
\
   TRG_PTR = &string_array[sa_idx];\
}

// - ziskani ukazatele na string_s aaref podle hodnot na zasobniku -
#define POP_STRING_AAREF(TRG_PTR) \
{\
   GET_STRING_AAREF(TRG_PTR,0);\
   sem_run_stack.used -= 3;\
}

// - out instrukce -
void parser_s::pi_sott_out_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables[sem_run_stack.pop()].print();
}

void parser_s::pi_sott_out_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_array_s &unsigned_array = int_array_variables[sem_run_stack.pop()];

   if (unsigned_array.used != 0) {
      unsigned e_idx = 0;
      do {
	 printf("%d ",unsigned_array.data[e_idx]);
      } while(++e_idx < unsigned_array.used);
   }
}

void parser_s::pi_sott_out_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_s &string_array = string_array_variables[sem_run_stack.pop()];

   if (string_array.used != 0) {
      unsigned e_idx = 0;
      do {
	 string_array[e_idx].print();
	 putchar(' ');
      } while(++e_idx < string_array.used);
   }
}

void parser_s::pi_sott_out_int_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_arrays_s &unsigned_array_array = int_array_array_variables[sem_run_stack.__pop()];

   if (unsigned_array_array.used != 0) {
      unsigned a_idx = 0;
      do {
	 ui_array_s &unsigned_array = unsigned_array_array[a_idx];

	 if (unsigned_array.used != 0) {
	    unsigned e_idx = 0;
	    do {
	       printf("%d ",unsigned_array.data[e_idx]);
	    } while(++e_idx < unsigned_array.used);
	 }

	 putchar('\n');
      } while(++a_idx < unsigned_array_array.used);
   }
}

void parser_s::pi_sott_out_string_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_arrays_s &string_array_array = string_array_array_variables[sem_run_stack.__pop()];

   if (string_array_array.used != 0) {
      unsigned a_idx = 0;
      do {
	 string_array_s &string_array = string_array_array[a_idx];

	 if (string_array.used != 0) {
	    unsigned e_idx = 0;
	    do {
	       string_array[e_idx].print();
	       putchar(' ');
	    } while(++e_idx < string_array.used);
	 }
	 
	 putchar('\n');
      } while(++a_idx < string_array_array.used);
   }
}

void parser_s::pi_sott_out_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   printf("%d",sem_run_stack.pop());
}

void parser_s::pi_sott_out_string_tmp(p_parse_data_s &parse_data)
{
   string_array_s &string_variables = parse_data.string_variables;
   string_variables.pop().print();
}

void parser_s::pi_sott_out_int_array_tmp(p_parse_data_s &parse_data)
{
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_array_s &unsigned_array = int_array_variables.pop();

   if (unsigned_array.used != 0) {
      unsigned e_idx = 0;
      do {
	 printf("%d ",unsigned_array.data[e_idx]);
      } while(++e_idx < unsigned_array.used);
   }
}

void parser_s::pi_sott_out_string_array_tmp(p_parse_data_s &parse_data)
{
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_s &string_array = string_array_variables.pop();

   if (string_array.used != 0) {
      unsigned e_idx = 0;
      do {
	 string_array[e_idx].print();
	 putchar(' ');
      } while(++e_idx < string_array.used);
   }
}

void parser_s::pi_sott_out_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_constants[sem_run_stack.pop()].print();
}

void parser_s::pi_sott_out_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   string_aref_ptr->print();
}

void parser_s::pi_sott_out_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   POP_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr);

   ui_array_s &unsigned_array = *unsigned_array_aref_ptr;

   if (unsigned_array.used != 0) {
      unsigned e_idx = 0;
      do {
	 printf("%d ",unsigned_array.data[e_idx]);
      } while(++e_idx < unsigned_array.used);
   }
}

void parser_s::pi_sott_out_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   POP_STRING_ARRAY_AREF(string_array_aref_ptr);

   string_array_s &string_array = *string_array_aref_ptr;

   if (string_array.used != 0) {
      unsigned e_idx = 0;
      do {
	 string_array[e_idx].print();
	 putchar(' ');
      } while(++e_idx < string_array.used);
   }
}

void parser_s::pi_sott_out_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);
   
   string_aaref_ptr->print();
}

// - instrukce vetveni if -
void parser_s::pi_sott_jump(p_parse_data_s &parse_data)
{
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sc_idx = sem_code.get_unsigned(sc_idx);
}

void parser_s::pi_sott_jump_if_zero(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   unsigned new_sc_idx = sem_code.get_unsigned(sc_idx);

   if (sem_run_stack.__pop() == 0) {
      sc_idx = new_sc_idx;
   }
}

void parser_s::pi_sott_jump_if_no_zero(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   unsigned new_sc_idx = sem_code.get_unsigned(sc_idx);

   if (sem_run_stack.__pop() != 0) {
      sc_idx = new_sc_idx;
   }
}

// - assert instrukce -
void parser_s::pi_sott_assert_int_value_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_s &string = string_constants[sem_run_stack.__last()];

   if (sem_run_stack.__tail(1) == 0) {
      printf("ERROR: PARSER: ASSERT: \"%s\" on_line %d\n",string.data,parse_data.line_cnt);
   }

   sem_run_stack.used -= 2;
}

// - exp instrukce -
void parser_s::pi_sott_exp_end(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.used--;
}

void parser_s::pi_sott_exp_end_string_tmp(p_parse_data_s &parse_data)
{
   string_array_s &string_variables = parse_data.string_variables;
   string_variables.used--;
}

void parser_s::pi_sott_exp_end_int_array_tmp(p_parse_data_s &parse_data)
{
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   int_array_variables.used--;
}

void parser_s::pi_sott_exp_end_string_array_tmp(p_parse_data_s &parse_data)
{
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_variables.used--;
}

void parser_s::pi_sott_exp_end_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_exp_end_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.used -= 3;
}

// - set instrukce -
void parser_s::pi_sott_set_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   int_variables[sem_run_stack.__tail(1)] = sem_run_stack.__last();
   sem_run_stack.used--;
}

void parser_s::pi_sott_set_string_var_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables[sem_run_stack.__tail(1)] = string_variables[sem_run_stack.__last()];
   sem_run_stack.used--;
}

void parser_s::pi_sott_set_string_var_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables[sem_run_stack.__last()].swap(string_variables.__pop());
}

void parser_s::pi_sott_set_string_var_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_s &str = string_constants[sem_run_stack.__pop()];
   string_variables[sem_run_stack.__last()] = str;
}

void parser_s::pi_sott_set_string_var_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   string_variables[sem_run_stack.__last()] = *string_aref_ptr;
}

void parser_s::pi_sott_set_string_var_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   string_variables[sem_run_stack.__last()] = *string_aaref_ptr;
}

void parser_s::pi_sott_set_int_array_var_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   int_array_variables[sem_run_stack.__tail(1)] = int_array_variables[sem_run_stack.__last()];
   sem_run_stack.used--;
}

void parser_s::pi_sott_set_int_array_var_int_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   int_array_variables[sem_run_stack.__last()].swap(int_array_variables.__pop());
}

void parser_s::pi_sott_set_int_array_var_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   int_array_variables[sem_run_stack.__tail(1)] = int_array_constants[sem_run_stack.__last()];
   sem_run_stack.used--;
}

void parser_s::pi_sott_set_int_array_var_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   POP_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr);

   int_array_variables[sem_run_stack.__last()] = *unsigned_array_aref_ptr;
}

void parser_s::pi_sott_set_int_array_var_blank_array(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   int_array_variables[sem_run_stack.__last()].used = 0;
}

void parser_s::pi_sott_set_string_array_var_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_variables[sem_run_stack.__tail(1)] = string_array_variables[sem_run_stack.__last()];
   sem_run_stack.used--;
}

void parser_s::pi_sott_set_string_array_var_string_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_variables[sem_run_stack.__last()].swap(string_array_variables.pop());
}

void parser_s::pi_sott_set_string_array_var_string_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_variables[sem_run_stack.__tail(1)] = string_array_constants[sem_run_stack.__last()];
   sem_run_stack.used--;
}

void parser_s::pi_sott_set_string_array_var_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   POP_STRING_ARRAY_AREF(string_array_aref_ptr);

   string_array_variables[sem_run_stack.__last()] = *string_array_aref_ptr;
}

void parser_s::pi_sott_set_string_array_var_blank_array(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_variables[sem_run_stack.__last()].used = 0;
}

void parser_s::pi_sott_set_int_array_array_var_int_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   int_array_array_variables[sem_run_stack.__tail(1)] = int_array_array_variables[sem_run_stack.__last()];
   sem_run_stack.used--;
}

void parser_s::pi_sott_set_int_array_array_var_int_array_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_array_constants;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   int_array_array_variables[sem_run_stack.__tail(1)] = int_array_array_constants[sem_run_stack.__last()];
   sem_run_stack.used--;
}

void parser_s::pi_sott_set_int_array_array_var_blank_array(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   int_array_array_variables[sem_run_stack.__last()].used = 0;
}

void parser_s::pi_sott_set_string_array_array_var_string_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_array_variables[sem_run_stack.__tail(1)] = string_array_array_variables[sem_run_stack.__last()];
   sem_run_stack.used--;
}

void parser_s::pi_sott_set_string_array_array_var_string_array_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_array_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_array_variables[sem_run_stack.__tail(1)] = string_array_array_constants[sem_run_stack.__last()];
   sem_run_stack.used--;
}

void parser_s::pi_sott_set_string_array_array_var_blank_array(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_array_variables[sem_run_stack.__last()].used = 0;
}

void parser_s::pi_sott_set_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   *unsigned_aref_ptr = sem_run_stack.__pop();
}

void parser_s::pi_sott_set_string_aref_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,1);

   *string_aref_ptr = string_variables[sem_run_stack.__pop()];
}

void parser_s::pi_sott_set_string_aref_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);

   string_aref_ptr->swap(string_variables.pop());
}

void parser_s::pi_sott_set_string_aref_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,1);

   *string_aref_ptr = string_constants[sem_run_stack.__pop()];
}

void parser_s::pi_sott_set_string_aref_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_src_ptr;
   POP_STRING_AREF(string_aref_src_ptr);

   string_s *string_aref_trg_ptr;
   GET_STRING_AREF(string_aref_trg_ptr,0);

   *string_aref_trg_ptr = *string_aref_src_ptr;
}

void parser_s::pi_sott_set_string_aref_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);

   *string_aref_ptr = *string_aaref_ptr;
}

void parser_s::pi_sott_set_int_array_aref_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,1);

   *unsigned_array_aref_ptr = int_array_variables[sem_run_stack.__pop()];
}

void parser_s::pi_sott_set_int_array_aref_int_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,0);

   unsigned_array_aref_ptr->swap(int_array_variables.pop());
}

void parser_s::pi_sott_set_int_array_aref_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,1);

   *unsigned_array_aref_ptr = int_array_constants[sem_run_stack.__pop()];
}

void parser_s::pi_sott_set_int_array_aref_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_src_ptr;
   POP_UNSIGNED_ARRAY_AREF(unsigned_array_aref_src_ptr);

   ui_array_s *unsigned_array_aref_trg_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_trg_ptr,0);

   *unsigned_array_aref_trg_ptr = *unsigned_array_aref_src_ptr;
}

void parser_s::pi_sott_set_int_array_aref_blank_array(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,0);

   unsigned_array_aref_ptr->used = 0;
}

void parser_s::pi_sott_set_string_array_aref_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);

   *string_array_aref_ptr = string_array_variables[sem_run_stack.__pop()];
}

void parser_s::pi_sott_set_string_array_aref_string_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);

   string_array_aref_ptr->swap(string_array_variables.pop());
}

void parser_s::pi_sott_set_string_array_aref_string_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);

   *string_array_aref_ptr = string_array_constants[sem_run_stack.__pop()];
}

void parser_s::pi_sott_set_string_array_aref_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_src_ptr;
   POP_STRING_ARRAY_AREF(string_array_aref_src_ptr);

   string_array_s *string_array_aref_trg_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_trg_ptr,0);

   *string_array_aref_trg_ptr = *string_array_aref_src_ptr;
}

void parser_s::pi_sott_set_string_array_aref_blank_array(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);

   string_array_aref_ptr->used = 0;
}

void parser_s::pi_sott_set_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   *unsigned_aaref_ptr = sem_run_stack.__pop();
}

void parser_s::pi_sott_set_string_aaref_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,1);

   *string_aaref_ptr = string_variables[sem_run_stack.__pop()];
}

void parser_s::pi_sott_set_string_aaref_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);

   *string_aaref_ptr = string_variables.pop();
}

void parser_s::pi_sott_set_string_aaref_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,1);

   *string_aaref_ptr = string_constants[sem_run_stack.__pop()];
}

void parser_s::pi_sott_set_string_aaref_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);

   *string_aaref_ptr = *string_aref_ptr;
}

void parser_s::pi_sott_set_string_aaref_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_src_ptr;
   POP_STRING_AAREF(string_aaref_src_ptr);

   string_s *string_aaref_trg_ptr;
   GET_STRING_AAREF(string_aaref_trg_ptr,0);

   *string_aaref_trg_ptr = *string_aaref_src_ptr;
}

// - instrukce prohozeni obsahu dvou stejnych poli, nebo retezcu -
void parser_s::pi_sott_swap_string_var_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables[sem_run_stack.__tail(1)].swap(string_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_swap_string_var_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   string_variables[sem_run_stack.__last()].swap(*string_aref_ptr);
}

void parser_s::pi_sott_swap_string_var_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   string_variables[sem_run_stack.__last()].swap(*string_aaref_ptr);
}

void parser_s::pi_sott_swap_string_aref_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,1);

   string_variables[sem_run_stack.__pop()].swap(*string_aref_ptr);
}

void parser_s::pi_sott_swap_string_aref_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_second_ptr;
   POP_STRING_AREF(string_aref_second_ptr);

   string_s *string_aref_first_ptr;
   GET_STRING_AREF(string_aref_first_ptr,0);

   string_aref_first_ptr->swap(*string_aref_second_ptr);
}

void parser_s::pi_sott_swap_string_aref_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);

   string_aref_ptr->swap(*string_aaref_ptr);
}

void parser_s::pi_sott_swap_string_aaref_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,1);

   string_variables[sem_run_stack.__pop()].swap(*string_aaref_ptr);
}

void parser_s::pi_sott_swap_string_aaref_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);

   string_aaref_ptr->swap(*string_aref_ptr);
}

void parser_s::pi_sott_swap_string_aaref_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_second_ptr;
   POP_STRING_AAREF(string_aaref_second_ptr);

   string_s *string_aaref_first_ptr;
   GET_STRING_AAREF(string_aaref_first_ptr,0);

   string_aaref_first_ptr->swap(*string_aaref_second_ptr);
}

void parser_s::pi_sott_swap_int_array_var_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   int_array_variables[sem_run_stack.__tail(1)].swap(int_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_swap_int_array_var_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   POP_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr);

   int_array_variables[sem_run_stack.__last()].swap(*unsigned_array_aref_ptr);
}

void parser_s::pi_sott_swap_int_array_aref_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,1);

   int_array_variables[sem_run_stack.__pop()].swap(*unsigned_array_aref_ptr);
}

void parser_s::pi_sott_swap_int_array_aref_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_second_ptr;
   POP_UNSIGNED_ARRAY_AREF(unsigned_array_aref_second_ptr);

   ui_array_s *unsigned_array_aref_first_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_first_ptr,0);

   unsigned_array_aref_first_ptr->swap(*unsigned_array_aref_second_ptr);
}

void parser_s::pi_sott_swap_string_array_var_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_variables[sem_run_stack.__tail(1)].swap(string_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_swap_string_array_var_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   POP_STRING_ARRAY_AREF(string_array_aref_ptr);

   string_array_variables[sem_run_stack.__last()].swap(*string_array_aref_ptr);
}

void parser_s::pi_sott_swap_string_array_aref_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);

   string_array_variables[sem_run_stack.__pop()].swap(*string_array_aref_ptr);
}

void parser_s::pi_sott_swap_string_array_aref_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_second_ptr;
   POP_STRING_ARRAY_AREF(string_array_aref_second_ptr);

   string_array_s *string_array_aref_first_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_first_ptr,0);

   string_array_aref_first_ptr->swap(*string_array_aref_second_ptr);
}

void parser_s::pi_sott_swap_int_array_array_var_int_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   int_array_array_variables[sem_run_stack.__tail(1)].swap(int_array_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_swap_string_array_array_var_string_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_array_variables[sem_run_stack.__tail(1)].swap(string_array_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}


// - instrukce logicke operace and -
void parser_s::pi_sott_logic_and_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = (int_variables[sem_run_stack.__tail(1)] && sem_run_stack.__last());
   sem_run_stack.used --;
}

void parser_s::pi_sott_logic_and_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__tail(1) = (sem_run_stack.__tail(1) && sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_logic_and_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = (*unsigned_aref_ptr && sem_run_stack.__last());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_logic_and_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(3) = (*unsigned_aaref_ptr && sem_run_stack.__last());
   sem_run_stack.used -= 3;
}

// - instrukce logicke operace or -
void parser_s::pi_sott_logic_or_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = (int_variables[sem_run_stack.__tail(1)] || sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_logic_or_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__tail(1) = (sem_run_stack.__tail(1) || sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_logic_or_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = (*unsigned_aref_ptr || sem_run_stack.__last());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_logic_or_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(3) = (*unsigned_aaref_ptr || sem_run_stack.__last());
   sem_run_stack.used -= 3;
}

// - instrukce porovnani -
void parser_s::pi_sott_compare_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = (int_variables[sem_run_stack.__tail(1)] == sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_var_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.__tail(1) = (string_variables[sem_run_stack.__tail(1)] == string_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_var_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.__last() = (string_variables[sem_run_stack.__last()] == string_variables.pop());
}

void parser_s::pi_sott_compare_string_var_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.__tail(1) = (string_variables[sem_run_stack.__tail(1)] == string_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_var_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   sem_run_stack.__last() = (string_variables[sem_run_stack.__last()] == *string_aref_ptr);
}

void parser_s::pi_sott_compare_string_var_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   sem_run_stack.__last() = (string_variables[sem_run_stack.__last()] == *string_aaref_ptr);
}

void parser_s::pi_sott_compare_int_array_var_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   sem_run_stack.__tail(1) = (int_array_variables[sem_run_stack.__tail(1)] == int_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_int_array_var_int_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   sem_run_stack.__last() = (int_array_variables[sem_run_stack.__last()] == int_array_variables.pop());
}

void parser_s::pi_sott_compare_int_array_var_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   sem_run_stack.__tail(1) = (int_array_variables[sem_run_stack.__tail(1)] == int_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_int_array_var_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   POP_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr);

   sem_run_stack.__last() = (int_array_variables[sem_run_stack.__last()] == *unsigned_array_aref_ptr);
}

void parser_s::pi_sott_compare_string_array_var_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__tail(1) = (string_array_variables[sem_run_stack.__tail(1)] == string_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_array_var_string_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__last() = (string_array_variables[sem_run_stack.__last()] == string_array_variables.pop());
}

void parser_s::pi_sott_compare_string_array_var_string_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__tail(1) = (string_array_variables[sem_run_stack.__tail(1)] == string_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_array_var_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   POP_STRING_ARRAY_AREF(string_array_aref_ptr);

   sem_run_stack.__last() = (string_array_variables[sem_run_stack.__last()] == *string_array_aref_ptr);
}

void parser_s::pi_sott_compare_int_array_array_var_int_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   sem_run_stack.__tail(1) = (int_array_array_variables[sem_run_stack.__tail(1)] == int_array_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_int_array_array_var_int_array_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_array_constants;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   sem_run_stack.__tail(1) = (int_array_array_variables[sem_run_stack.__tail(1)] == int_array_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_array_array_var_string_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   sem_run_stack.__tail(1) = (string_array_array_variables[sem_run_stack.__tail(1)] == string_array_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_array_array_var_string_array_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_array_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   sem_run_stack.__tail(1) = (string_array_array_variables[sem_run_stack.__tail(1)] == string_array_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__tail(1) = (sem_run_stack.__tail(1) == sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_tmp_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.__last() = (string_variables.pop() == string_variables[sem_run_stack.__last()]);
}

void parser_s::pi_sott_compare_string_tmp_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.push(string_variables.pop() == string_variables.pop());
}

void parser_s::pi_sott_compare_string_tmp_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.__last() = (string_variables.pop() == string_constants[sem_run_stack.__last()]);
}

void parser_s::pi_sott_compare_string_tmp_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);
   
   sem_run_stack.__tail(1) = (string_variables.pop() == *string_aref_ptr);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_tmp_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);
   
   sem_run_stack.__tail(2) = (string_variables.pop() == *string_aaref_ptr);
   sem_run_stack.used -= 2;

}

void parser_s::pi_sott_compare_int_array_tmp_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   sem_run_stack.__last() = (int_array_variables.pop() == int_array_variables[sem_run_stack.__last()]);
}

void parser_s::pi_sott_compare_int_array_tmp_int_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   sem_run_stack.push(int_array_variables.pop() == int_array_variables.pop());
}

void parser_s::pi_sott_compare_int_array_tmp_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   sem_run_stack.__last() = (int_array_variables.pop() == int_array_constants[sem_run_stack.__last()]);
}

void parser_s::pi_sott_compare_int_array_tmp_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,0);
   
   sem_run_stack.__tail(1) = (int_array_variables.pop() == *unsigned_array_aref_ptr);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_array_tmp_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__last() = (string_array_variables.pop() == string_array_variables[sem_run_stack.__last()]);
}

void parser_s::pi_sott_compare_string_array_tmp_string_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.push(string_array_variables.pop() == string_array_variables.pop());
}

void parser_s::pi_sott_compare_string_array_tmp_string_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__last() = (string_array_variables.pop() == string_array_constants[sem_run_stack.__last()]);
}

void parser_s::pi_sott_compare_string_array_tmp_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);

   sem_run_stack.push(string_array_variables.pop() == *string_array_aref_ptr);
}

void parser_s::pi_sott_compare_string_const_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.__tail(1) = (string_constants[sem_run_stack.__tail(1)] == string_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_const_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.__last() = (string_constants[sem_run_stack.__last()] == string_variables.pop());
}

void parser_s::pi_sott_compare_string_const_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   sem_run_stack.__tail(1) = (string_constants[sem_run_stack.__tail(1)] == string_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_const_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   sem_run_stack.__last() = (string_constants[sem_run_stack.__last()] == *string_aref_ptr);
}

void parser_s::pi_sott_compare_string_const_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   sem_run_stack.__last() = (string_constants[sem_run_stack.__last()] == *string_aaref_ptr);
}

void parser_s::pi_sott_compare_int_array_const_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   sem_run_stack.__tail(1) = (int_array_constants[sem_run_stack.__tail(1)] == int_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_int_array_const_int_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   sem_run_stack.__last() = (int_array_constants[sem_run_stack.__last()] == int_array_variables.pop());
}

void parser_s::pi_sott_compare_int_array_const_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   sem_run_stack.__tail(1) = (int_array_constants[sem_run_stack.__tail(1)] == int_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_int_array_const_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   POP_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr);

   sem_run_stack.__last() = (int_array_constants[sem_run_stack.__last()] == *unsigned_array_aref_ptr);
}

void parser_s::pi_sott_compare_string_array_const_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__tail(1) = (string_array_constants[sem_run_stack.__tail(1)] == string_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_array_const_string_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__last() = (string_array_constants[sem_run_stack.__last()] == string_array_variables.pop());
}

void parser_s::pi_sott_compare_string_array_const_string_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   sem_run_stack.__tail(1) = (string_array_constants[sem_run_stack.__tail(1)] == string_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_array_const_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   POP_STRING_ARRAY_AREF(string_array_aref_ptr);

   sem_run_stack.__last() = (string_array_constants[sem_run_stack.__last()] == *string_array_aref_ptr);
}

void parser_s::pi_sott_compare_int_array_array_const_int_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_array_constants;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   sem_run_stack.__tail(1) = (int_array_array_constants[sem_run_stack.__tail(1)] == int_array_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_int_array_array_const_int_array_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_array_constants;
   sem_run_stack.__tail(1) = (int_array_array_constants[sem_run_stack.__tail(1)] == int_array_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_array_array_const_string_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_array_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   sem_run_stack.__tail(1) = (string_array_array_constants[sem_run_stack.__tail(1)] == string_array_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_array_array_const_string_array_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_array_constants;
   sem_run_stack.__tail(1) = (string_array_array_constants[sem_run_stack.__tail(1)] == string_array_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = (*unsigned_aref_ptr == sem_run_stack.__last());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_compare_string_aref_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,1);

   sem_run_stack.__tail(2) = (*string_aref_ptr == string_variables[sem_run_stack.__last()]);
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_compare_string_aref_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);

   sem_run_stack.__tail(1) = (*string_aref_ptr == string_variables.pop());
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_aref_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,1);

   sem_run_stack.__tail(2) = (*string_aref_ptr == string_constants[sem_run_stack.__last()]);
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_compare_string_aref_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_second_ptr;
   GET_STRING_AREF(string_aref_second_ptr,0);

   string_s *string_aref_first_ptr;
   GET_STRING_AREF(string_aref_first_ptr,2);

   sem_run_stack.__tail(3) = (*string_aref_first_ptr == *string_aref_second_ptr);
   sem_run_stack.used -= 3;
}

void parser_s::pi_sott_compare_string_aref_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);

   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,3);

   sem_run_stack.__tail(4) = (*string_aref_ptr == *string_aaref_ptr);
   sem_run_stack.used -= 4;
}

void parser_s::pi_sott_compare_int_array_aref_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,1);

   sem_run_stack.__tail(2) = (*unsigned_array_aref_ptr == int_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_compare_int_array_aref_int_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,0);

   sem_run_stack.__tail(1) = (*unsigned_array_aref_ptr == int_array_variables.pop());
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_int_array_aref_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,1);

   sem_run_stack.__tail(2) = (*unsigned_array_aref_ptr == int_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_compare_int_array_aref_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_second_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_second_ptr,0);

   ui_array_s *unsigned_array_aref_first_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_first_ptr,2);

   sem_run_stack.__tail(3) = (*unsigned_array_aref_first_ptr == *unsigned_array_aref_second_ptr);
   sem_run_stack.used -= 3;
}

void parser_s::pi_sott_compare_string_array_aref_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);

   sem_run_stack.__tail(2) = (*string_array_aref_ptr == string_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_compare_string_array_aref_string_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);

   sem_run_stack.__tail(1) = (*string_array_aref_ptr == string_array_variables.pop());
   sem_run_stack.used--;
}

void parser_s::pi_sott_compare_string_array_aref_string_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);

   sem_run_stack.__tail(2) = (*string_array_aref_ptr == string_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_compare_string_array_aref_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_second_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_second_ptr,0);

   string_array_s *string_array_aref_first_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_first_ptr,2);

   sem_run_stack.__tail(3) = (*string_array_aref_first_ptr == *string_array_aref_second_ptr);
   sem_run_stack.used -= 3;
}

void parser_s::pi_sott_compare_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(2) = (*unsigned_aaref_ptr == sem_run_stack.__last());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_compare_string_aaref_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,1);

   sem_run_stack.__tail(3) = (*string_aaref_ptr == string_variables[sem_run_stack.__last()]);
   sem_run_stack.used -= 3;
}

void parser_s::pi_sott_compare_string_aaref_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);

   sem_run_stack.__tail(2) = (*string_aaref_ptr == string_variables.pop());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_compare_string_aaref_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,1);

   sem_run_stack.__tail(3) = (*string_aaref_ptr == string_constants[sem_run_stack.__last()]);
   sem_run_stack.used -= 3;
}

void parser_s::pi_sott_compare_string_aaref_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);

   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,2);

   sem_run_stack.__tail(4) = (*string_aaref_ptr == *string_aref_ptr);
   sem_run_stack.used -= 4;
}

void parser_s::pi_sott_compare_string_aaref_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_second_ptr;
   GET_STRING_AAREF(string_aaref_second_ptr,0);

   string_s *string_aaref_first_ptr;
   GET_STRING_AAREF(string_aaref_first_ptr,3);

   sem_run_stack.__tail(5) = (*string_aaref_first_ptr == *string_aaref_second_ptr);
   sem_run_stack.used -= 5;
}

// - less instrukce -
void parser_s::pi_sott_less_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = (int_variables[sem_run_stack.__tail(1)] < sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_less_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__tail(1) = (sem_run_stack.__tail(1) < sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_less_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = (*unsigned_aref_ptr < sem_run_stack.__last());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_less_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(3) = (*unsigned_aaref_ptr < sem_run_stack.__last());
   sem_run_stack.used -= 3;
}

// - greater instrukce -
void parser_s::pi_sott_greater_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = (int_variables[sem_run_stack.__tail(1)] > sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_greater_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__tail(1) = (sem_run_stack.__tail(1) > sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_greater_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = (*unsigned_aref_ptr > sem_run_stack.__last());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_greater_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(3) = (*unsigned_aaref_ptr > sem_run_stack.__last());
   sem_run_stack.used -= 3;
}

// - less_equal instrukce -
void parser_s::pi_sott_less_equal_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = (int_variables[sem_run_stack.__tail(1)] <= sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_less_equal_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__tail(1) = (sem_run_stack.__tail(1) <= sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_less_equal_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = (*unsigned_aref_ptr <= sem_run_stack.__last());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_less_equal_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(3) = (*unsigned_aaref_ptr <= sem_run_stack.__last());
   sem_run_stack.used -= 3;
}

// - greater_equal instrukce -
void parser_s::pi_sott_greater_equal_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = (int_variables[sem_run_stack.__tail(1)] >= sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_greater_equal_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__tail(1) = (sem_run_stack.__tail(1) >= sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_greater_equal_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = (*unsigned_aref_ptr >= sem_run_stack.__last());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_greater_equal_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(3) = (*unsigned_aaref_ptr >= sem_run_stack.__last());
   sem_run_stack.used -= 3;
}

// - add instrukce -
void parser_s::pi_sott_add_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = int_variables[sem_run_stack.__tail(1)] + sem_run_stack.__last();
   sem_run_stack.used--;
}

void parser_s::pi_sott_add_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__tail(1) += sem_run_stack.__last();
   sem_run_stack.used--;
}

void parser_s::pi_sott_add_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = *unsigned_aref_ptr + sem_run_stack.__last();
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_add_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(3) = *unsigned_aaref_ptr + sem_run_stack.__last();
   sem_run_stack.used -= 3;
}

// - sub instrukce -
void parser_s::pi_sott_sub_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = int_variables[sem_run_stack.__tail(1)] - sem_run_stack.__last();
   sem_run_stack.used--;
}

void parser_s::pi_sott_sub_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__tail(1) -= sem_run_stack.__last();
   sem_run_stack.used--;
}

void parser_s::pi_sott_sub_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = *unsigned_aref_ptr - sem_run_stack.__last();
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_sub_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(3) = *unsigned_aaref_ptr - sem_run_stack.__last();
   sem_run_stack.used -= 3;
}

// - conc instrukce -
void parser_s::pi_sott_conc_string_var_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables.push_blank();

   string_s &second = string_variables[sem_run_stack.__last()];
   string_s &first = string_variables[sem_run_stack.__tail(1)];
   sem_run_stack.used -= 2;

   string_variables.last().conc_set(first.size - 1,first.data,second.size - 1,second.data);
}

void parser_s::pi_sott_conc_string_var_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_s tmp_str;
   tmp_str.init();
  
   string_s &second = string_variables.last();
   string_s &first = string_variables[sem_run_stack.__pop()];

   tmp_str.conc_set(first.size - 1,first.data,second.size - 1,second.data);
   string_variables.last().swap(tmp_str);

   tmp_str.clear();
}

void parser_s::pi_sott_conc_string_var_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables.push_blank();

   string_s &second = string_constants[sem_run_stack.__last()];
   string_s &first = string_variables[sem_run_stack.__tail(1)];
   sem_run_stack.used -= 2;

   string_variables.last().conc_set(first.size - 1,first.data,second.size - 1,second.data);
}

void parser_s::pi_sott_conc_string_var_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_variables.push_blank();

   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);

   string_s &first = string_variables[sem_run_stack.__tail(2)];
   sem_run_stack.used -= 3;

   string_variables.last().conc_set(first.size - 1,first.data,string_aref_ptr->size - 1,string_aref_ptr->data);
}

void parser_s::pi_sott_conc_string_var_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_variables.push_blank();

   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);
   
   string_s &first = string_variables[sem_run_stack.__tail(3)];
   sem_run_stack.used -= 3;
   
   string_variables.last().conc_set(first.size - 1,first.data,string_aaref_ptr->size - 1,string_aaref_ptr->data);
}

void parser_s::pi_sott_conc_string_tmp_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_s tmp_str;
   tmp_str.init();

   string_s &second = string_variables[sem_run_stack.__pop()];
   string_s &first = string_variables.last();

   tmp_str.conc_set(first.size - 1,first.data,second.size - 1,second.data);
   string_variables.last().swap(tmp_str);

   tmp_str.clear();
}

void parser_s::pi_sott_conc_string_tmp_string_tmp(p_parse_data_s &parse_data)
{
   string_array_s &string_variables = parse_data.string_variables;
   string_s tmp_str;
   tmp_str.init();

   string_s &second = string_variables.pop();
   string_s &first = string_variables.last();

   tmp_str.conc_set(first.size - 1,first.data,second.size - 1,second.data);
   string_variables.last().swap(tmp_str);

   tmp_str.clear();
}

void parser_s::pi_sott_conc_string_tmp_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_s tmp_str;
   tmp_str.init();

   string_s &second = string_constants[sem_run_stack.__pop()];
   string_s &first = string_variables.last();

   tmp_str.conc_set(first.size - 1,first.data,second.size - 1,second.data);
   string_variables.last().swap(tmp_str);

   tmp_str.clear();
}

void parser_s::pi_sott_conc_string_tmp_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s tmp_str;
   tmp_str.init();
   
   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   string_s &first = string_variables.last();

   tmp_str.conc_set(first.size - 1,first.data,string_aref_ptr->size - 1,string_aref_ptr->data);
   string_variables.last().swap(tmp_str);

   tmp_str.clear();
}

void parser_s::pi_sott_conc_string_tmp_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s tmp_str;
   tmp_str.init();

   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   string_s &first = string_variables.last();

   tmp_str.conc_set(first.size - 1,first.data,string_aaref_ptr->size - 1,string_aaref_ptr->data);
   string_variables.last().swap(tmp_str);

   tmp_str.clear();
}

void parser_s::pi_sott_conc_string_const_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables.push_blank();

   string_s &second = string_variables[sem_run_stack.__last()];
   string_s &first = string_constants[sem_run_stack.__tail(1)];
   sem_run_stack.used -= 2;

   string_variables.last().conc_set(first.size - 1,first.data,second.size - 1,second.data);
}

void parser_s::pi_sott_conc_string_const_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_s tmp_str;
   tmp_str.init();
  
   string_s &second = string_variables.last();
   string_s &first = string_constants[sem_run_stack.__pop()];

   tmp_str.conc_set(first.size - 1,first.data,second.size - 1,second.data);
   string_variables.last().swap(tmp_str);

   tmp_str.clear();
}

void parser_s::pi_sott_conc_string_const_string_const(p_parse_data_s &parse_data)
{
   assert(0);
}

void parser_s::pi_sott_conc_string_const_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_variables.push_blank();

   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);

   string_s &first = string_constants[sem_run_stack.__tail(2)];
   sem_run_stack.used -= 3;

   string_variables.last().conc_set(first.size - 1,first.data,string_aref_ptr->size - 1,string_aref_ptr->data);
}

void parser_s::pi_sott_conc_string_const_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_variables.push_blank();

   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);
   
   string_s &first = string_constants[sem_run_stack.__tail(3)];
   sem_run_stack.used -= 3;
   
   string_variables.last().conc_set(first.size - 1,first.data,string_aaref_ptr->size - 1,string_aaref_ptr->data);
}

void parser_s::pi_sott_conc_string_aref_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_variables.push_blank();

   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,1);

   string_s &second = string_variables[sem_run_stack.__last()];
   sem_run_stack.used -= 3;

   string_variables.last().conc_set(string_aref_ptr->size - 1,string_aref_ptr->data,second.size - 1,second.data);
}

void parser_s::pi_sott_conc_string_aref_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s tmp_str;
   tmp_str.init();

   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   string_s &second = string_variables.last();

   tmp_str.conc_set(string_aref_ptr->size - 1,string_aref_ptr->data,second.size - 1,second.data);
   string_variables.last().swap(tmp_str);

   tmp_str.clear();
}

void parser_s::pi_sott_conc_string_aref_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_variables.push_blank();

   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,1);

   string_s &second = string_constants[sem_run_stack.__last()];
   sem_run_stack.used -= 3;

   string_variables.last().conc_set(string_aref_ptr->size - 1,string_aref_ptr->data,second.size - 1,second.data);
}

void parser_s::pi_sott_conc_string_aref_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_variables.push_blank();
   
   string_s *string_aref_second_ptr;
   GET_STRING_AREF(string_aref_second_ptr,0);

   string_s *string_aref_first_ptr;
   GET_STRING_AREF(string_aref_first_ptr,2);
   sem_run_stack.used -= 4;

   string_variables.last().conc_set(string_aref_first_ptr->size - 1,string_aref_first_ptr->data,string_aref_second_ptr->size - 1,string_aref_second_ptr->data);
}

void parser_s::pi_sott_conc_string_aref_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_variables.push_blank();

   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);

   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,3);
   sem_run_stack.used -= 5;

   string_variables.last().conc_set(string_aref_ptr->size - 1,string_aref_ptr->data,string_aaref_ptr->size - 1,string_aaref_ptr->data);
}

void parser_s::pi_sott_conc_string_aaref_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_variables.push_blank();

   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,1);

   string_s &second = string_variables[sem_run_stack.__last()];
   sem_run_stack.used -= 4;

   string_variables.last().conc_set(string_aaref_ptr->size - 1,string_aaref_ptr->data,second.size - 1,second.data);
}

void parser_s::pi_sott_conc_string_aaref_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s tmp_str;
   tmp_str.init();

   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   string_s &second = string_variables.last();

   tmp_str.conc_set(string_aaref_ptr->size - 1,string_aaref_ptr->data,second.size - 1,second.data);
   string_variables.last().swap(tmp_str);

   tmp_str.clear();
}

void parser_s::pi_sott_conc_string_aaref_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_variables.push_blank();

   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,1);

   string_s &second = string_constants[sem_run_stack.__last()];
   sem_run_stack.used -= 4;

   string_variables.last().conc_set(string_aaref_ptr->size - 1,string_aaref_ptr->data,second.size - 1,second.data);
}

void parser_s::pi_sott_conc_string_aaref_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_variables.push_blank();

   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);

   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,2);
   sem_run_stack.used -= 5;

   string_variables.last().conc_set(string_aaref_ptr->size - 1,string_aaref_ptr->data,string_aref_ptr->size - 1,string_aref_ptr->data);
}

void parser_s::pi_sott_conc_string_aaref_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_variables.push_blank();

   string_s *string_aaref_second_ptr;
   GET_STRING_AAREF(string_aaref_second_ptr,0);

   string_s *string_aaref_first_ptr;
   GET_STRING_AAREF(string_aaref_first_ptr,3);
   sem_run_stack.used -= 6;

   string_variables.last().conc_set(string_aaref_first_ptr->size - 1,string_aaref_first_ptr->data,string_aaref_second_ptr->size - 1,string_aaref_second_ptr->data);
}

// - post_inc instrukce -
void parser_s::pi_sott_post_inc_int_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__last() = int_variables[sem_run_stack.__last()]++;
}

void parser_s::pi_sott_post_inc_int_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,0);

   sem_run_stack.__tail(1) = (*unsigned_aref_ptr)++;
   sem_run_stack.used--;
}

void parser_s::pi_sott_post_inc_int_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,0);

   sem_run_stack.__tail(2) = (*unsigned_aaref_ptr)++;
   sem_run_stack.used -= 2;
}

// - post_dec instrukce -
void parser_s::pi_sott_post_dec_int_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__last() = int_variables[sem_run_stack.__last()]--;
}

void parser_s::pi_sott_post_dec_int_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,0);

   sem_run_stack.__tail(1) = (*unsigned_aref_ptr)--;
   sem_run_stack.used--;
}

void parser_s::pi_sott_post_dec_int_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,0);

   sem_run_stack.__tail(2) = (*unsigned_aaref_ptr)--;
   sem_run_stack.used -= 2;
}


// - pre_inc instrukce -
void parser_s::pi_sott_pre_inc_int_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   int_variables[sem_run_stack.__last()]++;
}

void parser_s::pi_sott_pre_inc_int_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,0);

   (*unsigned_aref_ptr)++;
}

void parser_s::pi_sott_pre_inc_int_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,0);

   (*unsigned_aaref_ptr)++;
}

// - pre_dec instrukce -
void parser_s::pi_sott_pre_dec_int_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   int_variables[sem_run_stack.__last()]--;
}

void parser_s::pi_sott_pre_dec_int_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,0);

   (*unsigned_aref_ptr)--;
}

void parser_s::pi_sott_pre_dec_int_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,0);
   
   (*unsigned_aaref_ptr)--;
}

// - instrukce inverze -
void parser_s::pi_sott_invertion_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__last() = - sem_run_stack.__last();
}

// - instrukce negace -
void parser_s::pi_sott_negation_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.__last() = (!sem_run_stack.__last());
}

// - instrukce ziskani podretezce -
void parser_s::pi_sott_substr_string_var_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables.push_blank();
   
   unsigned length = sem_run_stack.__last();
   unsigned from = sem_run_stack.__tail(1);
   string_s &str_ref = string_variables[sem_run_stack.__tail(2)];
   sem_run_stack.used -= 3;

   assert(from + length < str_ref.size);

   string_variables.last().set(length,str_ref.data + from);
}

void parser_s::pi_sott_substr_string_tmp_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_s new_string;
   new_string.init();

   unsigned length = sem_run_stack.__last();
   unsigned from = sem_run_stack.__tail(1);
   string_s &str_ref = string_variables.last();
   sem_run_stack.used -= 2;

   assert(from + length < str_ref.size);

   new_string.set(length,str_ref.data + from);
   str_ref.swap(new_string);
   new_string.clear();
}

void parser_s::pi_sott_substr_string_const_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables.push_blank();
   
   unsigned length = sem_run_stack.__last();
   unsigned from = sem_run_stack.__tail(1);
   string_s &str_ref = string_constants[sem_run_stack.__tail(2)];
   sem_run_stack.used -= 3;

   assert(from + length < str_ref.size);

   string_variables.last().set(length,str_ref.data + from);
}

void parser_s::pi_sott_substr_string_aref_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,2);

   unsigned length = sem_run_stack.__last();
   unsigned from = sem_run_stack.__tail(1);
   sem_run_stack.used -= 4;

   assert(from + length < string_aref_ptr->size);

   string_variables.push_blank();
   string_variables.last().set(length,string_aref_ptr->data + from);
}

void parser_s::pi_sott_substr_string_aaref_int_value_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,2);

   unsigned length = sem_run_stack.__last();
   unsigned from = sem_run_stack.__tail(1);
   sem_run_stack.used -= 5;

   assert(from + length < string_aaref_ptr->size);

   string_variables.push_blank();
   string_variables.last().set(length,string_aaref_ptr->data + from);
}

// - instrukce formatovani retezce podle predanych parametru -
void parser_s::pi_sott_format_string_var_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables.push_blank();

   ui_array_s &unsigned_array = int_array_constants[sem_run_stack.__last()];
   string_s &string = string_variables[sem_run_stack.__tail(1)];
   sem_run_stack.used -= 2;

   string_variables.last().set_format_parm(string.size - 1,string.data,unsigned_array.used,unsigned_array.data);
}

void parser_s::pi_sott_format_string_tmp_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_s tmp_str;
   tmp_str.init();

   ui_array_s &unsigned_array = int_array_constants[sem_run_stack.__pop()];
   string_s &string = string_variables.last();

   tmp_str.set_format_parm(string.size - 1,string.data,unsigned_array.used,unsigned_array.data);
   string_variables.last().swap(tmp_str);
   
   tmp_str.clear();
}

void parser_s::pi_sott_format_string_const_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   string_array_s &string_variables = parse_data.string_variables;
   ui_array_s &unsigned_array = int_array_constants[sem_run_stack.__last()];
   string_s &string = string_constants[sem_run_stack.__tail(1)];
   sem_run_stack.used -= 2;

   string_variables.push_blank();
   string_variables.last().set_format_parm(string.size - 1,string.data,unsigned_array.used,unsigned_array.data);
}

void parser_s::pi_sott_format_string_aref_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,1);

   ui_array_s &unsigned_array = int_array_constants[sem_run_stack.__last()];
   sem_run_stack.used -= 3;

   string_variables.push_blank();
   string_variables.last().set_format_parm(string_aref_ptr->size - 1,string_aref_ptr->data,unsigned_array.used,unsigned_array.data);
}

void parser_s::pi_sott_format_string_aaref_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,1);

   ui_array_s &unsigned_array = int_array_constants[sem_run_stack.__last()];
   sem_run_stack.used -= 4;

   string_variables.push_blank();
   string_variables.last().set_format_parm(string_aaref_ptr->size - 1,string_aaref_ptr->data,unsigned_array.used,unsigned_array.data);
}


// - nacteni retezce elementu z tela pravidla -
void parser_s::pi_sott_get_rb_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_s &source_string = *((string_s *)parse_data.source_string_ptr);
   string_array_s &string_variables = parse_data.string_variables;
   p_rule_descrs_s &rule_descrs = ((parser_s *)parse_data.parser_ptr)->rule_descrs;
   p_lalr_stack_s &lalr_stack = parse_data.lalr_stack;

   unsigned rb_idx = sem_run_stack.__pop();
   assert(parse_data.action != c_idx_not_exist && rb_idx < rule_descrs[parse_data.action].ui_second);
   
   unsigned lalr_se_idx = lalr_stack.used - rule_descrs[parse_data.action].ui_second + rb_idx;
   p_lalr_stack_element_s &lalr_se = lalr_stack[lalr_se_idx];

   string_variables.push_blank();
   string_variables.last().set(lalr_se.ui_third - lalr_se.ui_second,source_string.data + lalr_se.ui_second);
}

// - ziskani cisla radku zdrojoveho souboru -
void parser_s::pi_sott_get_line_cnt(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   sem_run_stack.push(parse_data.line_cnt);
}

// - instrukce konverze na unsigned -
void parser_s::pi_sott_convert_int_string_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.__tail(1) = strtol(string_variables[sem_run_stack.__tail(1)].data,NULL,sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_convert_int_string_tmp_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.__last() = strtol(string_variables.pop().data,NULL,sem_run_stack.__last());
}

void parser_s::pi_sott_convert_int_string_const_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   sem_run_stack.__tail(1) = strtol(string_constants[sem_run_stack.__tail(1)].data,NULL,sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_convert_int_string_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,1);

   sem_run_stack.__tail(2) = strtol(string_aref_ptr->data,NULL,sem_run_stack.__last());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_convert_int_string_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,1);

   sem_run_stack.__tail(3) = strtol(string_aaref_ptr->data,NULL,sem_run_stack.__last());
   sem_run_stack.used -= 3;
}

// - instrukce konverze na string -
void parser_s::pi_sott_convert_string_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_variables.push_clear();
   string_s &str = string_variables.last();

   str.data = (char *)cmalloc(c_chars_needed_for_number_string*sizeof(char));
   str.size = sprintf(str.data,"%d",sem_run_stack.__pop()) + 1;
}

// - instrukce vlozeni indexu promennych na zasobnik -
void parser_s::pi_sott_int_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

void parser_s::pi_sott_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

void parser_s::pi_sott_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

void parser_s::pi_sott_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

void parser_s::pi_sott_int_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

void parser_s::pi_sott_string_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

      // - vlozeni konstant na zasobnik -
void parser_s::pi_sott_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

void parser_s::pi_sott_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

void parser_s::pi_sott_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

void parser_s::pi_sott_string_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

void parser_s::pi_sott_int_array_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

void parser_s::pi_sott_string_array_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   buffer_s &sem_code = *((buffer_s *)parse_data.sem_code_ptr);
   unsigned &sc_idx = parse_data.sc_idx;
   sem_run_stack.push(sem_code.get_unsigned(sc_idx));
}

// - vlozeni obsahu unsigned promenne na zasobnik vyrazu -
void parser_s::pi_sott_push_value_int_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__last() = int_variables[sem_run_stack.__last()];
}

void parser_s::pi_sott_push_value_int_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,0);
   
   sem_run_stack.__tail(1) = *unsigned_aref_ptr;
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_value_int_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,0);
   
   sem_run_stack.__tail(2) = *unsigned_aaref_ptr;
   sem_run_stack.used -= 2;
}

// - instrukce vlozeni hodnot dvou unsigned promennych na zasobnik -
void parser_s::pi_sott_push_two_values_int_var_int_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = int_variables[sem_run_stack.__tail(1)];
   sem_run_stack.__last() = int_variables[sem_run_stack.__last()];
}

void parser_s::pi_sott_push_two_values_int_aref_int_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = *unsigned_aref_ptr;
   sem_run_stack.__tail(1) = int_variables[sem_run_stack.__last()];
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_two_values_int_aaref_int_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(3) = *unsigned_aaref_ptr;
   sem_run_stack.__tail(2) = int_variables[sem_run_stack.__last()];
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_push_two_values_int_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   sem_run_stack.__tail(1) = int_variables[sem_run_stack.__tail(1)];
}

void parser_s::pi_sott_push_two_values_int_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,1);

   sem_run_stack.__tail(2) = *unsigned_aref_ptr;
   sem_run_stack.__tail(1) = sem_run_stack.__last();
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_two_values_int_aaref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,1);

   sem_run_stack.__tail(3) = *unsigned_aaref_ptr;
   sem_run_stack.__tail(2) = sem_run_stack.__last();
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_push_two_values_int_var_int_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,0);

   sem_run_stack.__tail(2) = int_variables[sem_run_stack.__tail(2)];
   sem_run_stack.__tail(1) = *unsigned_aref_ptr;
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_two_values_int_aref_int_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   unsigned *unsigned_aref_second_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_second_ptr,0);

   unsigned *unsigned_aref_first_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_first_ptr,2);

   sem_run_stack.__tail(3) = *unsigned_aref_first_ptr;
   sem_run_stack.__tail(2) = *unsigned_aref_second_ptr;
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_push_two_values_int_aaref_int_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,0);

   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,2);

   sem_run_stack.__tail(4) = *unsigned_aaref_ptr;
   sem_run_stack.__tail(3) = *unsigned_aref_ptr;
   sem_run_stack.used -= 3;
}

void parser_s::pi_sott_push_two_values_int_var_int_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_array_s &int_variables = parse_data.int_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,0);

   sem_run_stack.__tail(3) = int_variables[sem_run_stack.__tail(3)];
   sem_run_stack.__tail(2) = *unsigned_aaref_ptr;
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_push_two_values_int_aref_int_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_ptr,0);

   unsigned *unsigned_aref_ptr;
   GET_UNSIGNED_AREF(unsigned_aref_ptr,3);

   sem_run_stack.__tail(4) = *unsigned_aref_ptr;
   sem_run_stack.__tail(3) = *unsigned_aaref_ptr;
   sem_run_stack.used -= 3;
}

void parser_s::pi_sott_push_two_values_int_aaref_int_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   unsigned *unsigned_aaref_second_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_second_ptr,0);

   unsigned *unsigned_aaref_first_ptr;
   GET_UNSIGNED_AAREF(unsigned_aaref_first_ptr,3);

   sem_run_stack.__tail(5) = *unsigned_aaref_first_ptr;
   sem_run_stack.__tail(4) = *unsigned_aaref_second_ptr;
   sem_run_stack.used -= 4;
}

// - zjisteni velikosti retezce nebo pole -
void parser_s::pi_sott_size_of_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.__last() = string_variables[sem_run_stack.__last()].size - 1;
}

void parser_s::pi_sott_size_of_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   sem_run_stack.__last() = int_array_variables[sem_run_stack.__last()].used;
}

void parser_s::pi_sott_size_of_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__last() = string_array_variables[sem_run_stack.__last()].used;
}

void parser_s::pi_sott_size_of_int_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   sem_run_stack.__last() = int_array_array_variables[sem_run_stack.__last()].used;
}

void parser_s::pi_sott_size_of_string_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   sem_run_stack.__last() = string_array_array_variables[sem_run_stack.__last()].used;
}

void parser_s::pi_sott_size_of_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   sem_run_stack.push(string_variables.pop().size - 1);
}

void parser_s::pi_sott_size_of_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   sem_run_stack.__last() = string_constants[sem_run_stack.__last()].size - 1;
}

void parser_s::pi_sott_size_of_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);

   sem_run_stack.__tail(1) = string_aref_ptr->size - 1;
   sem_run_stack.used--;
}

void parser_s::pi_sott_size_of_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,0);

   sem_run_stack.__tail(1) = unsigned_array_aref_ptr->used;
   sem_run_stack.used--;
}

void parser_s::pi_sott_size_of_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);

   sem_run_stack.__tail(1) = string_array_aref_ptr->used;
   sem_run_stack.used--;
}

void parser_s::pi_sott_size_of_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);

   sem_run_stack.__tail(2) = string_aaref_ptr->size - 1;
   sem_run_stack.used -= 2;
}

// - vlozeni elementu na konec pole -
void parser_s::pi_sott_push_to_int_array_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   int_array_variables[sem_run_stack.__tail(1)].push(sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_to_string_array_var_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_variables[sem_run_stack.__tail(1)].push(string_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_to_string_array_var_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_s &string_array = string_array_variables[sem_run_stack.__last()];

   string_array.push_blank();
   string_array.last().swap(string_variables.pop());
}

void parser_s::pi_sott_push_to_string_array_var_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_variables[sem_run_stack.__tail(1)].push(string_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_to_string_array_var_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_s &string_array = string_array_variables[sem_run_stack.__tail(2)];
   string_array.push_blank();

   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);
  
   string_array.last() = *string_aref_ptr;
}

void parser_s::pi_sott_push_to_string_array_var_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   string_array_variables[sem_run_stack.__last()].push(*string_aaref_ptr);
}

void parser_s::pi_sott_push_to_int_array_array_var_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   int_array_array_variables[sem_run_stack.__tail(1)].push(int_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_to_int_array_array_var_int_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_arrays_s &unsigned_array_array = int_array_array_variables[sem_run_stack.__last()];
   unsigned_array_array.push_blank();
   unsigned_array_array.last().swap(int_array_variables.pop());
}

void parser_s::pi_sott_push_to_int_array_array_var_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   int_array_array_variables[sem_run_stack.__tail(1)].push(int_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_to_int_array_array_var_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_arrays_s &unsigned_array_array = int_array_array_variables[sem_run_stack.__tail(2)];
   unsigned_array_array.push_blank();

   ui_array_s *unsigned_array_aref_ptr;
   POP_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr);

   unsigned_array_array.last() = *unsigned_array_aref_ptr;
}

void parser_s::pi_sott_push_to_int_array_array_var_blank_array(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   int_array_array_variables[sem_run_stack.__last()].push_clear();
}

void parser_s::pi_sott_push_to_string_array_array_var_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_array_variables[sem_run_stack.__tail(1)].push(string_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_to_string_array_array_var_string_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_arrays_s &string_array_array = string_array_array_variables[sem_run_stack.__last()];
   string_array_array.push_blank();
   string_array_array.last().swap(string_array_variables.pop());
}

void parser_s::pi_sott_push_to_string_array_array_var_string_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_array_variables[sem_run_stack.__tail(1)].push(string_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_push_to_string_array_array_var_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_arrays_s &string_array_array = string_array_array_variables[sem_run_stack.__tail(2)];
   string_array_array.push_blank();

   string_array_s *string_array_aref_ptr;
   POP_STRING_ARRAY_AREF(string_array_aref_ptr);

   string_array_array.last() = *string_array_aref_ptr;
}

void parser_s::pi_sott_push_to_string_array_array_var_blank_array(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_array_variables[sem_run_stack.__last()].push_clear();
}

void parser_s::pi_sott_push_to_int_array_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,1);

   unsigned_array_aref_ptr->push(sem_run_stack.__pop());
}

void parser_s::pi_sott_push_to_string_array_aref_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);

   string_array_aref_ptr->push(string_variables[sem_run_stack.__pop()]);
}

void parser_s::pi_sott_push_to_string_array_aref_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);
  
   string_array_aref_ptr->push_blank();
   string_array_aref_ptr->last().swap(string_variables.pop());
}

void parser_s::pi_sott_push_to_string_array_aref_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);

   string_array_aref_ptr->push(string_constants[sem_run_stack.__pop()]);
}

void parser_s::pi_sott_push_to_string_array_aref_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);

   string_array_aref_ptr->push(*string_aref_ptr);
}

void parser_s::pi_sott_push_to_string_array_aref_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);

   string_array_aref_ptr->push(*string_aaref_ptr);
}

// - vybrani elementu z konce pole -
void parser_s::pi_sott_pop_from_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_array_s &unsigned_array = int_array_variables[sem_run_stack.__last()];
   assert(unsigned_array.used != 0);

   sem_run_stack.__last() = unsigned_array.pop();
}

void parser_s::pi_sott_pop_from_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_s &string_array = string_array_variables[sem_run_stack.__pop()];
   assert(string_array.used != 0);
   
   string_variables.push_blank();
   string_variables.last().swap(string_array.pop());
}

void parser_s::pi_sott_pop_from_int_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_arrays_s &unsigned_array_array = int_array_array_variables[sem_run_stack.__pop()];
   assert(unsigned_array_array.used != 0);

   int_array_variables.push_blank();
   int_array_variables.last().swap(unsigned_array_array.pop());
}

void parser_s::pi_sott_pop_from_string_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_arrays_s &string_array_array = string_array_array_variables[sem_run_stack.__pop()];
   assert(string_array_array.used != 0);

   string_array_variables.push_blank();
   string_array_variables.last().swap(string_array_array.pop());
}

void parser_s::pi_sott_pop_from_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,0);
   assert(unsigned_array_aref_ptr->used != 0);

   sem_run_stack.__tail(1) = unsigned_array_aref_ptr->pop();
   sem_run_stack.used--;
}

void parser_s::pi_sott_pop_from_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);
   assert(string_array_aref_ptr->used != 0);

   string_variables.push_blank();
   string_variables.last().swap(string_array_aref_ptr->pop());
   sem_run_stack.used--;
}

// - instrukce odtraneni x poslednich prvku z pole -
void parser_s::pi_sott_remove_from_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_array_s &unsigned_array = int_array_variables[sem_run_stack.__tail(1)];
   unsigned r_cnt = sem_run_stack.__pop();
   
   assert(unsigned_array.used >= r_cnt);
   unsigned_array.used -= r_cnt;
}

void parser_s::pi_sott_remove_from_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_s &string_array = string_array_variables[sem_run_stack.__tail(1)];
   unsigned r_cnt = sem_run_stack.__pop();

   assert(string_array.used >= r_cnt);
   string_array.used -= r_cnt;
}

void parser_s::pi_sott_remove_from_int_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_arrays_s &unsigned_array_array = int_array_array_variables[sem_run_stack.__tail(1)];
   unsigned r_cnt = sem_run_stack.__pop();

   assert(unsigned_array_array.used >= r_cnt);
   unsigned_array_array.used -= r_cnt;
}

void parser_s::pi_sott_remove_from_string_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_arrays_s &string_array_array = string_array_array_variables[sem_run_stack.__tail(1)];
   unsigned r_cnt = sem_run_stack.__pop();

   assert(string_array_array.used >= r_cnt);
   string_array_array.used -= r_cnt;
}

void parser_s::pi_sott_remove_from_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,1);
   unsigned r_cnt = sem_run_stack.__pop();

   assert(unsigned_array_aref_ptr->used >= r_cnt);
   unsigned_array_aref_ptr->used -= r_cnt;
}

void parser_s::pi_sott_remove_from_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);
   unsigned r_cnt = sem_run_stack.__pop();

   assert(string_array_aref_ptr->used >= r_cnt);
   string_array_aref_ptr->used -= r_cnt;
}

// - instrukce vytvarejici referenci na posledni prvek v poli -
void parser_s::pi_sott_last_of_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_array_s &unsigned_array = int_array_variables[sem_run_stack.__last()];
   assert(unsigned_array.used != 0);

   sem_run_stack.push(unsigned_array.used - 1);
}

void parser_s::pi_sott_last_of_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_s &string_array = string_array_variables[sem_run_stack.__last()];
   assert(string_array.used != 0);

   sem_run_stack.push(string_array.used - 1);
}

void parser_s::pi_sott_last_of_int_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_arrays_s &unsigned_array_array = int_array_array_variables[sem_run_stack.__last()];
   assert(unsigned_array_array.used != 0);

   sem_run_stack.push(unsigned_array_array.used - 1);
}

void parser_s::pi_sott_last_of_string_array_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_arrays_s &string_array_array = string_array_array_variables[sem_run_stack.__last()];
   assert(string_array_array.used != 0);

   sem_run_stack.push(string_array_array.used - 1);
}

void parser_s::pi_sott_last_of_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,0);
   assert(unsigned_array_aref_ptr->used != 0);

   sem_run_stack.push(unsigned_array_aref_ptr->used - 1);
}

void parser_s::pi_sott_last_of_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);
   assert(string_array_aref_ptr->used != 0);

   sem_run_stack.push(string_array_aref_ptr->used - 1);
}

// - instrukce ziskani prvku indexovaneho od konce pole -
void parser_s::pi_sott_tail_of_int_array_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_array_s &unsigned_array = int_array_variables[sem_run_stack.__tail(1)];
   unsigned tail_idx = sem_run_stack.__last();

   assert(unsigned_array.used > tail_idx);
   sem_run_stack.__last() = unsigned_array.used - 1 - tail_idx;
}

void parser_s::pi_sott_tail_of_string_array_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_array_s &string_array = string_array_variables[sem_run_stack.__tail(1)];
   unsigned tail_idx = sem_run_stack.__last();

   assert(string_array.used > tail_idx);
   sem_run_stack.__last() = string_array.used - 1 - tail_idx;
}

void parser_s::pi_sott_tail_of_int_array_array_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_arrays_s &unsigned_array_array = int_array_array_variables[sem_run_stack.__tail(1)];
   unsigned tail_idx = sem_run_stack.__last();

   assert(unsigned_array_array.used > tail_idx);
   sem_run_stack.__last() = unsigned_array_array.used - 1 - tail_idx;
}

void parser_s::pi_sott_tail_of_string_array_array_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_arrays_s &string_array_array = string_array_array_variables[sem_run_stack.__tail(1)];
   unsigned tail_idx = sem_run_stack.__last();

   assert(string_array_array.used > tail_idx);
   sem_run_stack.__last() = string_array_array.used - 1 - tail_idx;
}

void parser_s::pi_sott_tail_of_int_array_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,1);
   unsigned tail_idx = sem_run_stack.__last();

   assert(unsigned_array_aref_ptr->used > tail_idx);
   sem_run_stack.__last() = unsigned_array_aref_ptr->used - 1 - tail_idx;
}

void parser_s::pi_sott_tail_of_string_array_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);
   unsigned tail_idx = sem_run_stack.__last();

   assert(string_array_aref_ptr->used > tail_idx);
   sem_run_stack.__last() = string_array_aref_ptr-> used - 1 - tail_idx;
}

// - instrukce ziskani indexu objektu v poli -
void parser_s::pi_sott_get_idx_int_array_var_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   sem_run_stack.__tail(1) = int_array_variables[sem_run_stack.__tail(1)].get_idx(sem_run_stack.__last());
   sem_run_stack.used--;
}

void parser_s::pi_sott_get_idx_string_array_var_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__tail(1) = string_array_variables[sem_run_stack.__tail(1)].get_idx(string_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_get_idx_string_array_var_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__last() = string_array_variables[sem_run_stack.__last()].get_idx(string_variables.pop());
}

void parser_s::pi_sott_get_idx_string_array_var_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   sem_run_stack.__tail(1) = string_array_variables[sem_run_stack.__tail(1)].get_idx(string_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_get_idx_string_array_var_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_s *string_aref_ptr;
   POP_STRING_AREF(string_aref_ptr);

   sem_run_stack.__last() = string_array_variables[sem_run_stack.__last()].get_idx(*string_aref_ptr);
}

void parser_s::pi_sott_get_idx_string_array_var_string_aaref(p_parse_data_s &parse_data)
{  
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   POP_STRING_AAREF(string_aaref_ptr);

   sem_run_stack.__last() = string_array_variables[sem_run_stack.__last()].get_idx(*string_aaref_ptr);
}

void parser_s::pi_sott_get_idx_int_array_array_var_int_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   sem_run_stack.__tail(1) = int_array_array_variables[sem_run_stack.__tail(1)].get_idx(int_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_get_idx_int_array_array_var_int_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_variables = parse_data.int_array_variables;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   sem_run_stack.__last() = int_array_array_variables[sem_run_stack.__last()].get_idx(int_array_variables.pop());
}

void parser_s::pi_sott_get_idx_int_array_array_var_int_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_s &int_array_constants = ((parser_s *)parse_data.parser_ptr)->int_array_constants;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   sem_run_stack.__tail(1) = int_array_array_variables[sem_run_stack.__tail(1)].get_idx(int_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_get_idx_int_array_array_var_int_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   POP_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr);

   sem_run_stack.__last() = int_array_array_variables[sem_run_stack.__last()].get_idx(*unsigned_array_aref_ptr);
}

void parser_s::pi_sott_get_idx_int_array_array_var_blank_array(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s unsigned_array;
   unsigned_array.init();

   sem_run_stack.__last() = int_array_array_variables[sem_run_stack.__last()].get_idx(unsigned_array);
   unsigned_array.clear();
}

void parser_s::pi_sott_get_idx_string_array_array_var_string_array_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   sem_run_stack.__tail(1) = string_array_array_variables[sem_run_stack.__tail(1)].get_idx(string_array_variables[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_get_idx_string_array_array_var_string_array_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   sem_run_stack.__last() = string_array_array_variables[sem_run_stack.__last()].get_idx(string_array_variables.pop());
}

void parser_s::pi_sott_get_idx_string_array_array_var_string_array_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_constants = ((parser_s *)parse_data.parser_ptr)->string_array_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   sem_run_stack.__tail(1) = string_array_array_variables[sem_run_stack.__tail(1)].get_idx(string_array_constants[sem_run_stack.__last()]);
   sem_run_stack.used--;
}

void parser_s::pi_sott_get_idx_string_array_array_var_string_array_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   POP_STRING_ARRAY_AREF(string_array_aref_ptr);

   sem_run_stack.__last() = string_array_array_variables[sem_run_stack.__last()].get_idx(*string_array_aref_ptr);
}

void parser_s::pi_sott_get_idx_string_array_array_var_blank_array(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s string_array;
   string_array.init();

   sem_run_stack.__last() = string_array_array_variables[sem_run_stack.__last()].get_idx(string_array);
   string_array.clear();
}

void parser_s::pi_sott_get_idx_int_array_aref_int_value(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   ui_arrays_array_s &int_array_array_variables = parse_data.int_array_array_variables;
   ui_array_s *unsigned_array_aref_ptr;
   GET_UNSIGNED_ARRAY_AREF(unsigned_array_aref_ptr,1);

   sem_run_stack.__tail(2) = unsigned_array_aref_ptr->get_idx(sem_run_stack.__last());
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_get_idx_string_array_aref_string_var(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);

   sem_run_stack.__tail(2) = string_array_aref_ptr->get_idx(string_variables[sem_run_stack.__last()]);
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_get_idx_string_array_aref_string_tmp(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_variables = parse_data.string_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,0);

   sem_run_stack.__tail(1) = string_array_aref_ptr->get_idx(string_variables.pop());
   sem_run_stack.used--;
}

void parser_s::pi_sott_get_idx_string_array_aref_string_const(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_array_s &string_constants = ((parser_s *)parse_data.parser_ptr)->string_constants;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,1);

   sem_run_stack.__tail(2) = string_array_aref_ptr->get_idx(string_constants[sem_run_stack.__last()]);
   sem_run_stack.used -= 2;
}

void parser_s::pi_sott_get_idx_string_array_aref_string_aref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_s &string_array_variables = parse_data.string_array_variables;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aref_ptr;
   GET_STRING_AREF(string_aref_ptr,0);

   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,2);

   sem_run_stack.__tail(3) = string_array_aref_ptr->get_idx(*string_aref_ptr);
   sem_run_stack.used -= 3;
}

void parser_s::pi_sott_get_idx_string_array_aref_string_aaref(p_parse_data_s &parse_data)
{
   ui_array_s &sem_run_stack = parse_data.sem_run_stack;
   string_arrays_array_s &string_array_array_variables = parse_data.string_array_array_variables;
   string_s *string_aaref_ptr;
   GET_STRING_AAREF(string_aaref_ptr,0);

   string_array_s *string_array_aref_ptr;
   GET_STRING_ARRAY_AREF(string_array_aref_ptr,3);
   
   sem_run_stack.__tail(4) = string_array_aref_ptr->get_idx(*string_aaref_ptr);
   sem_run_stack.used -= 4;
}

