
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

// - datovy typ instrukce v semantickem bytovem kodu -

typedef unsigned short t_si_type;

// --- struct string_s definition ---

/*
 * struktura implementujici retezec, zapsana takovym zpusobem,
 * aby se mohla pouzit v definicich objektu skriptu process.pl
 */
struct string_s
{
   unsigned size;
   char *data;

   inline void init();
   inline void clear();
   inline void set(unsigned a_length,char *a_data);
   void set_format(unsigned a_length,char *a_data);
   void set_format_parm(unsigned a_length,char *a_data,unsigned a_parm_cnt,unsigned *params);
   inline void conc_set(unsigned a_flength,char *a_fdata,unsigned a_slength,char *a_sdata);
   inline void flush_all();
   inline void swap(string_s &second);
   inline char &operator[](unsigned a_idx);
   inline string_s &operator=(string_s &src);
   inline bool operator==(string_s &second);
   inline bool compare_char_ptr(unsigned a_length,char *a_data);
   inline unsigned save_size();
   inline void save_to(char **b_ptr);
   inline void load_from(char **b_ptr);
   inline void print();
   inline bool load_text_file(const char *file);
   inline bool save_text_file(const char *file);
};

inline void string_s::init()
{
   size = 0;
   data = NULL;
}

inline void string_s::clear()
{
   if (data != NULL) {
      cfree(data);
   }

   init();
}

inline void string_s::set(unsigned a_length,char *a_data)
{
   clear();
   data = (char *)cmalloc((a_length + 1)*sizeof(char));

   if (a_length != 0) {
      memcpy(data,a_data,a_length*sizeof(char));
   }

   data[a_length] = '\0';
   size = a_length + 1;
}

void string_s::set_format(unsigned a_length,char *a_data)
{
   clear();
   data = (char *)cmalloc((a_length + 1)*sizeof(char));

   if (a_length != 0) {
      
      // - nastaveni retezce s prepisem escape sekvenci -
      char *a_ptr = a_data;
      char *a_ptr_end = a_ptr + a_length;
      char *ptr = data; 

      do {
	 if (*a_ptr != '\\') {
	    *ptr = *a_ptr;
	 }
	 else {
	    switch(*(++a_ptr)) {
	    case '\"':
	       *ptr = '\"';
	       break;
	    case 't':
	       *ptr = '\t';
	       break;
	    case 'n':
	       *ptr = '\n';
	       break;
	    case '0':
	       fprintf(stderr,"TODO ukoncovaci znak v retezci.\n");
	       assert(0);
	       break;
	    case '}':
	       *ptr = '}';
	       break;
	    case '\\':
	       *ptr = '\\';
	       break;
	    default:
	       assert(0);
	    }
	 }
      } while(++ptr,++a_ptr < a_ptr_end);
      
      size = (ptr - data) + 1;
   }
   else {
      size = 1;
   }

   data[size - 1] = '\0';
}

void string_s::set_format_parm(unsigned a_length,char *a_data,unsigned a_parm_cnt,unsigned *params)
{
   clear();
   data = (char *)cmalloc((a_length + 1)*sizeof(char));

   if (a_length != 0) {
      assert(a_parm_cnt >= 3 && (a_parm_cnt % 2 == 1));
      char escape = (char)params[0];

      char *a_ptr = a_data;
      char *a_ptr_end = a_ptr + a_length;
      char *ptr = data;

      do {
	 if (*a_ptr != escape) {
	    *ptr = *a_ptr;
	 }
	 else {
	    a_ptr++;
	    
	    unsigned *p_ptr = params + 1;
	    unsigned *p_ptr_end = params + a_parm_cnt;

	    do {
	       if (*a_ptr == (char)*p_ptr) {
		  *ptr = *(p_ptr + 1);
		  break;
	       }
	    } while((p_ptr += 2) < p_ptr_end);
	    
	    // - ujisteni ze je escapovany znak znam -
	    assert(p_ptr < p_ptr_end);
	 }
      } while(++ptr,++a_ptr < a_ptr_end);

      size = (ptr - data) + 1;
   }
   else {
      size = 1;
   }

   data[size - 1] = '\0';
}

inline void string_s::conc_set(unsigned a_flength,char *a_fdata,unsigned a_slength,char *a_sdata)
{
   clear();
   data = (char *)cmalloc((a_flength + a_slength + 1)*sizeof(char));

   if (a_flength != 0) {
      memcpy(data,a_fdata,a_flength*sizeof(char));
   }

   if (a_slength != 0) {
      memcpy(data + a_flength,a_sdata,a_slength*sizeof(char));
   }

   data[a_flength + a_slength] = '\0';
   size = a_flength + a_slength + 1;
}

inline void string_s::flush_all()
{
}

inline void string_s::swap(string_s &second)
{
   unsigned tmp_size = size;
   size = second.size;
   second.size = tmp_size;

   char *tmp_data = data;
   data = second.data;
   second.data = tmp_data;
}

inline char &string_s::operator[](unsigned a_idx)
{
   debug_assert(a_idx < size);
   return data[a_idx];
}

inline string_s &string_s::operator=(string_s &src)
{
   clear();
   if (src.size == 0) return *this;
   data = (char *)cmalloc(src.size*sizeof(char));
   memcpy(data,src.data,src.size*sizeof(char));
   size = src.size;

   return *this;
}

inline bool string_s::operator==(string_s &second)
{
   if (size != second.size) return false;
   if (size == 0) return true;
   return (memcmp(data,second.data,size*sizeof(char)) == 0);
}

inline bool string_s::compare_char_ptr(unsigned a_length,char *a_data)
{
   if (size != a_length + 1) return false;
   if (a_length == 0) return true;
   return (memcmp(data,a_data,a_length*sizeof(char)) == 0);
}

inline unsigned string_s::save_size()
{
   return sizeof(unsigned) + size*sizeof(char);
}

inline void string_s::save_to(char **b_ptr)
{
   *(unsigned *)*b_ptr = size; *b_ptr += sizeof(unsigned);

   if (size == 0) return;
   memcpy(*b_ptr,data,size*sizeof(char));
   *b_ptr += size*sizeof(char);
}

inline void string_s::load_from(char **b_ptr)
{
   clear();
   size = *(unsigned *)*b_ptr; *b_ptr += sizeof(unsigned);

   unsigned data_size = size*sizeof(char);
   data = (char *)cmalloc(data_size);
   memcpy(data,*b_ptr,data_size);
   *b_ptr += data_size;
}

inline void string_s::print()
{
   if (size == 0) return;
   printf("%s",data);
}

inline bool string_s::load_text_file(const char *file)
{
   if (file == NULL) return false;
   FILE *f = fopen(file,"rb");
   if (f == NULL) return false;

   fseek(f,0,SEEK_END);
   unsigned file_size = ftell(f);
   fseek(f,0,SEEK_SET);

   clear();
   data = (char *)cmalloc((file_size + 1)*sizeof(char));
   fread(data,file_size,1,f);
   fclose(f);

   data[file_size] = '\0';
   size = file_size + 1;

   return true;
}

inline bool string_s::save_text_file(const char *file)
{
   if (file == NULL) return false;
   FILE *f = fopen(file,"wb");
   if (f == NULL) return false;

   if (size > 1) {
      fwrite(data,size - 1,1,f);
   }

   fclose(f);
   return true;
}

// --- struct buffer_s definition ---

/*
 * struktura popisujici buffer znaku, urcena pro reprezentaci kodu,
 * textovych souboru apod.
 */

struct buffer_s
{
   unsigned size;
   unsigned used;
   char *data;

   inline void init();
   inline void clear();
   inline void set(unsigned a_length,char *a_data);
   inline void flush_all();
   inline void swap(buffer_s &second);
   void copy_resize(unsigned a_size);
   inline buffer_s &operator=(buffer_s &src);
   inline bool operator==(buffer_s &second);
   inline unsigned save_size();
   inline void save_to(char **b_ptr);
   inline void load_from(char **b_ptr);
   void print();

   // - metody pro praci s ruznymi daty -
   
   inline void push_sem_instruction(t_si_type a_value);
   inline t_si_type get_sem_instruction(unsigned &a_idx);

   void push_string(unsigned a_length,const char *a_data);
   void push_format_string(unsigned a_length,const char *a_data);

   inline void push_char(char a_value);
   inline char get_char(unsigned &a_idx);
   inline void push_short(short a_value);
   inline short get_short(unsigned &a_idx);
   inline void push_int(int a_value);
   inline int get_int(unsigned &a_idx);
   inline void push_unsigned(unsigned a_value);
   inline unsigned get_unsigned(unsigned &a_idx);
   inline void push_data(unsigned a_length,const char *a_data);
   inline void push_terminated_string(const char *t_str);

};

inline void buffer_s::init()
{
   size = 0;
   used = 0;
   data = NULL;
}

inline void buffer_s::clear()
{
   if (data != NULL) {
      cfree(data);
   }

   init();
}

inline void buffer_s::set(unsigned a_length,char *a_data)
{
   clear();

   if (a_length == 0) return;

   data = (char *)cmalloc(a_length*sizeof(char));
   memcpy(data,a_data,a_length*sizeof(char));

   size = a_length;
   used = a_length;
}

inline void buffer_s::flush_all()
{
   copy_resize(used);
}


inline void buffer_s::swap(buffer_s &second)
{
   unsigned tmp_size = size;
   size = second.size;
   second.size = tmp_size;

   unsigned tmp_used = used;
   used = second.used;
   second.used = tmp_used;

   char *tmp_data = data;
   data = second.data;
   second.data = tmp_data;
}

void buffer_s::copy_resize(unsigned a_size)
{
   debug_assert(a_size >= used);

   char *n_data;

   if (a_size == 0) {
      n_data = NULL;
   }
   else {
      n_data = (char *)cmalloc(a_size*sizeof(char));
   }
   
   if (used != 0) {
      memcpy(n_data,data,used*sizeof(char));
   }

   if (size != 0) {
      cfree(data);
   }

   data = n_data;
   size = a_size;
}

inline buffer_s &buffer_s::operator=(buffer_s &src)
{
   clear();

   if (src.size == 0) return *this;

   data = (char *)cmalloc(src.used*sizeof(char));
   memcpy(data,src.data,src.used*sizeof(char));

   size = src.used;
   used = src.used;

   return *this;
}

inline bool buffer_s::operator==(buffer_s &second)
{
   if (used != second.used) return false;
   if (used == 0) return true;
   return (memcmp(data,second.data,used*sizeof(char)) == 0);
}

inline unsigned buffer_s::save_size()
{
   return sizeof(unsigned) + used*sizeof(char);
}

inline void buffer_s::save_to(char **b_ptr)
{
   *(unsigned *)*b_ptr = used; *b_ptr += sizeof(unsigned);
   if (used == 0) return;
   memcpy(*b_ptr,data,used*sizeof(char));
   *b_ptr += used*sizeof(char);
}

inline void buffer_s::load_from(char **b_ptr)
{
   clear();

   size = *(unsigned *)*b_ptr; *b_ptr += sizeof(unsigned);
   data = (char *)cmalloc(size*sizeof(char));
   memcpy(data,*b_ptr,size*sizeof(char));

   *b_ptr += size*sizeof(char);
   used = size;
}

void buffer_s::print()
{
   if (used != 0) {
      char *ptr = data;
      char *ptr_end = ptr + used;

      do {
	 if (*ptr >= 32 && *ptr <= 126) {
	    putchar(*ptr);
	 }
	 else {
	    printf("<%d>",(unsigned char)*ptr);
	 }
      } while(++ptr < ptr_end);
   }
}

inline void buffer_s::push_sem_instruction(t_si_type a_value)
{
   if (used + sizeof(t_si_type) > size) {
      copy_resize((size << 1) + c_array_add*sizeof(t_si_type));
   }
   
   *(t_si_type *)(data + used) = a_value;
   used += sizeof(t_si_type);
}

inline t_si_type buffer_s::get_sem_instruction(unsigned &a_idx)
{
   assert(a_idx + sizeof(t_si_type) <= used);

   register t_si_type *r_ret_ptr = (t_si_type *)(data + a_idx);
   a_idx += sizeof(t_si_type);

   return *r_ret_ptr;
}

void buffer_s::push_string(unsigned a_length,const char *a_data)
{
   unsigned a_size = sizeof(unsigned) + a_length + 1;
   if (used + a_size > size) {
      copy_resize((size << 1) + c_array_add + a_size);
   }

   *(unsigned *)(data + used) = a_length;
   used += sizeof(unsigned);

   if (a_length != 0) {
      memcpy(data + used,a_data,a_length);
      used += a_length;
   }

   data[used++] = '\0';
}

void buffer_s::push_format_string(unsigned a_length,const char *a_data)
{
   unsigned a_size = sizeof(unsigned) + a_length + 1;
   if (used + a_size > size) {
      copy_resize((size << 1) + c_array_add + a_size);
   }

   if (a_length != 0) {
      
      // - kopirovani retezce s prepisem escape sekvenci -
      char *ptr_start = data + used + sizeof(unsigned);

      const char *a_ptr = a_data;
      const char *a_ptr_end = a_ptr + a_length;
      char *ptr = ptr_start; 

      do {
	 if (*a_ptr != '\\') {
	    *ptr = *a_ptr;
	 }
	 else {
	    switch(*(++a_ptr)) {
	    case '\"':
	       *ptr = '\"';
	       break;
	    case 't':
	       *ptr = '\t';
	       break;
	    case 'n':
	       *ptr = '\n';
	       break;
	    case '0':
	       fprintf(stderr,"TODO ukoncovaci znak v retezci.\n");
	       assert(0);
	       break;
	    case '}':
	       *ptr = '}';
	       break;
	    case '\\':
	       *ptr = '\\';
	       break;
	    default:
	       assert(0);
	    }
	 }
      } while(++ptr,++a_ptr < a_ptr_end);
      
      unsigned str_len = ptr - ptr_start;

      *(unsigned *)(data + used) = str_len;
      used += sizeof(unsigned) + str_len;
   }
   else {

      // - pokud je retezec prazdny -
      *(unsigned *)(data + used) = 0;
      used += sizeof(unsigned);
   }

   data[used++] = '\0';
}

inline void buffer_s::push_char(char a_value)
{
   if (used >= size) {
      copy_resize((size << 1) + c_array_add);
   }

   data[used++] = a_value;
}

inline char buffer_s::get_char(unsigned &a_idx)
{
   assert(a_idx < used);
   return data[a_idx++];
}

inline void buffer_s::push_short(short a_value)
{
   if (used + sizeof(short) > size) {
      copy_resize((size << 1) + c_array_add*sizeof(short));
   }

   *(short *)(data + used) = a_value;
   used += sizeof(short);
}

inline short buffer_s::get_short(unsigned &a_idx)
{
   assert(a_idx + sizeof(short) <= used);

   register short *r_ret_ptr = (short *)(data + a_idx);
   a_idx += sizeof(short);

   return *r_ret_ptr;
}

inline void buffer_s::push_int(int a_value)
{
   if (used + sizeof(int) > size) {
      copy_resize((size << 1) + c_array_add*sizeof(int));
   }

   *(int *)(data + used) = a_value;
   used += sizeof(int);
}

inline int buffer_s::get_int(unsigned &a_idx)
{
   assert(a_idx + sizeof(int) <= used);

   register int *r_ret_ptr = (int *)(data + a_idx);
   a_idx += sizeof(int);

   return *r_ret_ptr;
}

inline void buffer_s::push_unsigned(unsigned a_value)
{
   if (used + sizeof(unsigned) > size) {
      copy_resize((size << 1) + c_array_add*sizeof(unsigned));
   }

   *(unsigned *)(data + used) = a_value;
   used += sizeof(unsigned);
}

inline unsigned buffer_s::get_unsigned(unsigned &a_idx)
{
   assert(a_idx + sizeof(unsigned) <= used);

   register unsigned *r_ret_ptr = (unsigned *)(data + a_idx);
   a_idx += sizeof(unsigned);

   return *r_ret_ptr;
}

inline void buffer_s::push_data(unsigned a_length,const char *a_data)
{
   if (used + a_length > size) {
      copy_resize((size << 1) + c_array_add + a_length);
   }

   if (a_length != 0) {
      memcpy(data + used,a_data,a_length);
      used += a_length;
   }
}

inline void buffer_s::push_terminated_string(const char *t_str)
{
   push_data(strlen(t_str),t_str);
}

